/**	\file iso_base_exception.h
 *  \class iso::base::Exception class for handling all kinds of run time errors
 *	\brief class for handling all kinds of run time errors
 *
 *  defines the following exceptions:
 *	ERR_OUT_OF_BOUNDS : Out Of Bounds
 *	ERR_FILE_OPEN_FAILED : Failed To Open File
 *	ERR_FILE_CLOSE_FAILED : Failed To Close File
 *	ERR_FILE_WRITE_FAILED : Failed To Write to File
 *	ERR_UNKNOWN_FILE_FORMAT : Unknown File Format
 *  ERR_CORRUPT_FILE_FORMAT : Corrupt File Format 
 *	ERR_NOT_A_NUMBER : Not a Number
 *	ERR_NAME_CONFLICT : Name Conflict
 *  ERR_NAME_UNKNOWN : Name Unkown
 *  ERR_FUNCTION_UNKNOWN : Function Unkown
 *  ERR_FUNCTION_MISMATCH : Function Mismatch 
 *	ERR_TYPE_MISMATCH : Type Mismatch
 *	ERROR : Generic Error
 *	blank : Unknown Exception
 *
 *  Created by Daniel Bisig on 5/17/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_base_exception_h_
#define _iso_base_exception_h_

#include "iso_base_types.h"
#include <QHash>
#include <iostream>

namespace iso
{

namespace base
{

class Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	Exception(String pType, String pMessage, String pSourceFile, String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	Exception(String pMessage, String pSourceFile, String pFunction, int pLineNumber);
	
	/**
	\brief copy constructor
	\param pException exception
	*/
	Exception( const Exception& pException );
	
	/**
	\brief destructor
	*/
	virtual ~Exception();
	
	/**
	\brief return exception message
	\return exception message
	*/
	const String& message() const;
	
	/**
	\brief return exception type
	\return exception type
	*/
	const String& type() const;
	
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const Exception& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
	
	/**
	\brief obtain textual exception information
	\return String containing exception information
	*/
	String info() const; 
		
protected:
	/**
	\brief exception types
	
	association from type to type description
	*/
	static QHash<String, String> sExceptionTypes;
	
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();

	/**
	\brief default constructur
	*/
	Exception();
	
	/**
	\brief exception type
	*/
	String mType;
	
	/**
	\brief additional textual exception information
	*/
	String mMessage;
	
	/**
	\brief source file in which exception occurred
	*/
	String mSourceFile;
	
	/**
	\brief function in which exception occurred
	*/
	String mFunction;
	
	/**
	\brief line number within source file at which exception occurred
	*/
	int mLineNumber;
};

};

};


#endif


