/** \file iso_com_default_message_parser.h
 *
 *  Created by Daniel Bisig on 5/16/09.
 *
 */

#ifndef _iso_com_default_message_parser_h_
#define _iso_com_default_message_parser_h_

#include "iso_com_types.h"
#include "iso_com_exception.h"
#include "iso_com_message_parser.h"
#include <iso_data/iso_data_singleton.h>

namespace iso 
{

namespace com
{

class DefaultMessageParser : public MessageParser
{
public:
	/**
	\brief default contructor
	*/
	DefaultMessageParser();
	
	/**
	\brief destructor
	*/
	~DefaultMessageParser();

	/**
	\brief calculate message buffer size
	\param pMessage message
	\return message buffer size (number of bytes)
	\exception ComException message doesn't conform to expected message format
	*/
	unsigned int messageBufferSize( const Message& pMessage ) const throw (ComException);

	/**
	\brief check if message conforms to expected message format
	\param pMessage message
	\return true if message format is correct, false otherwise
	*/
	bool checkMessage( const Message& pMessage ) const;

	/**
	\brief convert message into message buffer
	\param pMessage message
	\param pMessageBuffer message buffer
	\exception ComException message doesn't conform to expected message format
	\remarks message buffer array must have been correctly allocated beforehand
	*/
	void message2buffer( const Message& pMessage, ByteBuffer& pMessageBuffer ) const throw (ComException);

	/**
	\brief convert message buffer into message
	\param pMessageBuffer message buffer
	\param pMessage message
	\exception ComException message doesn't conform to expected message format
	*/
	void buffer2message( const ByteBuffer& pMessageBuffer, Message& pMessage ) const throw (ComException);

	/**
	\brief convert message buffer into data packets
	\param pMessageBufferPos position of packet within message buffer
	\param pPacketBuffer message buffer
	\param pPackets vector of data packets
	\param pMaxPacketSize maximum packet size (in bytes)
	*/
	void buffer2packets( const ByteBuffer& pMessageBuffer, QVector<ByteBuffer*>& pPackets, unsigned int pMaxPacketSize ) const throw (ComException);

	/**
	\brief check if packet buffer contains data for message buffer
	\return true if yes, false otherwise
	*/
	bool match( ByteBuffer& pPacketBuffer, PendingByteBuffer& pMessageBuffer ) const throw (ComException);

	/**
	\brief convert data packet into message buffer
	\param pPacketBuffer data packet
	\param pMessageBuffer message buffer
	*/	
	virtual void packet2buffer( const ByteBuffer& pPacketBuffer, PendingByteBuffer& pMessageBuffer ) const throw (ComException);

	/**
	\brief retrieve textual information about message
	\param pMessage message
	\return string containing textual message information
	\exception ComException message doesn't conform to expected message format
	*/
	base::String messageInfo( const Message& pMessage ) const throw (ComException);	

	/**
	\brief retrieve textual information about data packet
	\param pPacketBuffer data packet buffer
	*/	
	base::String packetInfo( const ByteBuffer& pPacketBuffer ) const throw (ComException);

protected:
	static int sMessageHeaderSize; ///\brief size of message header information
	static int sPacketHeaderSize; ///\brief size of data packet header information

	QHash< const std::type_info*, char > mValueTypeMap;
};

};

};

#endif