/** \file iso_com_exception.h
 *
 *	\brief class for handling all kinds of run time errors
 *
 *	Defines a variety of run time errors, which are specific to the flock environment:
 *  ERR_DUPLICATE_SENDER : Sender already Exists
 *  ERR_UNKNOWN_SENDER : Sender Does Not Exist
 *  ERR_DUPLICATE_RECEIVER : Receiver already Exists
 *  ERR_UNKNOWN_RECEIVER : Receiver Does Not Exist
 *  ERR_SOCKET_CREATE_FAILED : Failed To Create Socket
 *  ERR_SOCKET_CLOSE_FAILED : Failed To Close Socket 
 *  ERR_SOCKET_BIND_FAILED : Failed To Bind Socket to Address
 *  ERR_SOCKET_LISTEN_FAILED : Failed To Listen to Socket
 *  ERR_SOCKET_CONNECT_FAILED : Failed To Connect Socket to Address
 *  ERR_SOCKET_ACCEPT_FAILED : Failed To Accept Client Connection
 *  ERR_RECEIVE_DATA_FAILED : Failed To Receive Data
 *  ERR_SEND_DATA_FAILED : Failed To Send Data
 *  ERR_RECEIVE_MESSAGE_FAILED : Failed To Receive Message
 *  ERR_SEND_MESSAGE_FAILED : Failed To Send Message  
 *  ERR_ILLEGAL_IP_ADDRESS : Illegal IP Address
 *  ERR_REGISTRATION : Registration Error
 *  ERR_DEREGISTRATION : Deregistration Error
 *  ERR_UNSUPPORTED_MESSAGE_VALUE_TYPE : Unsupported Message Value Type
 *  ERR_MESSAGE_VALUE_TYPE_MISMATCH : Message Value Type mismatch
 *  ERR_MESSAGE_PARSE_ERROR : Message Parse Error 
 *  ERR_PACKET_PARSE_ERROR : Packet Parse Error 
 *  ERR_OSC_PARSE : OSC Parsing Error
 *  ERR_OSC : Generic OSC Error
 *  ERR_COM : Generic Com Error
 *
 *  Created by Daniel Bisig on 10/4/07.
 */

#ifndef _iso_com_exception_h_
#define _iso_com_exception_h_

#include "iso_com_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace com
{

class ComException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	ComException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	ComException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);

	/**
	\brief copy constructor
	\param pException exception
	*/
	ComException( const Exception& pException );
	
	/**
	\brief destructor
	*/
	~ComException();
		
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const ComException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
			
protected:
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();
	
	/**
	\brief default constructur
	*/
	ComException();
};
	
};

};

#endif