/**	\file iso_com_message.h
 *
 *  Created by Daniel Bisig on 14/05/09.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_com_message_h_
#define _iso_com_message_h_

#include "iso_com_types.h"
#include "iso_com_exception.h"
#include "iso_com_message_value_group.h"
#include <iso_base/iso_base_exception.h>
#include <QVector>
#include <QByteArray>

namespace iso
{

namespace com
{

class Message
{
public:
	/**
	\brief default constructor
	*/
	Message();

	/**
	\brief copy constructor
	\param pMessage message to copy
	*/
	Message( const Message& pMessage );
	
	/**
	\brief destructor
	*/
	~Message();

	/**
	\brief assignment operator
	\param pMessage message to copy value groups from
	\return message reference
	*/
	const Message& operator=(const Message& pMessage);
	
	/**
	\brief return number of value groups
	\returns number of value groups
	*/
	unsigned int valueGroupCount() const;

	/**
	\brief return message value group
	\param pValueGroupIndex value group index
	\return value group
	\exception ComException value group index out of bounds
	*/
	const MessageValueGroup& valueGroup( unsigned int pValueGroupIndex ) const throw (ComException);
	
	/**
	\brief return message value groups
	*/
	const QVector< MessageValueGroup* >& valueGroups() const;

	/**
	\brief return number of values in value group
	\param pValueGroupIndex value group index
	\return number of values in value group
	\exception ComException value group index out of bounds
	*/
	unsigned int valueCount( unsigned int pValueGroupIndex ) const;

	/**
	\brief check values
	\param pValueGroupIndex value group index
	\return true if value group matches type, false otherwise
	\exception ComException value group index out of bounds
	*/
	template<class Type> bool checkValues( unsigned int pValueGroupIndex ) const;

	/**
	\brief check values
	\param pValueGroupIndex value group index
	\return true if value group matches type and value count, false otherwise
	\exception ComException value group index out of bounds
	*/
	template<class Type, int ValueCount> bool checkValues( unsigned int pValueGroupIndex ) const;

	/**
	\brief return values
	\param pValueGroupIndex value group index
	\return array of typed values
	\exception ComException value group index out of bounds or type mismatch
	*/
	template<class Type> const Type* values( unsigned int pValueGroupIndex ) const throw (ComException);

	/**
	\brief add values
	\param pValueCount number of values
	\param pValues values
	\remarks: creates new value group containing a copy of passed values
	*/
	template<class Type> void addValues( unsigned int pValueCount, Type* pValues );
	
	/**
	\brief add value group
	\param pValueGroup value group
	\remark value group is added to message (no copy is created, i.e, this value group will be deleted when the message is destroyed)
	*/
	void addValues( MessageValueGroup* pValueGroup );

	/**
	\brief remove values
	\param pValueGroupIndex index of value group
	\exception index out of bounds
	*/
	void removeValues( unsigned int pValueGroupIndex ) throw (ComException);
	
	/**
	\brief remove and deleted all value groups
	*/
	void clear();

	/**
	\brief print textual message information 
	*/
	base::String info() const;
	
	/**
	\brief print message information
	\param pOstream output stream
	\param pMessage message
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const Message& pMessage)
	{
		pOstream << pMessage.info().toStdString(); 
		
		return pOstream;
	}

protected:
	QVector< MessageValueGroup* > mValueGroups; ///\brief value groups
};

template<class Type> 
bool 
Message::checkValues( unsigned int pValueGroupIndex ) const
{
	if( pValueGroupIndex >= mValueGroups.size() ) throw( ComException( "ERR_COM", base::String("value group index out of bounds"), __FILE__, __FUNCTION__, __LINE__) );	

	return mValueGroups[pValueGroupIndex]->checkValues<Type>();
}

template<class Type, int ValueCount> 
bool 
Message::checkValues( unsigned int pValueGroupIndex ) const
{
	if( pValueGroupIndex >= mValueGroups.size() ) throw( ComException( "ERR_COM", base::String("value group index out of bounds"), __FILE__, __FUNCTION__, __LINE__) );

	return mValueGroups[pValueGroupIndex]->checkValues<Type, ValueCount>();
}

template<class Type> 
const Type* 
Message::values( unsigned int pValueGroupIndex ) const throw (ComException)
{
	if( pValueGroupIndex >= mValueGroups.size() ) throw( ComException( "ERR_COM", base::String("value group index out of bounds"), __FILE__, __FUNCTION__, __LINE__) );

	try
	{
		return mValueGroups[pValueGroupIndex]->values<Type>();
	}
	catch(ComException)
	{
		throw;
	}
}

template<class Type> 
void 
Message::addValues( unsigned int pValueCount, Type* pValues )
{
	mValueGroups.append( new MessageValueGroup( pValueCount, pValues ) );
}

};

};

#endif