/** \file iso_com_message_parser.h
 *
 *  Created by Daniel Bisig on 5/16/09.
 *
 */

#ifndef _iso_com_message_parser_h_
#define _iso_com_message_parser_h_

#include "iso_com_types.h"
#include "iso_com_exception.h"
#include "iso_com_message.h"

namespace iso 
{

namespace com
{

class ByteBuffer
{
public:
	ByteBuffer();
	ByteBuffer(unsigned int pSize);
	~ByteBuffer();
	
	char* buffer();
	const char* buffer() const;
	unsigned int size() const;
	
	void clear();
	virtual void resize(unsigned int pSize);

protected:
	char* mBuffer;
	unsigned int mSize;
};

class PendingByteBuffer : public ByteBuffer
{
public:
	PendingByteBuffer();
	PendingByteBuffer(unsigned int pSize);
	~PendingByteBuffer();
	
	bool empty() const;
	bool complete() const;
	
	void addBuffer( const ByteBuffer& pBuffer, unsigned int pSourceBufferOffset, unsigned int pTargetBufferOffset );

	void resize(unsigned int pSize);

protected:
	unsigned int mCompletedSize;
};

class MessageParser
{
public:
	MessageParser();
	
	virtual ~MessageParser();
	
	/**
	\brief calculate message buffer size
	\param pMessage message
	\return message buffer size (number of bytes)
	\exception ComException message doesn't conform to expected message format
	*/
	virtual unsigned int messageBufferSize( const Message& pMessage ) const throw (ComException) = 0;

	/**
	\brief check if message conforms to expected message format
	\param pMessage message
	\return true if message format is correct, false otherwise
	*/
	virtual bool checkMessage( const Message& pMessage ) const = 0;

	/**
	\brief convert message into message buffer
	\param pMessage message
	\param pMessageBuffer message buffer
	\exception ComException message doesn't conform to expected message format
	\remarks message buffer array must have been correctly allocated beforehand
	*/
	virtual void message2buffer( const Message& pMessage, ByteBuffer& pMessageBuffer ) const throw (ComException) = 0;

	/**
	\brief convert message buffer into message
	\param pMessageBuffer message buffer
	\param pMessage message
	\exception ComException message doesn't conform to expected message format
	*/
	virtual void buffer2message( const ByteBuffer& pMessageBuffer, Message& pMessage ) const throw (ComException) = 0;

	/**
	\brief convert message buffer into data packets
	\param pMessageBufferPos position of packet within message buffer
	\param pPacketBuffer message buffer
	\param pPackets vector of data packets
	\param pMaxPacketSize maximum packet size (in bytes)
	*/
	virtual void buffer2packets( const ByteBuffer& pMessageBuffer, QVector<ByteBuffer*>& pPackets, unsigned int pMaxPacketSize ) const throw (ComException) = 0;

	/**
	\brief check if packet buffer contains data for message buffer
	\return true if yes, false otherwise
	*/
	virtual bool match( ByteBuffer& pPacketBuffer, PendingByteBuffer& pMessageBuffer ) const throw (ComException) = 0;

	/**
	\brief convert data packet into message buffer
	\param pPacketBuffer data packet
	\param pMessageBuffer message buffer
	*/	
	virtual void packet2buffer( const ByteBuffer& pPacketBuffer, PendingByteBuffer& pMessageBuffer ) const throw (ComException) = 0;

	/**
	\brief convert message info into string
	\param pMessage message
	\return string containing textual message information
	\exception ComException message doesn't conform to expected message format
	*/
	virtual base::String messageInfo( const Message& pMessage ) const throw (ComException) = 0;	

	/**
	\brief retrieve textual information about data packet
	\param pPacketBuffer data packet buffer
	*/	
	virtual base::String packetInfo( const ByteBuffer& pPacketBuffer ) const throw (ComException) = 0;

};

};

};

#endif