/** \file iso_com_osc_message_parser.h
 *
 *  Created by Daniel Bisig on 5/20/09.
 */

#ifndef _iso_com_osc_message_parser_h_
#define _iso_com_osc_message_parser_h_

#include "iso_com_message_parser.h"

namespace iso 
{

namespace com
{

class OSCMessageParser : public MessageParser
{
public:
	/**
	\brief default contructor
	*/
	OSCMessageParser();
	
	/**
	\brief destructor
	*/
	~OSCMessageParser();

	/**
	\brief calculate message buffer size
	\param pMessage message
	\return message buffer size (number of bytes)
	\exception ComException message doesn't conform to expected message format
	*/
	unsigned int messageBufferSize( const Message& pMessage ) const throw (ComException);

	/**
	\brief check if message conforms to expected message format
	\param pMessage message
	\return true if message format is correct, false otherwise
	*/
	bool checkMessage( const Message& pMessage ) const;

	/**
	\brief convert message into message buffer
	\param pMessage message
	\param pMessageBuffer message buffer
	\exception ComException message doesn't conform to expected message format
	\remarks message buffer array must have been correctly allocated beforehand
	*/
	void message2buffer( const Message& pMessage, ByteBuffer& pMessageBuffer ) const throw (ComException);

	/**
	\brief convert message buffer into message
	\param pMessageBuffer message buffer
	\param pMessage message
	\exception ComException message doesn't conform to expected message format
	*/
	void buffer2message( const ByteBuffer& pMessageBuffer, Message& pMessage ) const throw (ComException);

	/**
	\brief convert message buffer into data packets
	\param pMessageBufferPos position of packet within message buffer
	\param pPacketBuffer message buffer
	\param pPackets vector of data packets
	\param pMaxPacketSize maximum packet size (in bytes)
	*/
	void buffer2packets( const ByteBuffer& pMessageBuffer, QVector<ByteBuffer*>& pPackets, unsigned int pMaxPacketSize ) const throw (ComException);

	/**
	\brief check if packet buffer contains data for message buffer
	\return true if yes, false otherwise
	*/
	bool match( ByteBuffer& pPacketBuffer, PendingByteBuffer& pMessageBuffer ) const throw (ComException);

	/**
	\brief convert data packet into message buffer
	\param pPacketBuffer data packet
	\param pMessageBuffer message buffer
	*/	
	void packet2buffer( const ByteBuffer& pPacketBuffer, PendingByteBuffer& pMessageBuffer ) const throw (ComException);

	/**
	\brief convert message info into string
	\param pMessage message
	\return string containing textual message information
	\exception ComException message doesn't conform to expected message format
	*/
	base::String messageInfo( const Message& pMessage ) const throw (ComException);	

	/**
	\brief retrieve textual information about data packet
	\param pPacketBuffer data packet buffer
	*/	
	base::String packetInfo( const ByteBuffer& pPacketBuffer ) const throw (ComException);

protected:
	inline unsigned int intmod4expand( unsigned int pValue ) const;

	QHash< const std::type_info*, char > mValueTypeMap;
};

unsigned int 
OSCMessageParser::intmod4expand( unsigned int pValue ) const
{
	int mod4val = pValue % 4;
	if( mod4val == 0 ) return pValue + 4;
	else return pValue + static_cast<unsigned int>(4 - mod4val);
}
	
};
	
};

#endif