/** \file iso_com_receiver.h
 *
 *  Created by Daniel Bisig on 6/1/09.
 *
 */

#ifndef _iso_com_receiver_h_
#define _iso_com_receiver_h_

#include "iso_com_message.h"
#include <QThread>
#include <QMutex>

namespace iso
{

namespace com
{

class MessageParser;
class NetworkReceiverProtocol;

class Receiver : public QThread
{
public:
	/**
	\brief constructor
	\param pName receiver name
	*/
	Receiver( const base::String& pName );

	/**
	\brief constructor
	\param pName receiver name
	\param pMessageParser message parser
	\param pNetworkProtocol
	*/
	Receiver( const base::String& pName, MessageParser* pMessageParser, NetworkReceiverProtocol* pNetworkProtocol );

	/**
	\brief destructor
	*/
	~Receiver();
	
	/**
	\brief return message parser
	\return message parser
	*/
	const MessageParser* parser() const;

	/**
	 \brief return network protocol
	 \return network protocol
	 */
	const NetworkReceiverProtocol* protocol() const;	
	
	/**
	\brief return receiver name
	\return receiver name
	*/
	const base::String& name() const;

	/**
	\brief check if messages are available
	\returns true if messages are available, false otherwise
	*/
	bool messagesAvailable() const;

	/**
	\brief remove available messages
	*/
	void removeAvailableMessages();
	
	/**
	\brief retrieve available messages
	\returns messages
	*/
	const QVector<Message*>& messages() const;

	/**
	\brief check if message is available
	\returns true if message is available, false otherwise
	*/
	bool messageAvailable() const;

	/**
	\brief remove available message
	*/
	void removeAvailableMessage();
	
	/**
	\brief retrieve available message
	\returns message
	\exception ComException no message available
	*/
	const Message& message() const throw (ComException);
	
	/**
	\brief directly set message
	\param pMessage message
	*/
	void setMessage(const Message& pMessage);

	/**
	void update receiver
	*/
	void receive(); 

	/**
	\brief thread run
	*/
	void run();
	
	/**
	\brief stop receiving current message
	*/
	void abort();

protected:
	/**
	\brief default constructor 
	*/
	Receiver();

	base::String mName; ///\brief name
	//Message* mMessage; ///\brief message
	MessageParser* mMessageParser; ///\brief message parser
	NetworkReceiverProtocol* mNetworkProtocol; ///\brief network message protocol
	bool mAbort;
	QMutex mLock; ///\brief receiver blocked
	
	QVector< Message* > mMessages;
};

};

};

#endif