/** \file iso_data_buffer.h
 *  \class iso::data::Buffer
 *
 *  n-dimensional interleaved buffer
 *
 *  Created by Daniel Bisig on 12/4/07.
 *
 */
 
#ifndef _iso_data_buffer_h_
#define _iso_data_buffer_h_
 
#include "iso_data_types.h"
#include <iso_math/iso_math_includes.h>
 
namespace iso
{
 
namespace data
{
 
template< typename DataType > 
class Buffer
{
public:
	/**
	\brief create buffer
	*/
	Buffer();

	/**
	\brief create buffer
	\param pSize size of buffer
	\param pChannelCount number of channels
	\param pNoData buffer stores no data
	
	if buffer contains data, all buffer data is set to zero
	*/
	Buffer( const math::Vector<unsigned int>& pSize, unsigned int pChannelCount, bool pNoData = false );

	/**
	\brief create buffer
	\param pSize size of buffer
	\param pChannelCount number of channels
	\param pValue value of buffer data
	
	sets value of all array elements to pValue
	*/
    Buffer( const math::Vector<unsigned int>& pSize, unsigned int pChannelCount, DataType pValue );

	/**
	 \brief create buffer from vectorfield
	 \param pVectorField vector field
	 
	 the content of the vector field is copied
	 */
    Buffer( const math::VectorField< DataType >& pVectorField );	
	
	/**
	\brief copy constructor
	\param pBuffer buffer
	*/
	Buffer(const Buffer<DataType>& pBuffer);
	
	/**
	\brief destructor
	*/
	~Buffer();
	
	/**
	\brief reset buffer to new size & channelcount
	\param pSize size
	\param pChannelCount channel count
	*/
	void reset( const math::Vector<unsigned int>& pSize, unsigned int pChannelCount );
	
	/**
	\brief return dimension
	\return dimension
	*/
	unsigned int dim() const;
 
	/**
	\brief return channelCount
	\return channelCount
	*/
	unsigned int channelCount() const;

	/**
	\brief return size
	*/
	const math::Vector<unsigned int>& size() const;

	/**
	\brief return data count
	\return data count
	*/
	unsigned int dataCount() const;
 
	/**
	\brief return bufer data
	\return buffer data
	*/
	DataType* data();

	/**
	\brief return bufer data
	\return buffer data
	*/
	const DataType* data() const;
 
	/**
	\brief set all buffer data to same value
	\param pValue value
	*/
	void set( DataType pValue );

	/**
	\brief set all buffer data in specified channel to same value
	\param pChannel channel
	\param pValue value
	\remarks fails if channel is out of bounds
	*/
	void set( unsigned int pChannel, DataType pValue );
 
	/**
	 \brief set buffer data to that of vector field
	 \param pChannel channel
	 \param pValue value
	 \remarks fails if vectorfield dimension doesn't match buffer dimension
	 */
	void set( const math::VectorField< DataType >& pVectorField );	
	
	/**
	\brief print textual buffer information 
	*/
	base::String info() const;
		
	/**
	\brief print buffer information
	\param pOstream output stream
	\param pBuffer buffer
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const Buffer<DataType>& pBuffer)
	{
		pOstream << pBuffer.info().toStdString(); 
		
		return pOstream;
	}

	/**
	\brief buffer data
	*/
	DataType* mData;
 
protected:
	static int sChannelCount; /**< \brief default channel count **/
	static math::Vector<unsigned int> sSize; /**< \brief default size **/

	unsigned int mDim; ///\brief dimension
	unsigned int mChannelCount; ///\brief channel count
	math::Vector<unsigned int> mSize; ///\brief size
	unsigned int mDataCount; ///\brief data count
};

template<typename DataType> int Buffer<DataType>::sChannelCount = 1;
template<typename DataType> math::Vector<unsigned int> Buffer<DataType>::sSize(2, 512, 512);

template<typename DataType>
Buffer< DataType >::Buffer()
	: mDim(sSize.dim())
	, mSize(sSize)
	, mChannelCount(sChannelCount)
	, mDataCount(sSize.product() * sChannelCount)
{
	mData = new DataType[mDataCount];
	DataType value = static_cast<DataType>( 0.0 );
	for(unsigned int i=0; i<mDataCount; ++i) mData[i] = value;
}
 
template< typename DataType >
Buffer< DataType >::Buffer( const math::Vector<unsigned int>& pSize, unsigned int pChannelCount, bool pNoData )
	: mDim(pSize.dim())
	, mSize(pSize)
	, mChannelCount(pChannelCount)
	, mDataCount(mSize.product() * pChannelCount)
{
	if(pNoData == false) 
	{
		mData = new DataType[mDataCount];
		DataType value = static_cast<DataType>( 0.0 );
		for(unsigned int i=0; i<mDataCount; ++i) mData[i] = value;
	}
	else mData = NULL;
}

template< typename DataType >
Buffer< DataType >::Buffer( const math::Vector<unsigned int>& pSize, unsigned int pChannelCount, DataType pValue )
	: mDim(pSize.dim())
	, mSize(pSize)
	, mChannelCount(pChannelCount)
	, mDataCount(mSize.product() * pChannelCount)
{
	mData = new DataType[mDataCount];
	for(unsigned int i=0; i<mDataCount; ++i) mData[i] = pValue;
}

template< typename DataType >
Buffer< DataType >::Buffer( const math::VectorField< DataType >& pVectorField )
	: mDim( pVectorField.fieldDim() )
	, mSize( pVectorField.size() )
	, mChannelCount( pVectorField.vectorDim() )
	, mDataCount( mSize.product() * mChannelCount )
{
	mData = new DataType[mDataCount];
	
	int vectorCount = pVectorField.vectorCount();
	const Vector<DataType>** vectors = pVectorField.vectors();
	
	for(int vI=0, dI=0; vI<vectorCount; ++vI)
	{
		for(int d=0; d<mDim; ++d, ++dI)
		{
			mData[dI] = vectors[vI].c[d];
		}
	}
}

template< typename DataType >
Buffer< DataType >::Buffer( const Buffer<DataType>& pBuffer )
	: mDim(pBuffer.mDim)
	, mSize(pBuffer.mSize)
	, mChannelCount(pBuffer.mChannelCount)
	, mDataCount(pBuffer.mDataCount)
{
	if(pBuffer.mData != NULL)
	{
		mData = new DataType[mDataCount];
		for(unsigned int i=0; i<mDataCount; ++i) mData[i] = pBuffer.mData[i];
	}
	else mData = NULL;
}
 
template< typename DataType >
Buffer< DataType >::~Buffer( )
{
	if(mData != NULL) delete [] mData;
}

template< typename DataType >
void 
Buffer< DataType >::reset( const math::Vector<unsigned int>& pSize, unsigned int pChannelCount )
{
	if( mSize == pSize && mChannelCount == pChannelCount ) return;
	
	mDim = pSize.dim();
	mSize.setDim( mDim ); mSize = pSize;
	mChannelCount = pChannelCount;
	mDataCount = mSize.product() * mChannelCount;

	if(mData != NULL)
	{
		delete [] mData;
		mData = new DataType[mDataCount];
	}
}

template< typename DataType >
unsigned int 
Buffer< DataType >::dim() const
{
	return mDim;
}

template< typename DataType >
unsigned int 
Buffer< DataType >::channelCount() const
{
	return mChannelCount;
}

template< typename DataType >
const math::Vector<unsigned int>& 
Buffer< DataType >::size() const
{
	return mSize;
}

template< typename DataType >
unsigned int 
Buffer< DataType >::dataCount() const
{
	return mDataCount;
}

template< typename DataType >
DataType* 
Buffer< DataType >::data()
{
	return mData;
}

template< typename DataType >
const DataType* 
Buffer< DataType >::data() const
{
	return mData;
}


template< typename DataType >
void 
Buffer< DataType >::set( DataType pValue )
{
	for(unsigned int i=0; i<mDataCount; ++i) mData[i] = pValue;
}

template< typename DataType >
void 
Buffer< DataType >::set( unsigned int pChannel, DataType pValue )
{
	assert(pChannel < mChannelCount && "channel out of bounds");

	for(unsigned int i=pChannel; i<mDataCount; i += mChannelCount) mData[i] = pValue;
}
	
template< typename DataType >
void
Buffer< DataType >::set( const math::VectorField< DataType >& pVectorField )
{
	assert( pVectorField.fieldDim() == mDim && pVectorField.size() == mSize && "dimension mismatch between vector field and buffer");

	int vectorCount = pVectorField.vectorCount();
	const Vector<DataType>** vectors = pVectorField.vectors();
	
	for(int vI=0, dI=0; vI<vectorCount; ++vI)
	{
		for(int d=0; d<mDim; ++d, ++dI)
		{
			mData[dI] = vectors[vI].c[d];
		}
	}
}

template <typename DataType >
base::String 
Buffer< DataType >::info() const
{
	base::String infoString = base::String("data count %1\n")
	.arg(mDataCount);
	
	if(mData != NULL)
	{
		infoString.append("    data: ");
		for(unsigned int i=0; i<mDataCount; ++i) infoString.append(base::String("%1 ").arg(mData[i]));
		infoString.append("\n");
	}
	return infoString;
}

};
 
};
 
#endif

