/** \file iso_data_values.h
 * \class iso::data::Values stores a hierarchical set of arbitrary values
 * \brief stores a hierarchical set of arbitrary values
 *
 *  The Values class acts as a data container for mixed data types (both base data types, objects and other Values instances). 
 *  It is currently not possible, to directly store arrays of values. For this, the user has to resort to a class such as Array class or math::Vector class or whatever. 
 *  For a class to be storable in Values, it must implement a copy constructor and assignment operator. 
 *  Currently, only up to four data values can be stored on one level of a storage hierarchy. 
 *  It is trivial to extend this number by implementing additional templated constructors and get functions for the Values class that support more parameters.
 *  For multi level hierarchies, any level above the lowest one must contain at least two data values.
 *
 *  Usage example: storage (single level hierarchy) :
 *  Values values( 4, 8.6, base::String("Hello World"), Array<double>(3, 1.1, 2.2, 3.3) );
 *
 *  Usage example: retrieval (single level hierarchy) :
 *	int iRet;
 *	double dRet;
 *	base::String sRet; 
 *	Array<double> aRet;
 *	values.get( iRet, dRet, sRet, aRet );
 *
 *  Usage example: storage (two level hierarchy) :
 *  Values values( 4, 8.6, base::String("Hello World"), Array<double>(3, 1.1, 2.2, 3.3) );
 *  Values values2( 7.3, values );
 *
 *  Usage example: retrieval (two level hierarchy) :
 *	Values vRet;
 *  int iRet;
 *	double dRet1;
 *  double dRet2;
 *	base::String sRet; 
 *	Array<double> aRet;
 *
 *	values2.get( dRet1, vRet );
 *  vRet.get( iRet, dRet2, sRet, aRet );
 * 
 *  Created by Daniel Bisig on 3/13/09.
 */

#ifndef _iso_data_values_h_
#define _iso_data_values_h_

#include "iso_data_void_value.h"
#include "iso_data_typed_value.h"

namespace iso
{

namespace data
{

#pragma mark Values (definition)

class Values
{
public:
	Values();
	Values( const VoidValue& pValue1 );
	Values( const VoidValue& pValue1, const VoidValue& pValue2 );
	Values( const VoidValue& pValue1, const VoidValue& pValue2, const VoidValue& pValue3 );
	Values( const VoidValue& pValue1, const VoidValue& pValue2, const VoidValue& pValue3, const VoidValue& pValue4 );
	Values( const VoidValue& pValue1, const VoidValue& pValue2, const VoidValue& pValue3, const VoidValue& pValue4, const VoidValue& pValue5 );
	Values( const VoidValue& pValue1, const VoidValue& pValue2, const VoidValue& pValue3, const VoidValue& pValue4, const VoidValue& pValue5, const VoidValue& pValue6 );
	Values( const VoidValue& pValue1, const VoidValue& pValue2, const VoidValue& pValue3, const VoidValue& pValue4, const VoidValue& pValue5, const VoidValue& pValue6, const VoidValue& pValue7 );
	Values( const Values& pValues );
	template <class Type> Values( const Type& pValue );	
	template <class TypeA, class TypeB > Values( const TypeA& pValue1, const TypeB& pValue2 );
	template <class TypeA, class TypeB, class TypeC > Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3 );
	template <class TypeA, class TypeB, class TypeC, class TypeD > Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4 );	
	template <class TypeA, class TypeB, class TypeC, class TypeD, class TypeE > Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4, const TypeE& pValue5 );
	template <class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF> Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4, const TypeE& pValue5, const TypeF& pValue6 );
	template <class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF, class TypeG> Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4, const TypeE& pValue5, const TypeF& pValue6, const TypeG& pValue7 );
	~Values();
	
	Values* copy() const;
	unsigned int valueCount() const;
	VoidValue** values();
	const VoidValue** values() const;
	VoidValue& operator[]( unsigned int pIndex ) throw (base::Exception);
	const VoidValue& operator[]( unsigned int pIndex ) const throw (base::Exception);
	const Values& operator=(const VoidValue& pValue) throw (base::Exception);
	const Values& operator=(const Values& pValues) throw (base::Exception);
	template <class Type> operator Type() const throw (base::Exception);	
	bool match(const Values& pValues) const;
	template <class Type> bool match() const;
	template< class TypeA, class TypeB > bool match() const;
	template< class TypeA, class TypeB, class TypeC > bool match() const;
	template< class TypeA, class TypeB, class TypeC, class TypeD > bool match() const;
	template< class Type > void get( Type& pValue ) const throw(base::Exception);
	template< class TypeA, class TypeB > void get( TypeA& pValue1, TypeB& pValue2 ) const throw(base::Exception);
	template< class TypeA, class TypeB, class TypeC > void get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3 ) const throw(base::Exception);
	template< class TypeA, class TypeB, class TypeC, class TypeD > void get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4 ) const throw(base::Exception);
	template< class TypeA, class TypeB, class TypeC, class TypeD, class TypeE > void get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4, TypeE& pValue5 ) const throw(base::Exception);
	template< class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF > void get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4, TypeE& pValue5, TypeF& pValue6 ) const throw(base::Exception);
	template< class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF, class TypeG > void get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4, TypeE& pValue5, TypeF& pValue6, TypeG& pValue7 ) const throw(base::Exception);


	base::String info() const;
	
	friend std::ostream& operator<< (std::ostream & pOstream, const Values& pValues)
	{
		pOstream << pValues.info().toStdString(); 
		
		return pOstream;
	}	
	
protected:
	VoidValue** mValues;
	unsigned int mValueCount;
};

#pragma mark Values (implementation)

template <class Type>
Values::Values( const Type& pValue )
	: mValueCount(1)
{
	mValues = new VoidValue*[mValueCount];
	mValues[0] = new TypedValue<Type>( pValue );
};

template <class TypeA, class TypeB >
Values::Values( const TypeA& pValue1, const TypeB& pValue2 )
	: mValueCount(2)
{
	mValues = new VoidValue*[mValueCount];
	mValues[0] = new TypedValue<TypeA>( pValue1 );
	mValues[1] = new TypedValue<TypeB>( pValue2 );
};	

template <class TypeA, class TypeB, class TypeC >
Values::Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3 )
	: mValueCount(3)
{
	mValues = new VoidValue*[mValueCount];
	mValues[0] = new TypedValue<TypeA>( pValue1 );
	mValues[1] = new TypedValue<TypeB>( pValue2 );
	mValues[2] = new TypedValue<TypeC>( pValue3 );
};	

template <class TypeA, class TypeB, class TypeC, class TypeD >
Values::Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4 )
	: mValueCount(4)
{
	mValues = new VoidValue*[mValueCount];
	mValues[0] = new TypedValue<TypeA>( pValue1 );
	mValues[1] = new TypedValue<TypeB>( pValue2 );
	mValues[2] = new TypedValue<TypeC>( pValue3 );
	mValues[3] = new TypedValue<TypeD>( pValue4 );
};	

template <class TypeA, class TypeB, class TypeC, class TypeD, class TypeE >
Values::Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4, const TypeE& pValue5 )
	: mValueCount(5)
{
	mValues = new VoidValue*[mValueCount];
	mValues[0] = new TypedValue<TypeA>( pValue1 );
	mValues[1] = new TypedValue<TypeB>( pValue2 );
	mValues[2] = new TypedValue<TypeC>( pValue3 );
	mValues[3] = new TypedValue<TypeD>( pValue4 );
	mValues[4] = new TypedValue<TypeE>( pValue5 );
};	

template <class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF >
Values::Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4, const TypeE& pValue5, const TypeF& pValue6 )
	: mValueCount(6)
{
	mValues = new VoidValue*[mValueCount];
	mValues[0] = new TypedValue<TypeA>( pValue1 );
	mValues[1] = new TypedValue<TypeB>( pValue2 );
	mValues[2] = new TypedValue<TypeC>( pValue3 );
	mValues[3] = new TypedValue<TypeD>( pValue4 );
	mValues[4] = new TypedValue<TypeE>( pValue5 );
	mValues[5] = new TypedValue<TypeF>( pValue6 );
};

template <class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF, class TypeG >
Values::Values( const TypeA& pValue1, const TypeB& pValue2, const TypeC& pValue3, const TypeD& pValue4, const TypeE& pValue5, const TypeF& pValue6, const TypeG& pValue7 )
	: mValueCount(7)
{
	mValues = new VoidValue*[mValueCount];
	mValues[0] = new TypedValue<TypeA>( pValue1 );
	mValues[1] = new TypedValue<TypeB>( pValue2 );
	mValues[2] = new TypedValue<TypeC>( pValue3 );
	mValues[3] = new TypedValue<TypeD>( pValue4 );
	mValues[4] = new TypedValue<TypeE>( pValue5 );
	mValues[5] = new TypedValue<TypeF>( pValue6 );
	mValues[6] = new TypedValue<TypeG>( pValue7 );
};

template <class Type> 
Values::operator Type() const throw (base::Exception)
{
	try
	{
		Type val;
		get(val);
		return val; 
	}
	catch(base::Exception& e)
	{
		throw;
	}
}	
	
template <class Type>
bool
Values::match() const
{
	if( mValueCount < 1 ) return false;
	return mValues[0]->match<Type>();
}

template< class TypeA, class TypeB >
bool
Values::match() const
{
	if( mValueCount < 2 ) return false;
	if( mValues[0]->match<TypeA>() == false ) return false;
	if( mValues[1]->match<TypeB>() == false ) return false;
	return true;
}	

template< class TypeA, class TypeB, class TypeC >
bool
Values::match() const
{
	if( mValueCount < 3 ) return false;
	if( mValues[0]->match<TypeA>() == false ) return false;
	if( mValues[1]->match<TypeB>() == false ) return false;
	if( mValues[2]->match<TypeC>() == false ) return false;
	return true;
}	

template< class TypeA, class TypeB, class TypeC, class TypeD >
bool
Values::match() const
{
	if( mValueCount < 4 ) return false;
	if( mValues[0]->match<TypeA>() == false ) return false;
	if( mValues[1]->match<TypeB>() == false ) return false;
	if( mValues[2]->match<TypeC>() == false ) return false;
	if( mValues[3]->match<TypeD>() == false ) return false;
	return true;
}	

template< class Type >
void 
Values::get( Type& pValue ) const throw(base::Exception)
{
	if( mValueCount < 1 ) throw( base::Exception( "ERR_OUT_OF_BOUNDS", base::String("number of requested elements (1) exceeds number of stored elements (%2)").arg( mValueCount ), __FILE__, __FUNCTION__, __LINE__ ));

	mValues[0]->get( pValue );
}

template< class TypeA, class TypeB >
void 
Values::get( TypeA& pValue1, TypeB& pValue2 ) const throw(base::Exception)
{
	if( mValueCount < 2 ) throw( base::Exception( "ERR_OUT_OF_BOUNDS", base::String("number of requested elements (2) exceeds number of stored elements (%2)").arg( mValueCount ), __FILE__, __FUNCTION__, __LINE__ ));
	
	mValues[0]->get( pValue1 );	
	mValues[1]->get( pValue2 );
}

template< class TypeA, class TypeB, class TypeC >
void 
Values::get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3 ) const throw(base::Exception)
{
	if( mValueCount < 3 ) throw( base::Exception( "ERR_OUT_OF_BOUNDS", base::String("number of requested elements (3) exceeds number of stored elements (%2)").arg( mValueCount ), __FILE__, __FUNCTION__, __LINE__ ));
	
	mValues[0]->get( pValue1 );	
	mValues[1]->get( pValue2 );
	mValues[2]->get( pValue3 );
}	

template< class TypeA, class TypeB, class TypeC, class TypeD >
void 
Values::get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4 ) const throw(base::Exception)
{
	if( mValueCount < 4 ) throw( base::Exception( "ERR_OUT_OF_BOUNDS", base::String("number of requested elements (4) exceeds number of stored elements (%2)").arg( mValueCount ), __FILE__, __FUNCTION__, __LINE__ ));
	
	mValues[0]->get( pValue1 );	
	mValues[1]->get( pValue2 );
	mValues[2]->get( pValue3 );
	mValues[3]->get( pValue4 );
}	

template< class TypeA, class TypeB, class TypeC, class TypeD, class TypeE >
void 
Values::get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4, TypeE& pValue5 ) const throw(base::Exception)
{
	if( mValueCount < 5 ) throw( base::Exception( "ERR_OUT_OF_BOUNDS", base::String("number of requested elements (5) exceeds number of stored elements (%2)").arg( mValueCount ), __FILE__, __FUNCTION__, __LINE__ ));
	
	mValues[0]->get( pValue1 );	
	mValues[1]->get( pValue2 );
	mValues[2]->get( pValue3 );
	mValues[3]->get( pValue4 );
	mValues[4]->get( pValue5 );
}

template< class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF >
void 
Values::get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4, TypeE& pValue5, TypeF& pValue6 ) const throw(base::Exception)
{
	if( mValueCount < 6 ) throw( base::Exception( "ERR_OUT_OF_BOUNDS", base::String("number of requested elements (6) exceeds number of stored elements (%2)").arg( mValueCount ), __FILE__, __FUNCTION__, __LINE__ ));
	
	mValues[0]->get( pValue1 );	
	mValues[1]->get( pValue2 );
	mValues[2]->get( pValue3 );
	mValues[3]->get( pValue4 );
	mValues[4]->get( pValue5 );
	mValues[5]->get( pValue6 );
}

template< class TypeA, class TypeB, class TypeC, class TypeD, class TypeE, class TypeF, class TypeG >
void 
Values::get( TypeA& pValue1, TypeB& pValue2, TypeC& pValue3, TypeD& pValue4, TypeE& pValue5, TypeF& pValue6, TypeG& pValue7 ) const throw(base::Exception)
{
	if( mValueCount < 7 ) throw( base::Exception( "ERR_OUT_OF_BOUNDS", base::String("number of requested elements (7) exceeds number of stored elements (%2)").arg( mValueCount ), __FILE__, __FUNCTION__, __LINE__ ));
	
	mValues[0]->get( pValue1 );	
	mValues[1]->get( pValue2 );
	mValues[2]->get( pValue3 );
	mValues[3]->get( pValue4 );
	mValues[4]->get( pValue5 );
	mValues[5]->get( pValue6 );
	mValues[6]->get( pValue7 );
}

};

};

#endif