/** \file iso_event.h
 *
 *  Created by Daniel Bisig on 1/28/09.
 */

#ifndef _iso_event_h_
#define _iso_event_h_

#include "iso_event_types.h"
#include "iso_event_exception.h"
#include "iso_event.h"
#include <iso_base/iso_base_types.h>

namespace iso
{

namespace event
{

enum EventTime
{
	EVENT_RELATIVE,
	EVENT_ABSOLUTE
};

class Event
{
public:
	/**
	\brief default constructor
	*/
	Event();
	
	/**
	\brief create event
	\param pTime time in milisecs when event will take place
	\param pDuration duration over which the event takes place
	\param pTimeMode whether pTime is relative to current time or absolute (with regard to start of program)	
	*/
	Event(double pTime, double pDuration, EventTime pTimeMode);

	/**
	\brief copy constructor
	\param pEvent event to copy from
	*/
	Event( const Event& pEvent );

	/**
	\brief copy constructor
	\param pTime time in milisecs when event will take place
	\param pEvent event to copy from
	*/
	Event( double pTime, const Event& pEvent );

	/**
	\brief destructor
	*/
	virtual ~Event();
	
	/**
	\brief copy event
	\return copy of event
	*/
	virtual Event* copy() const = 0;

	/**
	\brief copy event
	\param pTime time in milisecs when event will take place
	\return copy of event
	*/
	virtual Event* copy( double pTime ) const = 0;
	
	/**
	\brief returns absolute time of event (in microsecs)
	\return absolute time of event (in microsecs)
	*/
	base::int64 time() const;
	
	/**
	\brief return event time mode
	\return event time mode
	*/
	EventTime timeMode() const;

	/**
	\brief returns event duration (in microsecs)
	\return event duration (in microsecs)
	*/
	base::int64 duration() const;

	/**
	\brief check if event has started executing
	\return bool
	*/
	bool started() const;	
	
	/**
	\brief check if event has completely executed
	\return bool
	*/
	bool finished() const;	

	/**
	\brief set time
	\param pTime time (in milisecs)
	\param pTimeMode whether pTime is relative to current time or absolute (with regard to start of program)	
	*/
	void setTime(double pTime, EventTime pTimeMode );

	/**
	\brief set time
	\param pTime time (in microsecs)
	
	time is set in absolute mode
	*/
	void setTime(base::int64 pTime );
	
	/**
	\brief execute event
	*/
	virtual void execute() throw (EventException) = 0;

	/**
	\brief obtain textual event information
	\return base::String containing event information
	*/
	virtual base::String info() const;

	/**
	\brief print event information
	\param pOstream output stream
	\param pEvent event
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const Event& pEvent)
	{
		pOstream << pEvent.info().toStdString(); 
		
		return pOstream;
	}
	
protected:
	EventTime mTimeMode; ///\brief time mode
	base::int64 mTime; ///\brief absolute time (in microsecs), this event is going to happen
	base::int64 mDuration; ///\brief duration (in microsecs), over which the event takes place
	bool mStarted; ///\brief event started flag
	bool mFinished; ///\brief event completion flag
};

};

};

#endif