/**	\file iso_event_exception.h
 *  \class iso::event::EventException class for handling all kinds of event related run time errors
 *	\brief class for handling all kinds of event related run time errors
 *
 *	Defines a variety of run time errors, which are specific to the math classes:
 *  ERR_TARGET_RESIZE_NOT_ALLOWED : EventTarget Cannot Be Resized
 *  ERR_EVENT : Event Error 
 *
 *  Created by Daniel Bisig on 5/17/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_event_exception_h_
#define _iso_event_exception_h_

#include "iso_event_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace event
{

class EventException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	EventException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	EventException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);

	/**
	\brief copy constructor
	\param pException exception
	*/
	EventException( const Exception& pException );
	
	/**
	\brief destructor
	*/
	~EventException();
	
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const EventException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
		
protected:	
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();

	/**
	\brief default constructur
	*/
	EventException();	
};

};

};


#endif