/**	\file iso_event_scheduler.h
 *  \class iso::event::EventScheduler schedules events
 *	\brief schedules events
 *
 *  stores events for later execution\n
 *  executes all events that are ready\n
 *
 *  Created by Daniel Bisig on 7/14/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_event_scheduler_h_
#define _iso_event_scheduler_h_

#include "iso_event_types.h"
#include <QVector>
#include <QList>
#include <QThread>

namespace iso
{

namespace event
{

class Event;
class EventManager;

class EventScheduler : public QThread
{
public:
	/**
	\brief default name
	*/
	static const base::String sClassName;

	/**
	\brief constructor
	\param pEventManager pointer to event manager
	*/
	EventScheduler(EventManager* pEventManager);

	/**
	\brief constructor
	\param pEventManager pointer to event manager
	\param pUpdateInterval update interval ( > 0 : automated update interval in microsecs, == 0 : manual update )
	*/
	EventScheduler(EventManager* pEventManager, unsigned int pUpdateInterval);
			
	/**
	\brief destructor
	*/
	~EventScheduler();
	
	/**
	\brief set update interval (in microsecs)
	\param pUpdateInterval update interval (in microsecs)
	*/
	void setUpdateInterval(unsigned int pUpdateInterval);

	/**
	\brief return number of events currently scheduled for execution
	\ return number of events currently scheduled for execution
	*/
	unsigned int eventCount() const;
	
	/**
	\brief add event to scheduler
	\param pEvent event
	
	adds event to temporary event vector
	*/
	void scheduleEvent(Event* pEvent);
	
	/**
	\brief remove all events from scheduler
	*/
	void removeEvents();
	
	/**
	\brief manually update event scheduler
	*/
	void update();
	
	/**
	\brief thread run function
	*/
	void run();
	
	/**
	\brief stop scheduler
	*/
	void stop();
	
	/**
	\brief obtain textual scheduler information
	\param pPropagate if true, the scheduler calls the info methods of all stored events
	\return base::String containing scheduler information
	*/
	base::String info(bool pPropagate = false) const;
			
protected:
	/**
	\brief default constructor
	*/
	EventScheduler();

	/**
	\brief check temporary event vector
	
	removes events from the temporary event vector and:
	a) executes them right away
	b) transfers them into the event list for later execution
	*/
	void checkEvents();

	/**
	\brief store event in event list
	\param pEvent event to store
	
	inserts event in a temporally sorted manner
	*/
	void storeEvent(Event* pEvent);

	/**
	\brief processes event list and executes events if necessary
	*/
	void processEvents();
	
	/**
	\brief default update interval (in microsecs)
	*/
	static unsigned int sUpdateInterval;

	/**
	\brief update interval (in microsecs)
	*/
	unsigned int mUpdateInterval;
	
	/**
	\brief flag to tell the thread to either run or stop
	
	flag to tell the thread to either run or stop
	
	false: thread keeps running
	true: thread stops
	*/
	bool mTerminated;

	/**
	\brief time sorted list of events
	*/
	QList<Event*> mEvents;

	/**
	\brief unSorted vector of events
	
	in order to avoid event being inserted by threads 
	while the scheduler is processing threads this vector 
	is a temporary storage for events till the scheduler 
	has time to insert them into the regular vector
	*/
	QVector<Event*> mTmpEvents;
	
	/**
	\brief pointer to event manager
	*/
	EventManager* mEventManager;
	
	bool mBlocked;
	
};

};

};

#endif
