/** \file iso_font_exception.h
 *  \class iso::font::FontException class for handling all kinds of font related run time errors
 *	\brief class for handling all kinds of font related run time errors
 *
 *	Defines a variety of run time errors, which are specific to the font classes:
 *  ERR_FONT_INITIALIZATION : Font System Could Not Be Initialized
 *  ERR_DUPLICATE_FONT : Font Already Exists
 *  ERR_UNKNOWN_FONT : Font Does Not Exists
 *  ERR_FONT_FILE_LOAD : Failed to Load Font File
 *  ERR_GLYPH_LOAD : Failed to Load Glyph
 *  ERR_FONT : Font Error 
 *
 *  Created by Daniel Bisig on 7/10/09.
 *
 */

#ifndef _iso_font_exception_h_
#define _iso_font_exception_h_

#include "iso_font_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace font
{

class FontException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	FontException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	FontException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);

	/**
	\brief copy constructor
	\param pException exception
	*/
	FontException( const Exception& pException );
	
	/**
	\brief destructor
	*/
	~FontException();
	
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const FontException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
		
protected:	
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();

	/**
	\brief default constructur
	*/
	FontException();	
};

};

};

#endif