/** \file iso_geom_cuboid.h
 *
 *  Created by Daniel Bisig on 5/18/07.
 */

#ifndef _iso_geom_cuboid_h_
#define _iso_geom_cuboid_h_

#include "iso_geom_types.h"
#include "iso_geom_geometry.h"
#include <iso_math/iso_math_rectangle.h>

namespace iso
{

namespace geom
{

class Cuboid : public Geometry
{
public:
	/**
	\brief create cuboid
	\param pDim dimension
	*/
	Cuboid(unsigned int pDim);

	/**
	\brief create cuboid
	\param pRect cuboid rectangle
	*/
	Cuboid(const math::Rectangle<real>& pRect);

	/**
	\brief create cuboid
	\param pMinPos minimum position
	\param pMaxPos maximum position
	*/
	Cuboid(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);
	
	/**
	\brief destructor
	*/
	virtual ~Cuboid();

	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();
	
	/**
	\brief return size
	\return size
	*/
	math::Vector<real> size() const;

	/**
	\brief return minimum position
	\return minimum position
	*/	
	math::Vector<real>& minPos();

	/**
	\brief return maximum position
	\return maximum position
	*/		
	math::Vector<real>& maxPos();

	/**
	\brief return minimum position
	\return minimum position
	*/	
	const math::Vector<real>& minPos() const;

	/**
	\brief return maximum position
	\return maximum position
	*/		
	const math::Vector<real>& maxPos() const;	

	/**
	\brief set minimum position
	\param pValue minimum position value
	\remarks fails if position is bigger than maximum position
	*/
	void setMinPos(real pValue);
	
	/**
	\brief set minimum position
	\param pMinPos minimum position
	\remarks fails if dimension of position is wrong or position is bigger than maximum position
	*/
	void setMinPos(const math::Vector<real>& pMinPos);

	/**
	\brief set minimum position
	\param pValues array of minimum position values
	\remarks position is bigger than maximum position
	*/
	void setMinPos(const real* pValues);

	/**
	\brief set maximum position
	\param pValue maximum position value
	\remarks fails if position is smaller than minimum position
	*/
	void setMaxPos(real pValue);
	
	/**
	\brief set maximum position
	\param pMaxPos maximum position
	\remarks fails if dimension of position is wrong or position is smaller than minimum position
	*/
	void setMaxPos(const math::Vector<real>& pMaxPos);

	/**
	\brief set maximum position
	\param pValues array of maximum position values
	\remarks position is smaller than minimum position
	*/
	void setMaxPos(const real* pValues);

	/**
	\brief set min and max position
	\param pRect rect
	*/
	void set(const math::Rectangle<real>& pRect);

	/**
	\brief set minimum and maximum positions
	\param pMinPos minimum position
	\param pMaxPos maximum position
	\remarks fails if dimension of one position is wrong or minimum position is bigger than maximum position
	*/
	void set(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);

	/**
	\brief set minimum and maximum positions
	\param pMinValues array of minimum position values
	\param pMaxValues arary of maximum position values
	\remarks fails if minimum position is bigger than maximum position
	*/
	void set(const real* pMinValues, const real* pMaxValues);

	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

protected:
	/**
	\brief default constructor
	*/
	Cuboid();
	
	/**
	\brief cuboid rectangle
	*/
	math::Rectangle<real> mRect;
};

};

};

#endif