/** \file iso_geom_poly_line.h
 *
 *  Created by Daniel Bisig on 08/07/08.
 */

#ifndef _iso_geom_poly_line_h_
#define _iso_geom_poly_line_h_

#include "iso_geom_types.h"
#include "iso_geom_geometry.h"
#include "iso_geom_line.h"
#include "iso_geom_ray.h"
#include <iso_math/iso_math_vector.h>
#include <QVector>

namespace iso
{

namespace geom
{

class PolyLine : public Geometry
{
public:
	/**
	\brief default constructor
	*/
	PolyLine();
	
	/**
	\brief create polyline
	\param pDim dimension
	*/
	PolyLine( unsigned int pDim );

	/**
	\brief copy constructor
	\param pPoints vector of points
	\remark fails if pPoints contains zero points
	*/
	PolyLine(const QVector< math::Vector<real> >& pPoints);

	/**
	\brief copy constructor
	\param pDim dimension
	\param pPointCount number of points
	\param pPoints points
	*/
	PolyLine( unsigned int pDim, unsigned int pPointCount, const real* pPoints );
	
	/**
	\brief copy constructor
	\param pPolyLine poly line
	*/
	PolyLine(const PolyLine& pPolyLine);
	
	/**
	\brief destructor
	*/
	~PolyLine();

	bool closed() const;
	
	void setClosed( bool pClosed );
	
	/**
	\brief calculate minimum and maximum positions
	*/
	virtual void calcMinMaxPos();

	/**
	\brief return number of points
	\return number of points
	*/
	unsigned int pointCount() const;
	
	/**
	\brief return points
	\return points
	*/
	const QVector< math::Vector<real> >& points() const;

	/**
	\brief return points
	\return points
	*/
	QVector< math::Vector<real> >& points();

	/**
	\brief return point
	\param pIndex index of point
	\return point
	\remark fails if index is out of bounds
	*/	
	const math::Vector<real>& point( unsigned int pIndex ) const;

	/**
	\brief return point
	\param pIndex index of point
	\return point
	\remark fails if index is out of bounds
	*/	
	math::Vector<real>& point( unsigned int pIndex );
	
	/*
	\brief append point at end of polyline
	\param pPoint point
	\remark fails if dimension mismatch
	*/
	void append( const math::Vector<float>& pPoint );
	
	/**
	\brief remove all points from polyline
	*/
	void clear();
	
	/**
	\brief append polyline at end of polyline
	\param pPolyline polyline
	\remark fails if dimension mismatch
	*/
	void append( const PolyLine& pPolyLine );

	/**
	\brief replace polyline
	\param pPointCount
	\param pPoints points
	*/
	void replace( int pPointCount, const real* pPoints );
	
	/**
	\brief replace point in polyline
	\param pIndex index of point
	\param pPoint point
	\remark fails if index is out of bounds or dimension mismatch
	*/
	void replace( unsigned int pIndex, const math::Vector<float>& pPoint );
	
	/**
	\brief replace polyline with polyline
	\param pIndex starting index 
	\param pPolyline polyline
	\remark fails if index is out of bounds or dimension mismatch
	
	polyline will be extended if the new polyline exceeds the current one
	*/
	void replace( unsigned int pIndex, const PolyLine& pPolyLine );
	
	/**
	\brief assigment operator
	\param pPolyLine polyline
	\remark fails if dimension mismatch
	
	entire content of polyline will be replaced with content of passed polyline
	*/
	const PolyLine& operator=( const PolyLine& pPolyLine );

	/**
	\brief [] operator
	\param pIndex index of point
	\return point
	\remark fails if index is out of bounds
	*/
	inline math::Vector<real>& operator[]( unsigned int pIndex );

	/**
	\brief [] operator
	\param pIndex index of point
	\return point
	\remark fails if index is out of bounds
	*/
	inline const math::Vector<real>& operator[]( unsigned int pIndex ) const;
	
	/**
	\brief return closest point
	\param pRefPoint reference points
	\param pResPoint result point
	\remarks fails if dimension of reference and result points don't match dimension of geometry
	*/
	virtual void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);

	/**
	\brief return intersection point(s) with line
	\param pLine line 
	\param pResPoint result point
	\return true if there is an intersection, false otherwise
	\remarks fails if dimensions don't match
	*/
	virtual bool intersect( const Line& pLine, QVector< math::Vector<real> >& pResPoints );

	/**
	\brief return intersection point(s) with ray
	\param pRay ray 
	\param pResPoint result point
	\return true if there is an intersection, false otherwise
	\remarks fails if dimensions don't match
	*/
	virtual bool intersect( const Ray& pRay, QVector< math::Vector<real> >& pResPoints );

	/**
	\brief print vector information
	\param pOstream output stream
	\param pPolyLine polyline
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const PolyLine& pPolyLine)
    {
		pOstream << pPolyLine.info().toStdString();
		return pOstream;
    }; 
	
	operator base::String ()
	{
		return info();
	};	
	
	/**
	\brief print vector information
	*/
	base::String info() const;

protected:
	bool mClosed;
	QVector< math::Vector<real> > mPoints; ///\brief polyline points
};

math::Vector<real>& 
PolyLine::operator[]( unsigned int pIndex )
{
	#ifdef _geom_debug_
	assert(pIndex < mPoints.size() && "index out of bounds");
	#endif
	
	return mPoints[pIndex];
}

const math::Vector<real>& 
PolyLine::operator[]( unsigned int pIndex ) const
{
	#ifdef _geom_debug_
	assert(pIndex < mPoints.size() && "index out of bounds");
	#endif
	
	return mPoints[pIndex];
}

};

};

#endif