/** \file iso_geom_spline_alg.h
 *
 *  Created by Daniel Bisig on 4/1/07.
 */

#ifndef _iso_geom_spline_alg_h_
#define _iso_geom_spline_alg_h_

#include "iso_geom_types.h"
#include <iso_math/iso_math_vector.h>
#include <iso_math/iso_math_vector2.h>
#include <iso_math/iso_math_vector3.h>
#include <iso_math/iso_math_matrix.h>
#include <QVector>

namespace iso
{

namespace geom
{

class SplineAlg
{
public:
	/**
	\brief create spline algorithm
	\param p_dimension dimension of spline
	\param p_controlCount number of control points
	\param p_subdivisionCount number of subdivisions
	*/
	SplineAlg(unsigned int p_dimension, unsigned int p_controlCount, unsigned int p_subdivisionCount);
	
	/**
	\brief destructor
	*/
	virtual ~SplineAlg();
	
	/**
	\brief create a copy of a spline alg
	\return copy of spline alg
	*/
	virtual SplineAlg* copy() const = 0;
	
	/**
	\brief return spline dimension
	\return spline dimension
	*/
	unsigned int dimension() const;
	
	/**
	\brief return number of control points
	\return number of control points
	*/
	unsigned int controlCount() const;
	
	/**
	\brief return number of subdivisons
	\return number of subdivisions
	*/
	unsigned int subdivisionCount() const;

	/**
	\brief set subdivision count
	\param p_subdivisionCount subdivision count
	*/
	virtual void setSubdivisionCount(unsigned int p_subdivisionCount);
	
	/**
	\brief create spline
	\param p_controlPoints control points
	\param p_splinePoints resulting spline points
	\param b_closed whether spline should be closed or not
	*/
	virtual void createSpline(QVector< math::Vector<real> >& p_controlPoints, QVector< math::Vector<real> >& p_splinePoints, bool b_closed = false) = 0;
	
protected:
	/**
	\brief default constructor
	*/
	SplineAlg();
	
	/**
	\brief precalculate polynomial values
	*/
	virtual void preCalc() = 0;
	
	/**
	\brief spline dimension
	*/
	unsigned int m_dimension;

	/**
	\brief number of control points
	*/
	unsigned int m_controlCount;
	
	/**
	\brief number of subdivisions
	*/
	unsigned int m_subdivisionCount;
	
	/**
	\brief interpolation parameter matrix
	*/
	math::Matrix<real> m_SM;
	
	/**
	\brief spline parameter matrix
	*/
	math::Matrix<real> m_CM;
	
	/**
	\brief spline polynomial matrix
	*/
	math::Matrix<real> m_HM;
	
	/**
	\brief resulting point in matrix form
	*/
	math::Matrix<real> m_PM;

	/**
	\brief array of matrices containing precalculated polynomial values
	*/
	math::Matrix<real>** m_preCalcMatrices;
};

};

};

#endif