/** \file iso_gl_buffer_object.h
 *  iso_gl
 *
 *  Created by Daniel Bisig on 11/1/09.
 *
 */

#ifndef _iso_gl_buffer_object_h_
#define _iso_gl_buffer_object_h_

#include "iso_gl.h"
#include "iso_gl_types.h"
#include "iso_gl_exception.h"
#include <iso_data/iso_data_includes.h>

namespace iso
{

namespace gl
{

class BufferObject
{
public:
	BufferObject( GLenum pBufferType, GLenum pBufferUsage ) throw (GLException);
	~BufferObject();

	/**
	\brief returns true if texture has been created, false otherwise
	*/
	inline bool created() const;

	/**
	\brief create buffer object from array
	\param pArray array
	\param pDataStride number of values per group in array
	\remark fails if array has wrong format
	*/
	template<typename DataType> void create( data::Array< DataType >* pArray, GLint pDataStride ) throw (GLException);	

	/**
	\brief create buffer object from array
	\param pDataCount data count
	\param pDataStride number of values per group in array
	\param pDataValues data values
	\remark fails if array has wrong format
	*/
	template<typename DataType> void create( int pDataCount, GLint pDataStride, DataType* pDataValues ) throw (GLException);
	
	/**
	 \brief update buffer
	 \param pDataValues new buffer data values
	 \remark fails if array has wrong format
	 */	
	template<typename DataType> void update( DataType* pDataValues ) throw (GLException);
	
	GLenum bufferType() const;
	
	int dataCount() const;

	GLenum dataType() const;

	GLuint dataStride() const;

	/**
	\brief bind buffer object
	*/
	void bind() const;

	/**
	\brief release buffer object
	*/
	void release() const;
	
	/**
	\brief destroy buffer object
	*/
	void destroy();

protected:
	GLuint mId; /**< \brief id */
	GLenum mBufferType; /**< \brief buffer type */
	int mDataCount; /**< \brief data count */
	GLenum mDataType; /**< \brief data type */
	GLenum mBufferUsage; /**< \brief buffer usage */
	GLint mDataStride; /**< \brief group size */
	bool mCreated;
};

bool
BufferObject::created() const
{
	return mCreated;
} 

template<typename DataType> 
void 
BufferObject::create( data::Array< DataType >* pArray, GLint pDataStride ) throw (GLException)
{
	DataType dataType;
	
	mDataCount = pArray->dataCount();
	mDataType = glType( dataType );
	mDataStride = pDataStride;

	DataType* arrayData = pArray->data(); 

	GL::get().clearError();
	
	glBindBuffer( mBufferType, mId );  // bind buffer object
	glBufferData( mBufferType, sizeof(DataType) * mDataCount, arrayData, mBufferUsage );  // load array data into buffer object	

	try 
	{		
		GL::get().error();
	}
	catch(GLException& e)
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_BUFFEROBJECT", base::String("failed to create buffer object"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
	
	mCreated = true;
}

template<typename DataType> 
void 
BufferObject::create( int pDataCount, GLint pDataStride, DataType* pDataValues ) throw (GLException)
{
	if( mCreated == true ) return;
	
	DataType dataType;

	mDataCount = pDataCount;
	mDataType = glType( dataType  );
	mDataStride = pDataStride; 

	GL::get().clearError();

	glBindBuffer( mBufferType, mId );  // bind buffer object
	glBufferData( mBufferType, sizeof(DataType) * mDataCount * mDataStride, pDataValues, mBufferUsage );  // load array data into buffer object	
	// TODO: find a workaround. glBufferData crashes the program if it runs our of memory, this shouldn't happen according to OpenGL specifications. So it seems to be a OSX bug
	
	try 
	{	
		GL::get().error();
	}
	catch(GLException& e)
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_BUFFEROBJECT", base::String("failed to create buffer object"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
	
	mCreated = true;
}
	
template<typename DataType> 
void 
BufferObject::update( DataType* pDataValues ) throw (GLException)
{
	if( mCreated == false )
	{
		try 
		{
			create( mDataCount, mDataStride, pDataValues );
			return;
		}
		catch (GLException& e) 
		{
			throw;
		}
	}
	
	GL::get().clearError();
	
	glBindBuffer( mBufferType, mId );  // bind buffer object
	glBufferData( mBufferType, sizeof(DataType) * mDataCount * mDataStride, pDataValues, mBufferUsage );  // load array data into buffer object	

	
//	bind();
//	
//	GLvoid* bufferData = glMapBuffer( mBufferType, GL_READ_WRITE ); 
//	DataType* typedBufferData = (DataType*)bufferData;
//	
//	memcpy(typedBufferData, pDataValues, mDataCount * sizeof( DataType ) );
//	
//	glUnmapBuffer( mBufferType );
//	
//	release();
	
	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_TEXTURE", base::String("failed to update texture"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
}
	
};

};

#endif