/** \file iso_gl_buffered_texture.h
 *  \class iso::gl::BufferedTexture
 *
 *  Created by Daniel Bisig on 12/6/07.
 *
 */

#ifndef _iso_gl_buffered_texture_h_
#define _iso_gl_buffered_texture_h_

#include "iso_gl_texture.h"
#include <iso_data/iso_data_includes.h>

namespace iso
{

namespace gl
{

template<typename DataType>
class BufferedTexture : public Texture
{
public:
	BufferedTexture( DataType* pBufferData, const math::Vector<unsigned int>& pSize, GLenum pTarget, GLint pLevel, GLint pInternalFormat, GLenum pFormat ) throw (GLException);
	BufferedTexture( DataType* pBufferData, unsigned int pDim, unsigned int* pSize, GLenum pTarget, GLint pLevel, GLint pInternalFormat, GLenum pFormat ) throw (GLException);
	~BufferedTexture();
	
	void create() throw (GLException);
	void update() throw (GLException);
	void update( data::Buffer< DataType >* pBuffer ) throw (GLException);
	
	DataType* data();
	

protected:
	DataType* mBufferData;
};

template<typename DataType>
BufferedTexture<DataType>::BufferedTexture( DataType* pBufferData, const math::Vector<unsigned int>& pSize, GLenum pTarget, GLint pLevel, GLint pInternalFormat, GLenum pFormat ) throw (GLException)
	: Texture( pSize, pTarget, pLevel, pInternalFormat, pFormat, glType( DataType() ) )
	, mBufferData( pBufferData )
{}
	
template<typename DataType>
BufferedTexture<DataType>::BufferedTexture( DataType* pBufferData, unsigned int pDim, unsigned int* pSize, GLenum pTarget, GLint pLevel, GLint pInternalFormat, GLenum pFormat ) throw (GLException)
	: Texture( pDim, pSize, pTarget, pLevel, pInternalFormat, pFormat, glType( DataType() ) )
	, mBufferData( pBufferData )
{}

template<typename DataType>
BufferedTexture<DataType>::~BufferedTexture()
{}

template<typename DataType>
void
BufferedTexture<DataType>::create() throw (GLException)
{
	GL::get().clearError();
	
    glBindTexture(mTarget, mId);   
  
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 	glTexParameteri(mTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(mTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	
	if(mDim == 1) glTexImage1D(mTarget, mLevel, mInternalFormat, mSize[0], 0, mFormat, mType, mBufferData );
	else if(mDim == 2) glTexImage2D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], 0, mFormat, mType, mBufferData );
	else if(mDim == 3) glTexImage3D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], mSize[2], 0, mFormat, mType, mBufferData );	

	mCreated = true;

	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
	
		throw(GLException( "ERR_TEXTURE", base::String("failed to create buffered texture"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
}

template<typename DataType>
void
BufferedTexture<DataType>::update() throw (GLException)
{
	GL::get().clearError();
	
	glBindTexture(mTarget, mId);  
	
 	glTexParameteri(mTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(mTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	

	if(mDim == 1) glTexImage1D(mTarget, mLevel, mInternalFormat, mSize[0], 0, mFormat, mType, mBufferData );
	else if(mDim == 2) glTexImage2D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], 0, mFormat, mType, mBufferData );
	else if(mDim == 3) glTexImage3D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], mSize[2], 0, mFormat, mType, mBufferData );	

	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
	
		throw(GLException( "ERR_TEXTURE", base::String("failed to update buffered texture"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
}

template<typename DataType>
void
BufferedTexture<DataType>::update( data::Buffer<DataType>* pBuffer ) throw (GLException)
{
	GL::get().clearError();

	mBufferData = pBuffer->data();
	mSize = pBuffer->size();
	
	glBindTexture(mTarget, mId);
	
 	glTexParameteri(mTarget, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(mTarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(mTarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	

	if(mDim == 1) glTexImage1D(mTarget, mLevel, mInternalFormat, mSize[0], 0, mFormat, mType, mBufferData );
	else if(mDim == 2) glTexImage2D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], 0, mFormat, mType, mBufferData );
	else if(mDim == 3) glTexImage3D(mTarget, mLevel, mInternalFormat, mSize[0], mSize[1], mSize[2], 0, mFormat, mType, mBufferData );	

	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
	
		throw(GLException( "ERR_TEXTURE", base::String("failed to update buffered texture"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
}

template<typename DataType>
DataType*
BufferedTexture<DataType>::data()
{
	return mBufferData;
}

};

};

#endif