/** \file iso_gl_frame_buffer_object.h
 *  \class iso::gl::FrameBufferObject
 *
 *  make sure that opengl has been initialized before you create your first frame buffer object\n
 *
 *  Created by Daniel Bisig on 11/26/07.
 *
 */

#ifndef _iso_gl_frame_buffer_object_h_
#define _iso_gl_frame_buffer_object_h_

#include "iso_gl.h"
#include "iso_gl_types.h"
#include "iso_gl_exception.h"
#include <iso_math/iso_math_includes.h>
#include <iso_img/iso_img_includes.h>
#include <iso_data/iso_data_buffer.h>

namespace iso
{

namespace gl
{
	
enum FrameBufferAttachmentType
{
	ColorAttachment,
	DepthAttachment,
	StencilAttachment
};

class Texture;

class FrameBufferObject
{
public:
	/**
	\brief create frame buffer object for offscreen rendering
	\param pSize size of offscreen rendering buffer
	\param pColorType color type
	\param pDataType data type
	\remark fails if size exceeds maximum size allowed for render buffers
	*/
	FrameBufferObject( const math::Vector2<GLsizei> pSize, ColorType pColorType, DataType pDataType ) throw (GLException);

	/**
	 \brief create frame buffer object for offscreen rendering
	 \param pSize size of offscreen rendering buffer
	 \param pTextureTarget texture target
	 \param pColorType color type
	 \param pDataType data type
	 \remark fails if size exceeds maximum size allowed for render buffers
	 */
	FrameBufferObject( const math::Vector2<GLsizei> pSize, GLenum pTextureTarget, ColorType pColorType, DataType pDataType ) throw (GLException);
	
	/**
	 \brief create frame buffer object for offscreen rendering
	 \param pSize size of offscreen rendering buffer
	 \param pColorType color type
	 \param pDataType data type
	 \remark fails if size exceeds maximum size allowed for render buffers
	 */
	FrameBufferObject( GLsizei pSize[2], ColorType pColorType, DataType pDataType ) throw (GLException);	

	/**
	 \brief create frame buffer object for offscreen rendering
	 \param pSize size of offscreen rendering buffer
	 \param pTextureTarget texture target
	 \param pColorType color type
	 \param pDataType data type
	 \remark fails if size exceeds maximum size allowed for render buffers
	 */
	FrameBufferObject( GLsizei pSize[2], GLenum pTextureTarget, ColorType pColorType, DataType pDataType ) throw (GLException);
	
	/**
	\brief destructor
	*/
	~FrameBufferObject();

	inline bool created() const;
	
	void create() throw (GLException);

	/**
	\brief check frame buffer object status
	\return true if everything is fine, false otherwise
	*/
	bool checkStatus();

	/**
	 \brief return size
	 \param pSize size
	 */
	void size( GLsizei pSize[2] ) const;
	
	/**
	\brief return size
	\return size
	*/
	math::Vector2<GLsizei> size() const;

	/**
	\brief return internal color format
	\return internal color format
	*/
	ColorType colorType() const;

	/**
	\brief return data type
	\return data type
	*/
	DataType dataType() const;

	/**
	\brief switches rendering to frame buffer object
	\remark fails if frame buffer object is invalid
	
	rendering will be performed to all attached textures
	*/
	void bind( ) throw (GLException);
	
	/**
	\brief switches rendering to default mode
	*/
	void release();
	
	/**
	\brief retrieve color texture
	\param pAttachmentPoint texture attachment point
	\return color texture
	\remarks fails if attachement point is out of bounds
	
	returns NULL if there is no texture attached to supplied attachement point
	*/
	Texture* texture( unsigned int pAttachmentPoint  ) throw (GLException);
	
	/**
	 \brief retrieve depth texture
	 \return depth texture
	 
	 returns NULL if there is no depth texture attached
	 */
	Texture* depthTexture();
	
	/**
	 \brief retrieve stencil texture
	 \return stencil texture
	 
	 returns NULL if there is no stencil texture attached
	 */	
	Texture* stencilTexture();

	/**
	\brief add color texture
	\param pAttachmentPoint attachement point
	\remark fails if attachement point is out of bounds
	
	creates new color texture and attaches it to fbo\n
	*/
	void addTexture( unsigned int pAttachmentPoint ) throw (GLException);
	
	/**
	 \brief add depth texture
	 
	 creates new depth texture and attaches it to fbo\n
	 */
	void addDepthTexture();

	/**
	 \brief add stencil texture
	 
	 creates new stencil texture and attaches it to fbo\n
	 */
	void addStencilTexture();

	/**
	\brief add color texture
	\param pTexture color texture to attach
	\param pAttachmentPoint attachement point
	\remark fails if texture is of different size, color format or data type than frame buffer of attachement point is out of bounds

	attaches supplied texture to fbo\n
	*/
	void addTexture( Texture* pTexture, unsigned int pAttachmentPoint ) throw (GLException);

	/**
	 \brief add depth texture
	 \param pTexture depth texture to attach
	 \param pAttachmentPoint attachement point
	 \remark fails if texture is of different size or data type than frame buffer
	 
	 attaches supplied texture to fbo\n
	 */
	void addDepthTexture( Texture* pTexture ) throw (GLException);

	/**
	 \brief add stencil texture
	 \param pTexture stencil texture to attach
	 \param pAttachmentPoint attachement point
	 \remark fails if texture is of different size or data type than frame buffer
	 
	 attaches supplied texture to fbo\n
	 */
	void addStencilTexture( Texture* pTexture ) throw (GLException);
	
	/**
	\brief remove color texture
	\param pAttachmentPoint attachement point
	\remark fails if attachement point is out of bounds
	*/
	void removeTexture( unsigned int pAttachmentPoint ) throw (GLException);

	/**
	 \brief remove depth texture
	 */
	void removeDepthTexture();

	/**
	 \brief remove stencil texture
	 */
	void removeStencilTexture();	
	
	/**
	\brief add depth render buffer

	creates depth render buffer and attaches it to fbo\n
	*/
	void addDepthBuffer() throw (GLException);

	/**
	\brief add depth render buffer
	
	creates stencil render buffer and attaches it to fbo\n
	*/	
	void addStencilBuffer() throw (GLException);

	/**
	\brief return maximum number of textures
	\return maximum number of textures
	*/
	static int maxTextureCount();

	/**
	\brief return maximum size of render buffers
	\return maximum size of render buffers
	*/
	static int maxSize();
	
	/**
	\brief return content of framebuffer as image buffer
	\return content of framebuffer as image buffer
	*/
	template < class Type > data::Buffer<Type>* toBuffer() throw (GLException);
	
	template < typename DataType > void colorBuffer( DataType* pBuffer, unsigned int pAttachmentPoint ) throw (GLException);
	
	template < typename DataType > void depthBuffer( DataType* pBuffer ) throw (GLException);
	
	template < typename DataType > void stencilBuffer( DataType* pBuffer ) throw (GLException);

protected:
	/**
	\brief initialize global frame buffer object settings
	*/
	static void staticInitialize();

	/**
	\brief default constructor
	*/
	FrameBufferObject();
	
	/**
	\brief update draw buffers based on available textures
	*/
	void updateDrawBuffers();
	
	static bool sInitialized; /// \brief flag whether global frame buffer object settings have been initialized
	static GLint sMaxTextureCount; /// \brief maximum number of textures per fbo
	static GLint sMaxSize; /// \brief maximum size of fbo
	static GLenum sTextureTarget; /// \brief default texture target

	GLuint mId; /// \brief id
	GLsizei mSize[2];
	ColorType mColorType; ///\brief internal color format
	DataType mDataType; ///\brief data type
	
	Texture** mTextures; /// \brief all attached color textures
	Texture* mDepthTexture; ///\brief attached depth texture
	Texture* mStencilTexture; ///\brief attached stencil texture
	
	GLuint mDepthBuffer; /// \brief depth buffer 
	GLuint mStencilBuffer; /// \brief stencil buffer
	GLuint* mDrawBuffers; /// \brief draw buffers 
	unsigned int mDrawBufferCount; /// \brief number of currently available draw buffers
	bool* mTextureCreationFlags; /// \brief flags indicating whether color textures have been created by the fbo
	bool mDepthTextureCreationFlag; /// \brief flag indicating whether the depth texture have been created by the fbo
	bool mStencilTextureCreationFlag; /// \brief flag indicating whether the stencil texture have been created by the fbo
	bool mColorFlag; ///\brief flag indicating whether this fbo renders to a color or monochrome texture
	GLenum mTextureTarget; /// \brief texture target
};

template < class Type >
data::Buffer<Type>* 
FrameBufferObject::toBuffer() throw (GLException)
{
	GL::get().clearError();

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mId );
	
	int channelCount;
	
	if(mColorType == GL_RGB) channelCount = 3;
	else if(mColorType == GL_RGBA) channelCount = 4;
	else channelCount = 1;
	
	data::Buffer<Type>* buffer = new data::Buffer<Type>( math::Vector2<unsigned int>(mSize[0], mSize[1]), channelCount, false );
	glReadPixels(0, 0, mSize[0], mSize[1], mColorType, mDataType, buffer->data());

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	
	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_FRAMEBUFFEROBJECT", base::String("failed to retrieve color buffer at attachement point 0"), __FILE__, __FUNCTION__, __LINE__ )); 
	}
	
	return buffer;
}

template < typename DataType > 
void 
FrameBufferObject::colorBuffer( DataType* pBuffer, unsigned int pAttachmentPoint ) throw (GLException)
{
	GL::get().clearError();

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mId );
	
	glReadBuffer(GL_COLOR_ATTACHMENT0_EXT + pAttachmentPoint );
	glReadPixels(0, 0, mSize[0], mSize[1], mColorType, mDataType, pBuffer);
	
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	
	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_FRAMEBUFFEROBJECT", base::String("failed to retrieve color buffer at attachement point %1").arg(pAttachmentPoint), __FILE__, __FUNCTION__, __LINE__ )); 
	}	
}

template < typename DataType > 
void 
FrameBufferObject::depthBuffer( DataType* pBuffer ) throw (GLException)
{
	GL::get().clearError();

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mId );
	
	glReadBuffer( GL_DEPTH_ATTACHMENT_EXT );
	glReadPixels(0, 0, mSize[0], mSize[1], GL_DEPTH_COMPONENT, mDataType, pBuffer);
	
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	
	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_FRAMEBUFFEROBJECT", base::String("failed to retrieve depth buffer"), __FILE__, __FUNCTION__, __LINE__ )); 
	}		
}

template < typename DataType > 
void 
FrameBufferObject::stencilBuffer( DataType* pBuffer ) throw (GLException)
{
	GL::get().clearError();

	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mId );
	
	glReadBuffer( GL_STENCIL_ATTACHMENT_EXT );
	glReadPixels(0, 0, mSize[0], mSize[1], GL_STENCIL_INDEX, mDataType, pBuffer);
	
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);

	try 
	{
		GL::get().error();
	}
	catch (GLException& e) 
	{
		std::cout << e << "\n";
		
		throw(GLException( "ERR_FRAMEBUFFEROBJECT", base::String("failed to retrieve stencil buffer"), __FILE__, __FUNCTION__, __LINE__ )); 
	}	
}

};

};

#endif
