/** \file iso_gl_texture_manager.h
 *  \class iso::gl::TextureManager
 *
 *  Created by Daniel Bisig on 12/4/07.
 *
 */

#ifndef _iso_gl_texture_manager_h_
#define _iso_gl_texture_manager_h_

#include "iso_gl_types.h"
#include "iso_gl_exception.h"
#include "iso_gl_texture.h"
#include "iso_gl_buffered_texture.h"
#include <iso_math/iso_math_includes.h>
#include <QHash>

namespace iso
{

namespace gl
{

class TextureManager
{

public:
	/**
	\brief retrieve singleton instance
	\return singleton instance
	*/
	static TextureManager& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();
	
	/**
	\brief return texture target
	\return texture target
	*/
	GLenum textureTarget() const;
	
	/**
	 \brief set texture target
	 \return texture target
	 */
	void setTextureTarget( GLenum pTextureTarget );
	
	/**
	\brief create texture
	\param pSize texture size
	\param pColorType texture color type
	\param pDataType texture data type
	\return texture
	\remark fails if dimension of size vector is not either 1, 2 or 3
	*/
	Texture* createTexture( const math::Vector<unsigned int>& pSize, ColorType pColorType, DataType pDataType ) throw (GLException);

	/**
	 \brief create texture
	 \param pSize texture size
	 \param pTextureTarget texture target
	 \param pColorType texture color type
	 \param pDataType texture data type
	 \return texture
	 \remark fails if dimension of size vector is not either 1, 2 or 3
	 */
	Texture* createTexture( const math::Vector<unsigned int>& pSize, GLenum pTextureTarget, ColorType pColorType, DataType pDataType ) throw (GLException);
	
	/**
	\brief create buffered texture from buffer
	\param pSize texture size
	\param pColorType texture color type
	\return texture
	\remark fails if dimension of size vector is not either 1, 2 or 3
	*/
	BufferedTexture<unsigned char>* createBufferedTexture( unsigned char* pBufferData, const math::Vector<unsigned int>& pSize, ColorType pColorType ) throw (GLException);

	/**
	 \brief create buffered texture from buffer
	 \param pSize texture size
	 \param pTextureTarget texture target
	 \param pColorType texture color type
	 \return texture
	 \remark fails if dimension of size vector is not either 1, 2 or 3
	 */
	BufferedTexture<unsigned char>* createBufferedTexture( unsigned char* pBufferData, const math::Vector<unsigned int>& pSize, GLenum pTextureTarget, ColorType pColorType ) throw (GLException);
	
	/**
	\brief create buffered texture from buffer
	\param pSize texture size
	\param pColorType texture color type
	\return texture
	\remark fails if dimension of size vector is not either 1, 2 or 3
	*/
	BufferedTexture<float>* createBufferedTexture( float* pBufferData, const math::Vector<unsigned int>& pSize, ColorType pColorType ) throw (GLException);

	/**
	 \brief create buffered texture from buffer
	 \param pSize texture size
	  \param pTextureTarget texture target
	 \param pColorType texture color type
	 \return texture
	 \remark fails if dimension of size vector is not either 1, 2 or 3
	 */
	BufferedTexture<float>* createBufferedTexture( float* pBufferData, const math::Vector<unsigned int>& pSize, GLenum pTextureTarget, ColorType pColorType ) throw (GLException);	
	
protected:
	static TextureManager* sTextureManager; ///\brief singleton instance variable
	static GLenum sTextureTarget; ///\brief default texture target
	
	/**
	\brief default constructor
	*/
	TextureManager();
	
	/**
	\brief destructor
	*/
	~TextureManager();

	GLenum mTextureTarget; ///\brief texture target
};

};

};

#endif
