/** \file iso_img_buffer.h
 *  
 *  Created by Daniel Bisig on 9/21/04.
 */

#ifndef _iso_img_buffer_h_
#define _iso_img_buffer_h_

#include "iso_img_exception.h"
#include <iso_math/iso_math_vector2.h>

namespace iso
{

namespace img
{

template< class Type >
class Buffer : public math::Vector<Type>
{
public:
	/**
	\brief default constructor
	*/
	Buffer();
	
	/**
	\brief create buffer
	\param pSize size of buffer
	\param pChannelCount number of channels
	*/
	Buffer(const math::Vector2<unsigned int>& pSize, unsigned int pChannelCount);
	
	/**
	\brief copy constructor
	\param pBuffer buffer
	*/
	Buffer(const Buffer& pBuffer);
	
	/**
	\brief destructor
	*/
	virtual ~Buffer();

	/**
	\brief return buffer width
	\return buffer width
	*/
	unsigned int width() const;

	/**
	\brief return buffer height
	*/
	unsigned int height() const;

	/**
	\brief return buffer size
	\return buffer size
	*/
	const math::Vector2<unsigned int>& size() const;
	
	/**
	\brief returns number of channels
	\return number of channels
	*/
	unsigned int channelCount() const;

	/**
	\brief return buffer values
	\return buffer values
	*/
	Type* data(); 

	/**
	\brief return buffer values
	\return buffer values
	*/
	const Type* data() const; 
	
	/**
	\brief return single channel data
	\param pChannel channel index
	\exception ImgException channel index out of bounds 
	*/
	Type* data(unsigned int pChannel) throw (ImgException);
	
	/**
	\brief return single buffer value
	\param pPos pixel position
	\param pChannel channel index
	\exception ImgException pixel position or channel index out of bouhnds
	*/
	Type data(const math::Vector2<unsigned int>& pPos, int pChannel) throw (ImgException);

	/**
	\brief set all values within a channel to the same value
	\param pChannel channel
	\param pValue value
	\exception ImgException channel out of bounds
	*/
	void setChannel(unsigned int pChannel, Type pValue) throw (ImgException);

	/**
	\brief change buffer size
	\param pSize new buffer size
	\param pChannelCount new channel count
	*/
	virtual void setSize(const math::Vector2<unsigned int>& pSize, unsigned int pChannelCount);

	/**
	\brief change number of frames
	\param pSize new buffer size
	*/
	virtual void setSize(const math::Vector2<unsigned int>& pSize);

	/**
	\brief change number of channels
	\param pChannelCount new channelCount
	*/	
	virtual void setChannelCount(unsigned int pChannelCount);

	/**
	\brief assignment operator
	\param pBuffer buffer to copy properties from
	\return buffer
	\exception ImgException size or channelCount doesn't match
	*/
	virtual const Buffer& operator= ( const Buffer& pBuffer ) throw (ImgException);
	
	/**
	\brief set content of buffer to one value
	\param pValue value to fill the buffer with
	*/
	virtual Type operator= ( Type pValue );
	
	/**
	\brief comparison operator
	\param pBuffer buffer to which the comparison is made to
	\return true, if ChannelCount and FrameCount of both buffers are identical, otherwise false
	*/ 
	virtual bool operator==( const Buffer& pBuffer ) const;

	// -------------------------------
	// Buffer - Scalar math operations
	// -------------------------------	
	
	/**
	\brief += operator
	\param pValue value to be added
	\return resulting buffer
	*/
	virtual const Buffer& operator+= ( Type pValue );	

	/**
	\brief *= operator
	\param pValue value to be multiplied
	\return resulting buffer
	*/
	virtual const Buffer& operator*= ( Type pValue );		
	
	// -------------------------------
	// Buffer - Buffer math operations
	// -------------------------------	
	
	/**
	\brief += operator
	\param pBuffer buffer to be added
	\return resulting buffer
	\exception ImgException buffers don't possess same dimensions
	*/
	virtual const Buffer& operator+= ( const Buffer& pBuffer ) throw (ImgException);

	/**
	\brief *= operator
	\param pBuffer buffer to be multiplied
	\return resulting buffer
	\exception ImgException buffers don't possess same dimensions
	*/
	virtual const Buffer& operator*= ( const Buffer& pBuffer ) throw (ImgException);	
	
	// -----
	// infos 
	// -----
	
	/**
	\brief print buffer information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const Buffer& pBuffer)
	{
		pOstream << pBuffer.info().toStdString();
		
		return pOstream;
	};
	
	/**
	\brief obtain textual buffer information
	\return String containing buffer information
	*/
	virtual base::String info() const;

protected:
	/**
	\brief set dimension of vector
	\param p_dim new vector dimension
	
	original values are preserved as far as possible.
	when vector dimension increases, new elements contain zero
	when vector dimension decreases, previous values at positions that are beyond new dimension
	*/
	virtual void setDim(unsigned int p_dim);

	/**
	\brief default buffer size
	*/
	static math::Vector2<unsigned int> sSize;

	/**
	\brief default channel count
	*/
	static unsigned int sChannelCount;
	
	/**
	\brief buffer size
	*/
	math::Vector2<unsigned int> mSize;
	
	/**
	\brief channel count
	*/
	unsigned int mChannelCount;
	
	/**
	\brief total data count
	*/
	unsigned int& mCount;
	
	/**
	\brief buffer data
	*/
	Type* mData;
};

template<class Type> 
math::Vector2<unsigned int> Buffer<Type>::sSize(256, 256);

template<class Type> 
unsigned int Buffer<Type>::sChannelCount = 1;

template<class Type> 
Buffer<Type>::Buffer()
	: math::Vector<Type>(sSize.x * sSize.y * sChannelCount)
	, mCount(math::Vector<Type>::mDim)
	, mSize(sSize)
	, mChannelCount(sChannelCount)
	, mData(math::Vector<Type>::c)
{}

template<class Type> 	
Buffer<Type>::Buffer(const math::Vector2<unsigned int>& pSize, unsigned int pChannelCount)
	: math::Vector<Type>(pSize.x * pSize.y * pChannelCount)
	, mCount(math::Vector<Type>::mDim)
	, mSize(pSize)
	, mChannelCount(pChannelCount)
	, mData(math::Vector<Type>::c)
{}
	
template<class Type> 
Buffer<Type>::Buffer(const Buffer& pBuffer)
	: math::Vector<Type>(pBuffer)
	, mCount(math::Vector<Type>::mDim)
	, mSize(pBuffer.mSize)
	, mChannelCount(pBuffer.mChannelCount)
	, mData(math::Vector<Type>::c)
{}

template<class Type> 	
Buffer<Type>::~Buffer()
{}

template<class Type>
unsigned int 
Buffer<Type>::width() const
{
	return mSize.x;
}

template<class Type>
unsigned int 
Buffer<Type>::height() const
{
	return mSize.y;
}

template<class Type>
const math::Vector2<unsigned int>& 
Buffer<Type>::size() const
{
	return mSize;
}
	
template<class Type>
unsigned int 
Buffer<Type>::channelCount() const
{
	return mChannelCount;
}

template<class Type>
Type* 
Buffer<Type>::data()
{
	return mData;
}

template<class Type>
const Type* 
Buffer<Type>::data() const
{
	return mData;
}

template<class Type>	
Type* 
Buffer<Type>::data(unsigned int pChannel) throw (ImgException)
{
	if(pChannel >= mChannelCount) throw(ImgException( "ERR_IMG_CHANNELCOUNT", base::String("channel out of bounds"), __FILE__, __FUNCTION__, __LINE__ ));
	
	return &mData[pChannel * mSize.x * mSize.y];
}

template<class Type>	
Type 
Buffer<Type>::data(const math::Vector2<unsigned int>& pPos, int pChannel) throw (ImgException)
{
	if(pPos.x >= mSize.x || pPos.y >= mSize.y) throw(ImgException( "ERR_IMG_SIZE", base::String("image size exceeded"), __FILE__, __FUNCTION__, __LINE__ ));
	if(pChannel >= mChannelCount) throw(ImgException( "ERR_IMG_CHANNELCOUNT", base::String("channel out of bounds"), __FILE__, __FUNCTION__, __LINE__ ));
	
	return mData[pPos.x + pPos.y * mSize.x + pChannel * mSize.x * mSize.y];
}

template<class Type>	
void 
Buffer<Type>::setChannel(unsigned int pChannel, Type pValue) throw (ImgException)
{
	if(pChannel >= mChannelCount) throw(ImgException( "ERR_IMG_CHANNELCOUNT", base::String("channel out of bounds"), __FILE__, __FUNCTION__, __LINE__ ));
	
	//for(unsigned int i = 0, channelDataCount = mSize.x * mSize.y, index = pChannel * channelDataCount; i < channelDataCount; ++i, ++index) mData[index] = pValue;
	unsigned int channelDataCount = mSize.x * mSize.y;
	//std::cout << "channelDataCount " << channelDataCount << "\n";
	for(unsigned int i = 0, index = pChannel * channelDataCount; i < channelDataCount; ++i, ++index) 
	{
		//std::cout << "i " << i << " index " << index << " value " << (int)(pValue) << "\n";
		
		mData[index] = pValue;
	}
}

template<class Type>	
void 
Buffer<Type>::setSize(const math::Vector2<unsigned int>& pSize, unsigned int pChannelCount)
{
	if(pSize == mSize && pChannelCount == mChannelCount) return;

	setDim(pSize.x * pSize.y * pChannelCount);
	mData = math::Vector<Type>::c;
	mSize = pSize;
	mChannelCount = pChannelCount;
}

template<class Type>	
void 
Buffer<Type>::setSize(const math::Vector2<unsigned int>& pSize)
{
	if(pSize == mSize) return;

	setDim(pSize.x * pSize.y * mChannelCount);
	mData = math::Vector<Type>::c;
	mSize = pSize;
}

template<class Type>	
void 
Buffer<Type>::setChannelCount(unsigned int pChannelCount)
{
	if(pChannelCount == mChannelCount) return;

	setDim(mSize.x * mSize.y * pChannelCount);
	mData = math::Vector<Type>::c;
	mChannelCount = pChannelCount;
}

template<class Type>	
const Buffer<Type>& 
Buffer<Type>::operator= ( const Buffer<Type>& pBuffer ) throw (ImgException)
{
	if(mChannelCount != pBuffer.mChannelCount || mSize != pBuffer.mSize) throw(ImgException( "ERR_IMG_BUFFER_BUFFER_MISMATCH", base::String("buffers possess different size or channel count"), __FILE__, __FUNCTION__, __LINE__ ));
	
	memcpy(mData, pBuffer.mData, sizeof(Type) * mCount);
}

template<class Type>
Type 
Buffer<Type>::operator= ( Type pValue )
{
	for(unsigned int i=0; i<mCount; ++i) mData[i] = pValue;
}

template<class Type>	
bool 
Buffer<Type>::operator==( const Buffer& pBuffer ) const
{
	if(mCount != pBuffer.mCount) return false;
	if(mChannelCount != pBuffer.mChannelCount) return false;
	if(mSize != pBuffer.mSize) return false;
	
	for(unsigned int i=0; i<mCount; ++i) if(mData[i] != pBuffer.mData[i]) return false;
	
	return true;
}

template<class Type>
const Buffer<Type>& 
Buffer<Type>::operator+= ( Type pValue )
{
	for(unsigned int i=0; i<mCount; ++i) mData[i] += pValue;
	return *this;	
}

template<class Type>
const Buffer<Type>& 
Buffer<Type>::operator*= ( Type pValue )
{
	for(unsigned int i=0; i<mCount; ++i) mData[i] *= pValue;
	return *this;	
}

template<class Type>
const Buffer<Type>& 
Buffer<Type>::operator+= ( const Buffer<Type>& pBuffer ) throw (ImgException)
{
	if(mChannelCount != pBuffer.mChannelCount || mSize != pBuffer.mSize) throw(ImgException( "ERR_IMG_BUFFER_BUFFER_MISMATCH", base::String("buffers possess different size or channel count"), __FILE__, __FUNCTION__, __LINE__ ));

	for(unsigned int i=0; i<mCount; ++i) mData[i] += pBuffer.mData[i];
	return pBuffer;	
}

template<class Type>
const Buffer<Type>&  
Buffer<Type>::operator*= ( const Buffer<Type>& pBuffer ) throw (ImgException)
{
	if(mChannelCount != pBuffer.mChannelCount || mSize != pBuffer.mSize) throw(ImgException( "ERR_IMG_BUFFER_BUFFER_MISMATCH", base::String("buffers possess different size or channel count"), __FILE__, __FUNCTION__, __LINE__ ));

	for(unsigned int i=0; i<mCount; ++i) mData[i] *= pBuffer.mData[i];
	return pBuffer;	
}

template<class Type>
void
Buffer<Type>::setDim(unsigned int pDim)
{
	math::Vector<Type>::setDim(pDim);
}

template<class Type>
base::String
Buffer<Type>::info() const
{
	base::String infoString = base::String("Buffer\n    width: %1\n    height: %2\n    channelCount: %3\n    Data:\n")
		.arg(mSize.x)
		.arg(mSize.y)
		.arg(mChannelCount);

	for(unsigned int channel = 0, index = 0; channel < mChannelCount; ++channel)
	{
		infoString.append(base::String("channel %1\n").arg(channel));
		
		for(unsigned int y=0; y<mSize.y; ++y)
		{
			for(unsigned int x=0; x<mSize.x; ++x, ++index)
			{
				infoString.append(base::String("%1 ").arg(mData[index]));
			}
			infoString.append("\n");
		}
	}
	
	return infoString;
}

};

}; //namespace iso

#endif

