/** \file iso_img_buffer_tools.h
 *
 *  Created by Daniel Bisig on 10/3/07.
 */

#ifndef _iso_img_buffer_tools_h_
#define _iso_img_buffer_tools_h_

#include "iso_img_buffer.h"
#include <iso_math/iso_math_vector_field.h>
#include <QImage>
#include <QImageReader>

namespace iso
{

namespace img
{

class BufferTools
{
public:
	/**
	\brief retrieve singleton instance
	\return singleton instance
	*/
	static BufferTools& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();
	
	/**
	\brief create buffer from image file
	\param pFileName image file
	\return Buffer buffer containing image data
	\exception base::Exception failed to create buffer from image file
	*/
	template<class Type> Buffer<Type>* createBuffer(const base::String& pFileName) throw (base::Exception);
	
	/**
	\brief create vector field from image file
	\param pFileName image file
	\return 2D Vector field containing image data
	\exception base::Exception failed to create vector field from image file
	*/
	template<class Type> VectorField<Type>* createVectorField(const base::String& pFileName) throw (base::Exception);

protected:
	static BufferTools* sBufferTools; /// \brief singleton instance

	/**
	\brief default constructor
	*/
	BufferTools();
	
	/**
	\brief destructor
	*/
	~BufferTools();
};

template<class Type> 
Buffer<Type>* 
BufferTools::createBuffer(const base::String& pFileName) throw (base::Exception)
{
	QImage image;
	
    if ( !image.load( pFileName ) ) throw(base::Exception( "ERR_FILE_OPEN_FAILED", base::String("failed to read image %1").arg(pFileName), __FILE__, __FUNCTION__, __LINE__ ));
	
	math::Vector2<unsigned int> imageSize( image.width(), image.height() );
	unsigned int imageChannelCount = image.depth() / 8;
	unsigned char* imageData = image.bits();	
	
	//std::cout << "image width " << imageSize.x << " height " << imageSize.y << " channels " << imageChannelCount << "\n";
	
	Buffer<Type>* imageBuffer = new Buffer<Type>(imageSize, imageChannelCount);
	Type* imageBufferData = imageBuffer->data();	
    
	for(int y=0, index = 0; y<imageSize.y; y++)
	{
		for(int x=0; x<imageSize.x; ++x)
		{
			for(int c=0; c<imageChannelCount; ++c, ++index)
			{
				imageBufferData[index] = static_cast<Type>( imageData[index] );
				
				//std::cout << " x " << x << " y " << y << " c " << c << " val " << (int)(imageBufferData[index]) << "\n";
			}
		}
	}
	
	return imageBuffer;
}

template<class Type> 
VectorField<Type>* 
BufferTools::createVectorField(const base::String& pFileName) throw (base::Exception)
{
	QImage image;
	
    if ( !image.load( pFileName ) ) throw(base::Exception( "ERR_FILE_OPEN_FAILED", base::String("failed to read image %1").arg(pFileName), __FILE__, __FUNCTION__, __LINE__ ));
	
	math::Vector2<unsigned int> imageSize( image.width(), image.height() );
	unsigned int imageChannelCount = image.depth() / 8;
	unsigned char* imageData = image.bits();	
	
	//std::cout << "image width " << imageSize.x << " height " << imageSize.y << " channels " << imageChannelCount << "\n";
	
	VectorField<Type>* vectorField = new VectorField<Type>(imageSize, imageChannelCount);
	Vector<Type>** vectors = vectorField->vectors();	
    
	for(int y=0, vI = 0, dI = 0; y<imageSize.y; y++)
	{
		for(int x=0; x<imageSize.x; ++x, ++vI)
		{
			for(int c=0; c<imageChannelCount; ++c, ++dI)
			{
				vectors[vI]->c[c] = static_cast<Type>( imageData[dI] );
				
				//std::cout << " x " << x << " y " << y << " c " << c << " val " << (int)(imageBufferData[index]) << "\n";
			}
		}
	}
	
	return vectorField;
}

};

};

#endif