/**	\file iso_img_exception.h
 *  \class iso::img::ImgException class for handling all kinds of event related run time errors
 *	\brief class for handling all kinds of img related run time errors
 *
 *	Defines a variety of run time errors, which are specific to the img classes:
 *  ERR_IMG_SIZE : Image Size Error
 *  ERR_IMG_CHANNELCOUNT : Image Channel Count Error
 *  ERR_IMG_BUFFER_BUFFER_MISMATCH : Buffer Buffer Format Mismatch
 *  ERR_IMG : Image Error 
 *
 *  Created by Daniel Bisig on 5/17/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_img_exception_h_
#define _iso_img_exception_h_

#include "iso_img_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace img
{

class ImgException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	ImgException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	ImgException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief destructor
	*/
	~ImgException();
	
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const ImgException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
		
protected:	
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();

	/**
	\brief default constructur
	*/
	ImgException();	
};

};

};


#endif