/**	\file iso_math_matrix.h
 *	\class iso::math::Matrix matrix class
 *	\brief matrix class
 *
 *  mxn matrix
 *
 *  | e00 e01 e02 e03 ... e0m |
 *  | e10 e11 e12 e13 ... e1m |
 *	...
 *  | en0 ne1 en2 en3 ... enm |
 *
 *  elements stored in row major format
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_math_matrix_h_
#define _iso_math_matrix_h_

#include "iso_math_vector2.h"
#include <sstream>

#ifdef minor
	#undef minor
#endif

namespace iso
{

namespace math
{

template< class Type > 
class Matrix
{
public:
	/**
	\brief matrix array
	*/
	Type* m;
	
	/**
	\brief create default zero matrix
	*/
	Matrix();
	
	/**
	\brief create zero matrix
	\param pRowCount number of rows
	\param pColCount number of columns 
	*/
	Matrix(unsigned int pRowCount, unsigned int pColCount);

	/**
	\brief create zero matrix
	\param pDim size of matrix (dim.x: rowCount, dim.y: colCount) 
	*/
	Matrix(const Vector2<unsigned int>& pDim);

	/**
	\brief create matrix containing identical values
	\param pRowCount number of rows
	\param pColCount number of columns
	\param pValue value
	*/	
	Matrix(unsigned int pRowCount, unsigned int pColCount, Type pValue);

	/**
	\brief create matrix containing identical values
	\param pDim size of matrix (dim.x: rowCount, dim.y: colCount) 
	\param pValue value
	*/
	Matrix(const Vector2<unsigned int>& pDim, Type pValue);

	/**
	\brief create matrix
	\param pRowCount number of rows
	\param pColCount number of columns
	\param pValues values (must be of correct size!)
	*/	
	Matrix(unsigned int pRowCount, unsigned int pColCount, Type* pValues);

	/**
	\brief create matrix
	\param pRowCount number of rows
	\param pColCount number of columns
	\param pValues values (must be of correct size!)
	*/	
	template< class Type2> Matrix(unsigned int pRowCount, unsigned int pColCount, Type2* pValues);
	
	/**
	\brief create matrix
	\param pDim size of matrix (dim.x: rowCount, dim.y: colCount) 
	\param pValues values (must be of correct size!)
	*/
	Matrix(const Vector2<unsigned int>& pDim, Type* pValues);

	/**
	\brief create matrix
	\param pDim size of matrix (dim.x: rowCount, dim.y: colCount) 
	\param pValues values (must be of correct size!)
	*/
	template< class Type2 > Matrix(const Vector2<unsigned int>& pDim, Type2* pValues);
	
	/**
	\brief copy constructor
	\param pMatrix matrix
	*/	
	Matrix(const Matrix<Type>& pMatrix);

	/**
	\brief copy constructor
	\param pMatrix matrix
	*/	
	template< class Type2 > Matrix(const Matrix<Type2>& pMatrix);
	
	/**
	\brief destructor
	*/
	virtual ~Matrix();

	/**
	\brief return matrix dimension
	\return matrix dimension
	*/
	const Vector2<unsigned int>& dim() const;
	
	/**
	\brief return element count
	\return element count
	*/
	unsigned int count() const;

	/**
	\brief return matrix values
	\return matrix values
	*/	
	Type* values();

	/**
	\brief return row vector
	\param pRow row
	\return row vector
	\exception MathException row out of bounds
	*/
	Vector<Type> row(unsigned int pRow) const throw (MathException);

	/**
	\brief return column vector
	\param pColumn column
	\return column vector
	\exception MathException column out of bounds
	*/
	Vector<Type> column(unsigned int pColumn) const throw (MathException);
	
	/**
	\brief calculate submatrix lacking row pRow and column pColumn
	\param pRow row to remove
	\param pColumn column to remove
	\return submatrix lacking row pRow and column pColumn
	\exception MathException column and / or row out of bounds
	*/
	Matrix<Type> minor(unsigned int pRow, unsigned int pColumn) const throw (MathException);

	/**
	\brief return matrix element
	\param pRow row
	\param pColumn column
	\return matrix element
	\exception MathException column and / or row out of bounds
	*/
	inline Type& operator() (unsigned int pRow, unsigned int pColumn) throw (MathException);
	
	/**
	\brief return matrix element
	\param pRow row
	\param pColumn column
	\return matrix element
	\exception MathException column and / or row out of bounds
	*/
	inline const Type& operator() (unsigned int pRow, unsigned int pColumn) const throw (MathException);

	/**
	\brief return trace of matrix
	\return trace of matrix
	\exception MathException matrix not square
	*/
	Type trace() const throw (MathException);	
	
	/**
	\brief calculate determinant of matrix
	\return determinant
	\exception MathException matrix not square
	*/
	Type determinant() const throw (MathException);
	
	/**
	\brief return transposed matrix
	\return transposed matrix
	*/
	Matrix<Type> transpose() const;

	/**
	\brief calculate transposed matrix
	\param pMatrix matrix that will contain transposed matrix
	\exception MathException matrix dimension mismatch
	*/
	void transpose(Matrix<Type>& pMatrix) const throw (MathException);
	
	/**
	\brief return adjoint of matrix
	\return adjoint of matrix
	\exception MathException matrix not square
	*/
	Matrix<Type> adjoint() const throw (MathException);

	/**
	\brief calculate adjoint of matrix
	\param pMatrix adjoint matrix
	\exception MathException matrix not square or matrix dimension mismatch
	*/
	void adjoint(Matrix<Type>& pMatrix) const throw (MathException);;
	
	/**
	\brief get inverse of matrix
	\return inverse of matrix
	\exception MathException matrix not square or matrix not invertible
	*/
	virtual Matrix<Type> inverse() const throw (MathException);
	
	/**
	\brief calculate inverse of a matrix
	\param pMatrix inverse matrix
	\exception MathException matrix not square or matrix dimension mismatch or matrix not invertible
	*/
	virtual void inverse(Matrix<Type>& pMatrix) const throw (MathException);

	/**
	\brief set matrix value
	\param pValue value
	\param pRow row index
	\param pColumn column index
	\exception MathException column and / or row out of bounds
	*/
	void set(Type pValue, unsigned int pRow, unsigned int pColumn) throw (MathException);

	/**
	\brief set matrix values
	\param pValue value
	*/
	void set(Type pValue);
	
	/**
	\brief set matrix values
	\param pValues values
	\remarks the values array needs to be of the correct size
	*/
	void set(Type* pValues);
	
	/**
	\brief set matrix values
	\param pValues values
	\remarks the values array needs to be of the correct size
	*/
	template< class Type2> void set(Type2* pValues);	
	
	/**
	\brief set matrix values
	\param pMatrix matrix
	\exception MathException matrix dimension mismatch
	*/
	void set(const Matrix<Type>& pMatrix) throw (MathException);

	/**
	\brief set matrix values
	\param pMatrix matrix
	\exception MathException matrix dimension mismatch
	*/
	template< class Type2 > void set(const Matrix<Type2>& pMatrix) throw (MathException);
	
	/**
	\brief set row values
	\param pRow row index
	\param pValue value
	\exception MathException row out of bounds
	*/
	void setRow(unsigned int pRow, Type pValue) throw (MathException);
	
	/**
	\brief set row values
	\param pRow row index
	\param pValues values
	\exception MathException row out of bounds
	\remarks the values array needs to be of the correct size
	*/
	void setRow(unsigned int pRow, Type* pValues) throw (MathException);

	/**
	\brief set row values
	\param pRow row index
	\param pValues values
	\exception MathException row out of bounds
	\remarks the values array needs to be of the correct size
	*/
	template< class Type2 > void setRow(unsigned int pRow, Type2* pValues) throw (MathException);
	
	/**
	\brief set row values
	\param pRow row index
	\param pVector row vector
	\exception MathException row out of bounds or vector dimension mismatch
	*/
	void setRow(unsigned int pRow, const Vector<Type>& pVector) throw (MathException);

	/**
	\brief set row values
	\param pRow row index
	\param pVector row vector
	\exception MathException row out of bounds or vector dimension mismatch
	*/
	template< class Type2 > void setRow(unsigned int pRow, const Vector<Type2>& pVector) throw (MathException);
	
	/**
	\brief set column values
	\param pColumn column index
	\param pValue value
	\exception MathException column out of bounds
	*/	
	void setColumn(unsigned int pColumn, Type pValue) throw (MathException);

	/**
	\brief set column values
	\param pColumn column index
	\param pValues values
	\exception MathException column out of bounds
	\remarks the values array needs to be of the correct size
	*/	
	void setColumn(unsigned int pColumn, Type* pValues) throw (MathException);

	/**
	\brief set column values
	\param pColumn column index
	\param pValues values
	\exception MathException column out of bounds
	\remarks the values array needs to be of the correct size
	*/	
	template < class Type2 > void setColumn(unsigned int pColumn, Type2* pValues) throw (MathException);
	
	/**
	\brief set column values
	\param pColumn column index
	\param pVector column vector
	\exception MathException column out of bounds or vector dimension mismatch
	*/	
	void setColumn(unsigned int pColumn, const Vector<Type>& pVector) throw (MathException);

	/**
	\brief set column values
	\param pColumn column index
	\param pVector column vector
	\exception MathException column out of bounds or vector dimension mismatch
	*/	
	template < class Type2 > void setColumn(unsigned int pColumn, const Vector<Type2>& pVector) throw (MathException);
	
	/**
	\brief set matrix values
	\param pValue value
	*/
	const Matrix<Type>& operator=(Type pValue);
	
	/**
	\brief set matrix values
	\param pValues values
	\remarks the value array needs to be of the correct size!
	*/
	const Matrix<Type>& operator=(Type* pValues);

	/**
	\brief set matrix values
	\param pValues values
	\remarks the value array needs to be of the correct size!
	*/
	template< class Type2 > const Matrix<Type>& operator=(Type2* pValues);
	
	/**
	\brief set matrix values
	\param pMatrix matrix
	\exception MathException Matrix dimension mismatch
	*/
	const Matrix<Type>& operator=(const Matrix<Type>& pMatrix) throw (MathException);

	/**
	\brief set matrix values
	\param pMatrix matrix
	\exception MathException Matrix dimension mismatch
	*/
	template< class Type2 > const Matrix<Type>& operator=(const Matrix<Type2>& pMatrix) throw (MathException);
	
	/**
	\brief matrix += scalar
	\param pValue value
	\return matrix
	*/
    const Matrix<Type>& operator+=( Type  pValue );
 
	/**
	\brief matrix -= scalar
	\param pValue value
	\return matrix
	*/	
	const Matrix<Type>& operator-=( Type  pValue ); 
	
	/**
	\brief matrix *= scalar
	\param pValue value
	\return matrix
	*/	
    const Matrix<Type>& operator*=( Type  pValue );
 
	/**
	\brief matrix /= scalar
	\param pValue value
	\return matrix
	*/	
	const Matrix<Type>& operator/=( Type  pValue );

	/**
	\brief matrix + scalar
	\param pValue value
	\return matrix
	*/	
    Matrix<Type> operator+( Type  pValue ) const;

	/**
	\brief matrix - scalar
	\param pValue value
	\return matrix
	*/
    Matrix<Type> operator-( Type  pValue ) const; 

	/**
	\brief matrix * scalar
	\param pValue value
	\return matrix
	*/
    Matrix<Type> operator*( Type  pValue ) const;

	/**
	\brief matrix / scalar
	\param pValue value
	\return matrix
	*/
    Matrix<Type> operator/( Type  pValue ) const;

	/**
	\brief matrix += matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    const Matrix<Type>& operator+=( const Matrix<Type>& pMatrix ) throw (MathException); 

	/**
	\brief matrix += matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    template< class Type2 > const Matrix<Type>& operator+=( const Matrix<Type2>& pMatrix ) throw (MathException); 

	/**
	\brief matrix -= matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    const Matrix<Type>& operator-=( const Matrix<Type>& pMatrix ) throw (MathException); 

	/**
	\brief matrix -= matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    template< class Type2 > const Matrix<Type>& operator-=( const Matrix<Type2>& pMatrix ) throw (MathException); 

	/**
	\brief matrix *= matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch or matrix not square
	*/
    const Matrix<Type>& operator*=( const Matrix<Type>& pMatrix ) throw (MathException);  

	/**
	\brief matrix *= matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch or matrix not square
	*/
    template< class Type2 > const Matrix<Type>& operator*=( const Matrix<Type2>& pMatrix ) throw (MathException);  

	/**
	\brief matrix + matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
	Matrix<Type> operator+( const Matrix<Type>& pMatrix ) const throw (MathException); 

	/**
	\brief matrix + matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
	template< class Type2 > Matrix<Type> operator+( const Matrix<Type2>& pMatrix ) const throw (MathException); 

	/**
	\brief matrix - matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    Matrix<Type> operator-( const Matrix<Type>& pMatrix ) const throw (MathException);

	/**
	\brief matrix - matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    template< class Type2 > Matrix<Type> operator-( const Matrix<Type2>& pMatrix ) const throw (MathException);

	/**
	\brief matrix * matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    Matrix<Type> operator*( const Matrix<Type>& pMatrix ) const throw (MathException); 

	/**
	\brief matrix * matrix
	\param pMatrix matrix
	\return matrix
	\exception MathException Matrix dimension mismatch
	*/
    template< class Type2 > Matrix<Type> operator*( const Matrix<Type2>& pMatrix ) const throw (MathException); 

	/**
	\brief matrix * vector
	\param pVector vector
	\return vector
	\exception Vector dimension mismatch
	*/
    Vector<Type> operator*( const Vector<Type>& pVector ) const throw (MathException); 

	/**
	\brief matrix * vector
	\param pVector vector
	\return vector
	\exception Vector dimension mismatch
	*/
    template< class Type2 > Vector<Type> operator*( const Vector<Type2>& pVector ) const throw (MathException); 
	
	/**
	\brief reset matrix elements to zero
	*/
	void reset();
	
	/**
	\brief print matrix information
	\param pOstream output stream
	\param pMatrix matrix
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const Matrix<Type>& pMatrix)
    {
		pOstream << pMatrix.info().toStdString();
		return pOstream;
    }; 
	
	/**
	\brief print matrix information
	*/
	base::String info() const;
	
protected:
	/**
	\brief default matrix size
	*/
	static Vector2<unsigned int> sDim;	

	/**
	\brief matrix size
	
	mDim.x : row count
	mDim.y : column count
	*/
	Vector2<unsigned int> mDim;
	
	/**
	\brief matrix element count
	*/
	unsigned int mCount;
};

template<typename Type> 
Vector2<unsigned int> Matrix<Type>::sDim(2, 2);

typedef Matrix<int>    MatrixI;
typedef Matrix<float>  MatrixF;
typedef Matrix<double> MatrixD;

template< class Type > 
Matrix<Type>::Matrix() 
	: mDim(sDim)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];
	for(unsigned int i=0; i<mCount; ++i) m[i] = 0;
}

template< class Type > 
Matrix<Type>::Matrix(unsigned int pRowCount, unsigned int pColCount)
	: mDim(pRowCount, pColCount)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];
	for(unsigned int i=0; i<mCount; ++i) m[i] = 0;
}

template< class Type > 
Matrix<Type>::Matrix(const Vector2<unsigned int>& pDim) 
	: mDim(pDim)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];
	for(unsigned int i=0; i<mCount; ++i) m[i] = 0;
}

template< class Type > 
Matrix<Type>::Matrix(unsigned int pRowCount, unsigned int pColCount, Type pValue)
	: mDim(pRowCount, pColCount)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValue;
}

template< class Type > 
Matrix<Type>::Matrix(const Vector2<unsigned int>& pDim, Type pValue)
	: mDim(pDim)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValue;
}

template< class Type > 
Matrix<Type>::Matrix(unsigned int pRowCount, unsigned int pColCount, Type* pValues)
	: mDim(pRowCount, pColCount)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];	
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValues[i];
}

template< class Type > 
template< class Type2 >
Matrix<Type>::Matrix(unsigned int pRowCount, unsigned int pColCount, Type2* pValues)
	: mDim(pRowCount, pColCount)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];	
	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type>( pValues[i] );
}

template< class Type > 
Matrix<Type>::Matrix(const Vector2<unsigned int>& pDim, Type* pValues) 
	: mDim(pDim)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];	
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValues[i];
}

template< class Type > 
template< class Type2 > 
Matrix<Type>::Matrix(const Vector2<unsigned int>& pDim, Type2* pValues) 
	: mDim(pDim)
	, mCount(mDim.x * mDim.y)
{
	m = new Type[mCount];	
	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type>( pValues[i] );
}

template< class Type > 
Matrix<Type>::Matrix(const Matrix<Type>& pMatrix) 
	: mDim(pMatrix.mDim)
	, mCount(pMatrix.mCount)
{
	m = new Type[mCount];	
	for(unsigned int i=0; i<mCount; ++i) m[i] = pMatrix.m[i];
}

template< class Type > 
template< class Type2 >
Matrix<Type>::Matrix(const Matrix<Type2>& pMatrix) 
	: mDim(pMatrix.dim())
	, mCount(pMatrix.count())
{
	m = new Type[mCount];	
	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type>( pMatrix.m[i] );
}

template< class Type > 
Matrix<Type>::~Matrix()
{
	delete [] m;
}

template< class Type > 
const Vector2<unsigned int>& 
Matrix<Type>::dim() const
{
	return mDim;
}

template< class Type > 
unsigned int 
Matrix<Type>::count() const
{
	return mCount;
}

template< class Type > 
Type* 
Matrix<Type>::values()
{
	return m;
}

template< class Type > 
Vector<Type> 
Matrix<Type>::row(unsigned int pRow) const throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	return Vector<Type>(mDim.y, &(m[pRow * mDim.y]));
}

template< class Type > 
Vector<Type> 
Matrix<Type>::column(unsigned int pColumn) const throw (MathException)
{
	#ifdef _math_debug_
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Vector<Type> colVec(mDim.x);
	Type* vecValues = colVec.values();
	
	for(unsigned int row=0, mIndex=pColumn; row < mDim.x; ++row, mIndex += mDim.y) vecValues[row] = m[mIndex];
	
	return colVec;
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::minor(unsigned int pRow, unsigned int pColumn) const throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Matrix<Type> matrix( mDim.x - 1, mDim.y - 1 );
	
	for( unsigned int row = 0, index = 0, mIndex = 0; row < mDim.x; ++row )
	{
		for( unsigned int column = 0; column < mDim.y; ++column, ++index )
		{
			if(row != pRow && column != pColumn)
			{
				matrix.m[mIndex] = m[index];
			
				mIndex++;
			}
		}
	}
	
	return matrix;
}

template< class Type > 
Type&
Matrix<Type>::operator() (unsigned int pRow, unsigned int pColumn) throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	return m[pRow * mDim.y + pColumn];
}

template< class Type > 
const Type&
Matrix<Type>::operator() (unsigned int pRow, unsigned int pColumn) const throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	return m[pRow * mDim.y + pColumn];
}

template< class Type > 
Type 
Matrix<Type>::trace() const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Type traceVal = 0;
	
	for( unsigned int i=0, index=0; i<mDim.x; ++i, index += (mDim.y + 1) ) traceVal += m[index];
	
	return traceVal;
}

template< class Type >
Type
Matrix<Type>::determinant() const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
		
	// simple cases
	if(mDim.x == 2) return m[0] * m[3] - m[2] * m[1];
	if(mDim.x == 3) return m[0] * ( m[4] * m[8] - m[5] * m[7] ) - m[1] * ( m[3] * m[8] - m[5] * m[6] ) + m[2] * ( m[3] * m[7] - m[4] * m[6]);

	// larger matrix size
	Type detValue = 0.0;
	for(unsigned int column=0; column < mDim.y; ++column)
	{
		Matrix<Type> minorMatrix = minor(0, column);
		detValue += pow(-1.0, static_cast<int>(column)) * m[column] * minorMatrix.determinant();
	}
	return detValue;
}

template< class Type > 
Matrix<Type>
Matrix<Type>::transpose() const
{
	Vector2<unsigned int> transDim(mDim.y, mDim.x);
	Matrix<Type> transMatrix(transDim);
	
	int index;
	
	for(int row=0, transIndex=0; row<transDim.x; ++row)
	{
		for(int column=0; column<transDim.y; ++column, ++transIndex)
		{
			index = column * mDim.y + row;
		
			transMatrix.m[transIndex] = m[index];
		}
	}		
	
	return transMatrix;
}

template< class Type > 
void 
Matrix<Type>::transpose(Matrix<Type>& pMatrix) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif	
	
	int index;
	
	for(int row=0, transIndex=0; row<pMatrix.mDim.x; ++row)
	{
		for(int column=0; column<pMatrix.mDim.y; ++column, ++transIndex)
		{
			index = column * mDim.y + row;
		
			pMatrix.m[transIndex] = m[index];
		}
	}		
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::adjoint() const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Matrix<Type> matrix(mDim);
	
	for(unsigned int row = 0, index=0; row < mDim.x; ++row)
	{
		for(unsigned int column = 0; column < mDim.y; ++column, ++index)
		{
			matrix.m[index] = pow(-1.0, static_cast<int>(row + column)) * minor(row, column).determinant();
		}
	}
	
	return matrix.transpose();
}

template< class Type > 
void
Matrix<Type>::adjoint(Matrix<Type>& pMatrix) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int row = 0, index=0; row < mDim.x; ++row)
	{
		for(unsigned int column = 0; column < mDim.y; ++column, ++index)
		{
			pMatrix.m[index] = pow(-1.0, static_cast<int>(row + column)) * minor(row, column).determinant();
		}
	}
	
	pMatrix.transpose(pMatrix);
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::inverse() const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	if(mDim.x == 2)
	{
		Matrix<Type> matrix(2, 2);
		float scale = 1.0 / (m[0] * m[3] - m[1] * m[2] );
		
		matrix.m[0] = m[3] * scale;
		matrix.m[1] = -m[1] * scale;
		matrix.m[2] = -m[2] * scale;
		matrix.m[3] = m[0] * scale;
		
		return matrix;
	}
	else
	{
		Matrix<Type> matrix = adjoint();
		Type det = determinant();
		
		if(det == 0.0) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not invertible"), __FILE__, __FUNCTION__, __LINE__ );
		
		return matrix * (1.0 / det);
	}
}

template< class Type > 
void 
Matrix<Type>::inverse(Matrix<Type>& pMatrix) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	adjoint(pMatrix);
	Type det = determinant();
	
	if(det == 0.0) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not invertible"), __FILE__, __FUNCTION__, __LINE__ );
	
	pMatrix *= (1.0 / det);
}

template< class Type > 
void 
Matrix<Type>::set(Type pValue, unsigned int pRow, unsigned int pColumn) throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	m[pRow * mDim.y + pColumn] = pValue;
}

template< class Type > 
void 
Matrix<Type>::set(Type pValue)
{
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValue;
}

template< class Type > 
void 
Matrix<Type>::set(Type* pValues)
{
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValues[i];
}

template< class Type > 
template< class Type2 >
void 
Matrix<Type>::set(Type2* pValues)
{
	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type2>( pValues[i] );
}

template< class Type > 
void 
Matrix<Type>::set(const Matrix<Type>& pMatrix) throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	for(unsigned int i=0; i<mCount; ++i) m[i] = pMatrix.m[i];
}

template< class Type > 
template< class Type2 > 
void 
Matrix<Type>::set(const Matrix<Type2>& pMatrix) throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type>( pMatrix.m[i] );
}

template< class Type > 
void 
Matrix<Type>::setRow(unsigned int pRow, Type pValue) throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int i = 0, index = pRow * mDim.y; i < mDim.y; ++i, ++index) m[index] = pValue;
}

template< class Type > 
void 
Matrix<Type>::setRow(unsigned int pRow, Type* pValues) throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Type* rowValues = &(m[pRow * mDim.y]);
		for(unsigned int i=0; i<mDim.y; ++i) rowValues[i] = pValues[i];
}

template< class Type > 
template< class Type2 >
void 
Matrix<Type>::setRow(unsigned int pRow, Type2* pValues) throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Type* rowValues = &(m[pRow * mDim.y]);
	for(unsigned int i=0; i<mDim.y; ++i) rowValues[i] = static_cast<Type>( pValues[i] );
}

template< class Type > 
void 
Matrix<Type>::setRow(unsigned int pRow, const Vector<Type>& pVector) throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pVector.dim() != mDim.y ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( mDim.y ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Type* rowValues = &(m[pRow * mDim.y]);
	const Type* vecValues = pVector.values();
	for(unsigned int i=0; i<mDim.y; ++i) rowValues[i] = vecValues[i];	
}

template< class Type >
template< class Type2 > 
void 
Matrix<Type>::setRow(unsigned int pRow, const Vector<Type2>& pVector) throw (MathException)
{
	#ifdef _math_debug_
	if( pRow >= mDim.x ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pRow ).arg( mDim.x  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pVector.dim() != mDim.y ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( mDim.y ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Type* rowValues = &(m[pRow * mDim.y]);
	const Type2* vecValues = pVector.values();
	for(unsigned int i=0; i<mDim.y; ++i) rowValues[i] = static_cast<Type>( vecValues[i] );
}

template< class Type > 
void 
Matrix<Type>::setColumn(unsigned int pColumn, Type pValue) throw (MathException)
{
	#ifdef _math_debug_
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	for(unsigned int row = 0, index = pColumn; row < mDim.x; ++row, index += mDim.y) m[index] = pValue;
}

template< class Type > 
void 
Matrix<Type>::setColumn(unsigned int pColumn, Type* pValues) throw (MathException)
{
	#ifdef _math_debug_
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int row = 0, index = pColumn; row < mDim.x; ++row, index += mDim.y) m[index] = pValues[row];
}

template< class Type >
template< class Type2 > 
void 
Matrix<Type>::setColumn(unsigned int pColumn, Type2* pValues) throw (MathException)
{
	#ifdef _math_debug_
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("column %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int row = 0, index = pColumn; row < mDim.x; ++row, index += mDim.y) m[index] = static_cast<Type>( pValues[row] );
}

template< class Type > 
void 
Matrix<Type>::setColumn(unsigned int pColumn, const Vector<Type>& pVector) throw (MathException)
{
	#ifdef _math_debug_
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pVector.dim() != mDim.x ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( mDim.x ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	const Type* vecValues = pVector.values();
		
	for(unsigned int row = 0, index = pColumn; row < mDim.x; ++row, index += mDim.y) m[index] = vecValues[row];
}

template< class Type > 
template< class Type2 >
void 
Matrix<Type>::setColumn(unsigned int pColumn, const Vector<Type2>& pVector) throw (MathException)
{
	#ifdef _math_debug_
	if( pColumn >= mDim.y ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("row %1 out of bounds. max: %2").arg( pColumn ).arg( mDim.y  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	if( pVector.dim() != mDim.x ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("wrong vector dimension. expected: %1 supplied: %2").arg( mDim.x ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	const Type* vecValues = pVector.values();
		
	for(unsigned int row = 0, index = pColumn; row < mDim.x; ++row, index += mDim.y) m[index] = static_cast<Type>( vecValues[row] );
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator=(Type pValue)
{
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValue;
	
	return *this;
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator=(Type* pValues)
{
	for(unsigned int i=0; i<mCount; ++i) m[i] = pValues[i];
	
	return *this;
}

template< class Type > 
template< class Type2 >
const Matrix<Type>& 
Matrix<Type>::operator=(Type2* pValues)
{
	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type>( pValues[i] );
	
	return *this;
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator=(const Matrix<Type>& pMatrix) throw (MathException)
{ 
	#ifdef _math_debug_
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int i=0; i<mCount; ++i) m[i] = pMatrix.m[i];
	
	return *this;
}

template< class Type > 
template< class Type2 > 
const Matrix<Type>& 
Matrix<Type>::operator=(const Matrix<Type2>& pMatrix) throw (MathException) 
{ 
	#ifdef _math_debug_
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type>( pMatrix.m[i] );
	
	return *this;
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator+=( Type  pValue )
{
	for(unsigned int i=0; i<mCount; ++i) m[i] += pValue;
	
	return (*this);
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator-=( Type  pValue )
{
	for(unsigned int i=0; i<mCount; ++i) m[i] -= pValue;
	
	return (*this);
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator*=( Type  pValue )
{
	for(unsigned int i=0; i<mCount; ++i) m[i] *= pValue;
	
	return (*this);
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator/=( Type  pValue )
{
	for(unsigned int i=0; i<mCount; ++i) m[i] /= pValue;
	
	return (*this);
}

template< class Type > 	
Matrix<Type> 
Matrix<Type>::operator+( Type  pValue ) const
{
	Matrix<Type> matrix(*this);
	matrix += pValue;
	
	return matrix;
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::operator-( Type  pValue ) const
{
	Matrix<Type> matrix(*this);
	matrix -= pValue;
	
	return matrix;
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::operator*( Type  pValue ) const
{
	Matrix<Type> matrix(*this);
	matrix *= pValue;
	
	return matrix;
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::operator/( Type  pValue ) const
{
	Matrix<Type> matrix(*this);
	matrix /= pValue;
	
	return matrix;
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator+=( const Matrix<Type>& pMatrix ) throw (MathException)
{
	#ifdef _math_debug_
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int i=0; i<mCount; ++i) m[i] += pMatrix.m[i];
	
	return (*this);
}

template< class Type > 
template< class Type2 >
const Matrix<Type>& 
Matrix<Type>::operator+=( const Matrix<Type2>& pMatrix ) throw (MathException)
{
	#ifdef _math_debug_
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	for(unsigned int i=0; i<mCount; ++i) m[i] += static_cast<Type>( pMatrix.m[i] );
	
	return (*this);
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator-=( const Matrix<Type>& pMatrix ) throw (MathException)
{
	#ifdef _math_debug_
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int i=0; i<mCount; ++i) m[i] -= pMatrix.m[i];
	
	return (*this);
}

template< class Type > 
template< class Type2 >
const Matrix<Type>& 
Matrix<Type>::operator-=( const Matrix<Type2>& pMatrix ) throw (MathException)
{
	#ifdef _math_debug_
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	for(unsigned int i=0; i<mCount; ++i) m[i] -= static_cast<Type>( pMatrix.m[i] );
	
	return (*this);
}

template< class Type > 
const Matrix<Type>& 
Matrix<Type>::operator*=( const Matrix<Type>& pMatrix ) throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	const Matrix<Type> aMatrix( *this );
	const Matrix<Type>& bMatrix = pMatrix;
	unsigned int dim = Matrix<Type>::mDim.x;
	
	(*this) = 0.0;
	
	for(unsigned int cRow=0, cIndex=0; cRow < dim; ++cRow)
	{
		for(unsigned int cCol=0; cCol < dim; ++cCol, ++cIndex)
		{
			for(unsigned int aCol=0, aIndex = cRow * dim, bIndex = cCol; aCol < dim; ++aCol, ++aIndex, bIndex += dim)
			{
				m[cIndex] += aMatrix.m[aIndex] * bMatrix.m[bIndex]; 
			}
		}
	}
	
	return (*this);
}

template< class Type > 
template< class Type2 > 
const Matrix<Type>& 
Matrix<Type>::operator*=( const Matrix<Type2>& pMatrix ) throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.x != mDim.y ) throw MathException( "ERR_MATRIX_ERROR", base::String("matrix not square: rows %1 != columns %2").arg( mDim.x ).arg( mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
//	if( mDim.x != pMatrix.mDim.y || mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	if( mDim.x != pMatrix.mDim.x || mDim.y != pMatrix.mDim.y ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 columns: %2 supplied rows: %3 columns %4").arg( mDim.x ).arg( mDim.y ).arg( pMatrix.mDim.x ).arg( pMatrix.mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	const Matrix<Type> aMatrix( *this );
	const Matrix<Type2>& bMatrix = pMatrix;
	unsigned int dim = Matrix<Type>::mDim.x;
	
	(*this) = 0.0;
	
	for(unsigned int cRow=0, cIndex=0; cRow < dim; ++cRow)
	{
		for(unsigned int cCol=0; cCol < dim; ++cCol, ++cIndex)
		{
			for(unsigned int aCol=0, aIndex = cRow * dim, bIndex = cCol; aCol < dim; ++aCol, ++aIndex, bIndex += dim)
			{
				m[cIndex] += aMatrix.m[aIndex] * static_cast<Type>( bMatrix.m[bIndex] ); 
			}
		}
	}
	
	return (*this);
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::operator+( const Matrix<Type>& pMatrix ) const throw (MathException)
{
	try
	{
		Matrix<Type> matrix(*this);
		matrix += pMatrix;
	
		return matrix;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template< class Type > 
template< class Type2 > 
Matrix<Type> 
Matrix<Type>::operator+( const Matrix<Type2>& pMatrix ) const throw (MathException)
{
	try
	{
		Matrix<Type> matrix(*this);
		matrix += pMatrix;
	
		return matrix;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::operator-( const Matrix<Type>& pMatrix ) const throw (MathException)
{
	try
	{
		Matrix<Type> matrix(*this);
		matrix -= pMatrix;
		
		return matrix;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template< class Type > 
template< class Type2 >
Matrix<Type> 
Matrix<Type>::operator-( const Matrix<Type2>& pMatrix ) const throw (MathException)
{
	try
	{
		Matrix<Type> matrix(*this);
		matrix -= pMatrix;
		
		return matrix;
	}
	catch(MathException& e)
	{
		throw;
	}	
}

template< class Type > 
Matrix<Type> 
Matrix<Type>::operator*( const Matrix<Type>& pMatrix ) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 supplied rows: %2").arg( mDim.y ).arg( pMatrix.mDim.x ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	const Matrix<Type>& aMatrix = *this;
	const Matrix<Type>& bMatrix = pMatrix;
	const Vector2<unsigned int>& aDim = aMatrix.mDim;
	const Vector2<unsigned int>& bDim = bMatrix.mDim;
	Matrix<Type> cMatrix(aDim.x, bDim.y);
	const Vector2<unsigned int>& cDim = cMatrix.mDim;
	
	for(unsigned int cRow=0, cIndex=0; cRow < cDim.x; ++cRow)
	{
		for(unsigned int cCol=0; cCol < cDim.y; ++cCol, ++cIndex)
		{
			for(unsigned int aCol=0, aIndex = cRow * aDim.y, bIndex = cCol; aCol < aDim.y; ++aCol, ++aIndex, bIndex += bDim.y)
			{
				cMatrix.m[cIndex] += aMatrix.m[aIndex] * bMatrix.m[bIndex]; 
			}
		}
	}
	
	return cMatrix;
}

template< class Type > 
template< class Type2 > 
Matrix<Type> 
Matrix<Type>::operator*( const Matrix<Type2>& pMatrix ) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.y != pMatrix.mDim.x ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("expected rows: %1 supplied rows: %2").arg( mDim.y ).arg( pMatrix.mDim.x ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	const Matrix<Type>& aMatrix = *this;
	const Matrix<Type2>& bMatrix = pMatrix;
	const Vector2<unsigned int>& aDim = aMatrix.mDim;
	const Vector2<unsigned int>& bDim = bMatrix.mDim;
	Matrix<Type> cMatrix(aDim.x, bDim.y);
	const Vector2<unsigned int>& cDim = cMatrix.mDim;
	
	for(unsigned int cRow=0, cIndex=0; cRow < cDim.x; ++cRow)
	{
		for(unsigned int cCol=0; cCol < cDim.y; ++cCol, ++cIndex)
		{
			for(unsigned int aCol=0, aIndex = cRow * aDim.y, bIndex = cCol; aCol < aDim.y; ++aCol, ++aIndex, bIndex += bDim.y)
			{
				cMatrix.m[cIndex] += aMatrix.m[aIndex] * static_cast<Type>( bMatrix.m[bIndex] ); 
			}
		}
	}
	
	return cMatrix;
}

template< class Type > 
Vector<Type> 
Matrix<Type>::operator*( const Vector<Type>& pVector ) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.y != pVector.dim() ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("expected vector dimension: %1 supplied dimension: %2").arg( mDim.y ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Vector<Type> outVector(mDim.x);
	
	for(unsigned int row = 0, mIndex = 0; row < mDim.x; ++row)
	{
		for(unsigned int column = 0; column < mDim.y; ++column, ++mIndex)
		{
			outVector[row] += m[mIndex] * pVector[column];
		}
	}
	
	return outVector;	
}

template< class Type > 
template< class Type2 >
Vector<Type> 
Matrix<Type>::operator*( const Vector<Type2>& pVector ) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim.y != pVector.dim() ) throw MathException( "ERR_VECTOR_MATRIX_MISMATCH", base::String("expected vector dimension: %1 supplied dimension: %2").arg( mDim.y ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Vector<Type> outVector(mDim.x);
	
	for(unsigned int row = 0, mIndex = 0; row < mDim.x; ++row)
	{
		for(unsigned int column = 0; column < mDim.y; ++column, ++mIndex)
		{
			outVector[row] += m[mIndex] * static_cast<Type>( pVector[column] );
		}
	}
	
	return outVector;	
}

template< class Type >
void 
Matrix<Type>::reset()
{
	for(unsigned int i=0; i<mCount; ++i) m[i] = static_cast<Type>( 0 );
}

template< class Type >
base::String
Matrix<Type>::info() const
{
	std::ostringstream os;
	const std::ios::fmtflags flags = os.flags();
	const int prec  = os.precision();

	os.setf( std::ios::right, std::ios::adjustfield );
	os.precision( 5 );
	
	for(unsigned int row = 0, index = 0; row < mDim.x; ++row)
	{
		os << "|";
		
		for(unsigned int col = 0; col < mDim.y; ++col, ++index)
		{
			os << std::setw(10) << m[index] << "|";
		}
		
		os << std::endl;
	}
	
	os.precision( prec );
	os.setf( flags );
	
	std::string stdString = os.str();
	//stdString << os;
	
	base::String infoString(stdString.c_str());
	
	return infoString;
}

}

}

#endif