/**	\file iso_math_matrix3.h
 *	\class iso::math::Matrix3 3x3 matrix class
 *	\brief 3x3 matrix class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef _iso_math_matrix3_h_
 #define _iso_math_matrix3_h_
 
 #include "iso_math_vector.h"
 #include "iso_math_vector3.h"
 #include "iso_math_matrix.h"
 
 namespace iso
 {
 
 namespace math
 {
 
 template< class Type> 
 class Matrix3 : public Matrix<Type>
 {
 public:
	/**
	\brief zero matrix
	*/
	static const Matrix3<Type> ZERO;

	/**
	\brief identity matrix
	*/
	static const Matrix3<Type> IDENTITY;
 
	/**
	\brief create zero matrix
	*/
	Matrix3();

	/**
	\brief create matrix containing identical values
	\param pValue value
	*/	
	Matrix3(Type pValue);

	/**
	\brief create matrix
	\param pValues values (must be of correct size!)
	*/	
	Matrix3(Type* pValues);
	
	/**
	\brief create matrix
	\param pValues values (must be of correct size!)
	*/	
	template< class Type2 > Matrix3(Type2* pValues);	

	/**
	\brief create matrix
	\param p00 value
	\param p01 value
	\param p02 value
	\param p10 value
	\param p11 value
	\param p12 value
	\param p20 value
	\param p21 value
	\param p22 value
	*/	
    Matrix3( Type p00, Type p01, Type p02, Type p10, Type p11, Type p12, Type p20, Type p21, Type p22 );
 
 	/**
	\brief create matrix
	\param pV0 vector
	\param pV1 vector
	\param pV2 vector
	\param pColumnMode true: vectors represent matrix columns false: vectors represent matrix rows
	*/	
	Matrix3( const Vector3<Type>& pV0, const Vector3<Type>& pV1, const Vector3<Type>& pV2, bool pColumnMode = false );

 	/**
	\brief create matrix
	\param pV0 vector
	\param pV1 vector
	\param pV2 vector
	\param pColumnMode true: vectors represent matrix columns false: vectors represent matrix rows
	*/	
	template< class Type2 > Matrix3( const Vector3<Type2>& pV0, const Vector3<Type2>& pV1, const Vector3<Type2>& pV2, bool pColumnMode = false );
	
	/**
	\brief copy constructor
	\param pMatrix matrix
	*/	
	Matrix3(const Matrix3<Type>& pMatrix);

	/**
	\brief copy constructor
	\param pMatrix matrix
	*/	
	template< class Type2 > Matrix3(const Matrix3<Type2>& pMatrix);
	
	/**
	\brief copy constructor
	\param pMatrix matrix
	\exception MathException not 3x3 matrix
	*/	
	Matrix3(const Matrix<Type>& pMatrix) throw (MathException);

	/**
	\brief copy constructor
	\param pMatrix matrix
	\exception MathException not 3x3 matrix
	*/	
	template< class Type2 > Matrix3(const Matrix<Type2>& pMatrix) throw (MathException);

	/**
	\brief destructor
	*/
	virtual ~Matrix3();

	/**
	\brief set matrix values
	\param p00 value
	\param p01 value
	\param p02 value
	\param p10 value
	\param p11 value
	\param p12 value
	\param p20 value
	\param p21 value
	\param p22 value
	*/		
	void set( Type p00, Type p01, Type p02, Type p10, Type p11, Type p12, Type p20, Type p21, Type p22 );

 	/**
	\brief set matrix values
	\param pV0 vector
	\param pV1 vector
	\param pV2 vector
	\param pColumnMode true: vectors represent matrix columns false: vectors represent matrix rows
	*/		
	void set( const Vector3<Type>& pV0, const Vector3<Type>& pV1, const Vector3<Type>& pV2, bool pColumnMode = false ); 

	/**
	\brief set matrix values
	\param pV0 vector
	\param pV1 vector
	\param pV2 vector
	\param pColumnMode true: vectors represent matrix columns false: vectors represent matrix rows
	*/		
	template< class Type2 > void set( const Vector3<Type2>& pV0, const Vector3<Type2>& pV1, const Vector3<Type2>& pV2, bool pColumnMode = false ); 
 
 	/**
	\brief print matrix information
	\param pOstream output stream
	\param pMatrix matrix
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const Matrix3<Type>& pMatrix)
    {
		pOstream << pMatrix.info().toStdString();
		return pOstream;
    }; 
 }; 
 
 typedef Matrix3<int>    Matrix3I;
 typedef Matrix3<float>  Matrix3F;
 typedef Matrix3<double> Matrix3D;
 
 template<class Type>
 const Matrix3<Type> 
 Matrix3<Type>::ZERO(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

 template<class Type>
 const Matrix3<Type> 
 Matrix3<Type>::IDENTITY(1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0);
 
 template<class Type>
 Matrix3<Type>::Matrix3()
	: Matrix<Type>(3, 3)
 {}
 
 template<class Type>
 Matrix3<Type>::Matrix3(Type pValue)
	: Matrix<Type>(3, 3, pValue)
 {}
 
 template<class Type>
 Matrix3<Type>::Matrix3(Type* pValues)
	: Matrix<Type>(3, 3, pValues)
 {}
 
 template<class Type>
 template<class Type2>
 Matrix3<Type>::Matrix3(Type2* pValues)
	: Matrix<Type>(3, 3, pValues)
 {} 
 
 template<class Type>
 Matrix3<Type>::Matrix3( Type p00, Type p01, Type p02, Type p10, Type p11, Type p12, Type p20, Type p21, Type p22 )
	: Matrix<Type>(3, 3)
 {
	Matrix<Type>::m[0] = p00;
	Matrix<Type>::m[1] = p01;
	Matrix<Type>::m[2] = p02;
	Matrix<Type>::m[3] = p10;
	Matrix<Type>::m[4] = p11;
	Matrix<Type>::m[5] = p12;
	Matrix<Type>::m[6] = p20;
	Matrix<Type>::m[7] = p21;
	Matrix<Type>::m[8] = p22;
 }

 template<class Type>
 Matrix3<Type>::Matrix3( const Vector3<Type>& pV0, const Vector3<Type>& pV1, const Vector3<Type>& pV2, bool pColumnMode )
	: Matrix<Type>(3, 3)
 {
	if( pColumnMode == false )
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV0[1];
		Matrix<Type>::m[2] = pV0[2];
		Matrix<Type>::m[3] = pV1[0];
		Matrix<Type>::m[4] = pV1[1];
		Matrix<Type>::m[5] = pV1[2];
		Matrix<Type>::m[6] = pV2[0];
		Matrix<Type>::m[7] = pV2[1];
		Matrix<Type>::m[8] = pV2[2];
	}
	else
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV1[0];
		Matrix<Type>::m[2] = pV2[0];
		Matrix<Type>::m[3] = pV0[1];
		Matrix<Type>::m[4] = pV1[1];
		Matrix<Type>::m[5] = pV2[1];
		Matrix<Type>::m[6] = pV0[2];
		Matrix<Type>::m[7] = pV1[2];
		Matrix<Type>::m[8] = pV2[2];	
	}
 }
 
 template<class Type>
 template<class Type2>
 Matrix3<Type>::Matrix3( const Vector3<Type2>& pV0, const Vector3<Type2>& pV1, const Vector3<Type2>& pV2, bool pColumnMode )
	: Matrix<Type>(3, 3)
 {
	if( pColumnMode == false )
	{
		Matrix<Type>::m[0] = static_cast<Type>( pV0[0] );
		Matrix<Type>::m[1] = static_cast<Type>( pV0[1] );
		Matrix<Type>::m[2] = static_cast<Type>( pV0[2] );
		Matrix<Type>::m[3] = static_cast<Type>( pV1[0] );
		Matrix<Type>::m[4] = static_cast<Type>( pV1[1] );
		Matrix<Type>::m[5] = static_cast<Type>( pV1[2] );
		Matrix<Type>::m[6] = static_cast<Type>( pV2[0] );
		Matrix<Type>::m[7] = static_cast<Type>( pV2[1] );
		Matrix<Type>::m[8] = static_cast<Type>( pV2[2] );
	}
	else
	{
		Matrix<Type>::m[0] = static_cast<Type>( pV0[0] );
		Matrix<Type>::m[1] = static_cast<Type>( pV1[0] );
		Matrix<Type>::m[2] = static_cast<Type>( pV2[0] );
		Matrix<Type>::m[3] = static_cast<Type>( pV0[1] );
		Matrix<Type>::m[4] = static_cast<Type>( pV1[1] );
		Matrix<Type>::m[5] = static_cast<Type>( pV2[1] );
		Matrix<Type>::m[6] = static_cast<Type>( pV0[2] );
		Matrix<Type>::m[7] = static_cast<Type>( pV1[2] );
		Matrix<Type>::m[8] = static_cast<Type>( pV2[2] );	
	}
 }

 template<class Type>	
 Matrix3<Type>::Matrix3(const Matrix3<Type>& pMatrix)
	: Matrix<Type>(pMatrix)
 {}
 
 template<class Type>
 template<class Type2>	
 Matrix3<Type>::Matrix3(const Matrix3<Type2>& pMatrix)
	: Matrix<Type>(pMatrix)
 {}
 
 template<class Type>	
 Matrix3<Type>::Matrix3(const Matrix<Type>& pMatrix) throw (MathException)
	: Matrix<Type>(pMatrix)
 {
	#ifdef _math_debug_
	if( Matrix<Type>::mDim.x != Matrix<Type>::mDim.y || Matrix<Type>::mDim.x != 3 ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("matrix not 3x3. rows: %1 columns: %2 ").arg( Matrix<Type>::mDim.x ).arg( Matrix<Type>::mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
 }
 
 template<class Type>	
 template<class Type2>	
 Matrix3<Type>::Matrix3(const Matrix<Type2>& pMatrix) throw (MathException)
	: Matrix<Type>(pMatrix)
 {
	#ifdef _math_debug_
	if( Matrix<Type>::mDim.x != Matrix<Type>::mDim.y || Matrix<Type>::mDim.x != 3 ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("matrix not 3x3. rows: %1 columns: %2 ").arg( Matrix<Type>::mDim.x ).arg( Matrix<Type>::mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
 } 
 
 template<class Type>
 Matrix3<Type>::~Matrix3()
 {}
 
 template<class Type>
 void 
 Matrix3<Type>::set( Type p00, Type p01, Type p02, Type p10, Type p11, Type p12, Type p20, Type p21, Type p22 )
 {
 	Matrix<Type>::m[0] = p00;
	Matrix<Type>::m[1] = p01;
	Matrix<Type>::m[2] = p02;
	Matrix<Type>::m[3] = p10;
	Matrix<Type>::m[4] = p11;
	Matrix<Type>::m[5] = p12;
	Matrix<Type>::m[6] = p20;
	Matrix<Type>::m[7] = p21;
	Matrix<Type>::m[8] = p22;
 }

 template<class Type>
 void 
 Matrix3<Type>::set( const Vector3<Type>& pV0, const Vector3<Type>& pV1, const Vector3<Type>& pV2, bool pColumnMode )
 {
 	if( pColumnMode == false )
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV0[1];
		Matrix<Type>::m[2] = pV0[2];
		Matrix<Type>::m[3] = pV1[0];
		Matrix<Type>::m[4] = pV1[1];
		Matrix<Type>::m[5] = pV1[2];
		Matrix<Type>::m[6] = pV2[0];
		Matrix<Type>::m[7] = pV2[1];
		Matrix<Type>::m[8] = pV2[2];
	}
	else
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV1[0];
		Matrix<Type>::m[2] = pV2[0];
		Matrix<Type>::m[3] = pV0[1];
		Matrix<Type>::m[4] = pV1[1];
		Matrix<Type>::m[5] = pV2[1];
		Matrix<Type>::m[6] = pV0[2];
		Matrix<Type>::m[7] = pV1[2];
		Matrix<Type>::m[8] = pV2[2];	
	}
 }
 
 template<class Type>
 template<class Type2>
 void 
 Matrix3<Type>::set( const Vector3<Type2>& pV0, const Vector3<Type2>& pV1, const Vector3<Type2>& pV2, bool pColumnMode )
 {
 	if( pColumnMode == false )
	{
		Matrix<Type>::m[0] = static_cast<Type>( pV0[0] );
		Matrix<Type>::m[1] = static_cast<Type>( pV0[1] );
		Matrix<Type>::m[2] = static_cast<Type>( pV0[2] );
		Matrix<Type>::m[3] = static_cast<Type>( pV1[0] );
		Matrix<Type>::m[4] = static_cast<Type>( pV1[1] );
		Matrix<Type>::m[5] = static_cast<Type>( pV1[2] );
		Matrix<Type>::m[6] = static_cast<Type>( pV2[0] );
		Matrix<Type>::m[7] = static_cast<Type>( pV2[1] );
		Matrix<Type>::m[8] = static_cast<Type>( pV2[2] );
	}
	else
	{
		Matrix<Type>::m[0] = static_cast<Type>( pV0[0] );
		Matrix<Type>::m[1] = static_cast<Type>( pV1[0] );
		Matrix<Type>::m[2] = static_cast<Type>( pV2[0] );
		Matrix<Type>::m[3] = static_cast<Type>( pV0[1] );
		Matrix<Type>::m[4] = static_cast<Type>( pV1[1] );
		Matrix<Type>::m[5] = static_cast<Type>( pV2[1] );
		Matrix<Type>::m[6] = static_cast<Type>( pV0[2] );
		Matrix<Type>::m[7] = static_cast<Type>( pV1[2] );
		Matrix<Type>::m[8] = static_cast<Type>( pV2[2] );	
	}
 } 
 
 }
 
 }
 
 #endif