/**	\file iso_math_matrix4.h
 *	\class iso::math::Matrix4 4x4 matrix class
 *	\brief 4x4 matrix class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

 #ifndef _iso_math_matrix4_h_
 #define _iso_math_matrix4_h_
 
 #include "iso_math_vector.h"
 #include "iso_math_vector4.h"
 #include "iso_math_matrix.h"
 
 namespace iso
 {
 
 namespace math
 {
 
 template< class Type> 
 class Matrix4 : public Matrix<Type>
 {
 public:
 	/**
	\brief zero matrix
	*/
	static const Matrix4<Type> ZERO;

	/**
	\brief identity matrix
	*/
	static const Matrix4<Type> IDENTITY;
 
	/**
	\brief create zero matrix
	*/
	Matrix4();

	/**
	\brief create matrix containing identical values
	\param pValue value
	*/	
	Matrix4(Type pValue);

	/**
	\brief create matrix
	\param pValues values (must be of correct size!)
	*/	
	Matrix4(Type* pValues);

	/**
	\brief create matrix
	\param p00 value
	\param p01 value
	\param p02 value
	\param p03 value
	\param p10 value
	\param p11 value
	\param p12 value
	\param p13 value
	\param p20 value
	\param p21 value
	\param p22 value
	\param p23 value
	\param p30 value
	\param p31 value
	\param p32 value
	\param p33 value
	*/	
    Matrix4( Type p00, Type p01, Type p02, Type p03, Type p10, Type p11, Type p12, Type p13, Type p20, Type p21, Type p22, Type p23, Type p30, Type p31, Type p32, Type p33 );
 
 	/**
	\brief create matrix
	\param pV0 vector
	\param pV1 vector
	\param pV2 vector
	\param pV3 vector
	\param pColumnMode true: vectors represent matrix columns false: vectors represent matrix rows
	*/	
	Matrix4( const Vector4<Type>& pV0, const Vector4<Type>& pV1, const Vector4<Type>& pV2, const Vector4<Type>& pV3, bool pColumnMode = false );
	
	/**
	\brief copy constructor
	\param pMatrix matrix
	*/	
	Matrix4(const Matrix4<Type>& pMatrix);

	/**
	\brief copy constructor
	\param pMatrix matrix
	*/	
	template< class Type2 > Matrix4(const Matrix4<Type2>& pMatrix);
	
	/**
	\brief copy constructor
	\param pMatrix matrix
	\exception MathException pMatrix not a 4x4 matrix
	*/	
	Matrix4(const Matrix<Type>& pMatrix) throw (MathException);

	/**
	\brief destructor
	*/
	virtual ~Matrix4();

	/**
	\brief set matrix values
	\param p00 value
	\param p01 value
	\param p02 value
	\param p03 value
	\param p10 value
	\param p11 value
	\param p12 value
	\param p13 value
	\param p20 value
	\param p21 value
	\param p22 value
	\param p23 value
	\param p30 value
	\param p31 value
	\param p32 value
	\param p33 value
	*/		
	void set( Type p00, Type p01, Type p02, Type p03, Type p10, Type p11, Type p12, Type p13, Type p20, Type p21, Type p22, Type p23, Type p30, Type p31, Type p32, Type p33 );

 	/**
	\brief set matrix values
	\param pV0 vector
	\param pV1 vector
	\param pV2 vector
	\param pV3 vector
	\param pColumnMode true: vectors represent matrix columns false: vectors represent matrix rows
	*/		
	void set( const Vector4<Type>& pV0, const Vector4<Type>& pV1, const Vector4<Type>& pV2, const Vector4<Type>& pV3, bool pColumnMode = false ); 

	/**
	\brief rotate around x axis
	\param pAngle angle of rotation
	*/
    void rotateX( const Type pAngle );

	/**
	\brief rotate around y axis
	\param pAngle angle of rotation
	*/
    void rotateY( const Type pAngle );

	/**
	\brief rotate around z axis
	\param pAngle angle of rotation
	*/
    void rotateZ( const Type pAngle );

	/**
	\brief rotate around arbitrary axis
	\param pAngle angle of rotation
	\param pAxis axis of rotation
	*/
	void rotate( Type pAngle, const Vector3<Type>& pAxis );

	/**
	\brief change non uniform scale
	\param pScale non uniform scale
	*/
    void scale( const Type pScale[3] );

	/**
	\brief change non uniform scale
	\param pScaleX scale along x axis
	\param pScaleY scale along y axis
	\param pScaleZ scale along z axis
	*/
    void scale( const Type pScaleX, const Type pScaleY, const Type pScaleZ );

	/**
	\brief change non uniform scale
	\param pScale non uniform scale
	*/
    void scale( const Vector3< Type >& pScale );
	
	/**
	\brief change translation
	\param pTransX translation along x axis
	\param pTransY translation along y axis
	\param pTransZ translation along z axis
	*/	
    void translate( const Type pTransX, const Type pTransY, const Type pTransZ );

	/**
	\brief change translation
	\param pTrans translation
	*/	
    void translate( const Type pTrans[3] );

	/**
	\brief change translation
	\param pTrans translation
	*/	
    void translate( const Vector3<Type>& pTrans );

	/**
	\brief set rotation around x axis
	\param pAngle angle of rotation
	*/
    void setRotationX( const Type pAngle );

	/**
	\brief set rotation around y axis
	\param pAngle angle of rotation
	*/
    void setRotationY( const Type pAngle );

	/**
	\brief set rotation around z axis
	\param pAngle angle of rotation
	*/
    void setRotationZ( const Type pAngle );

	/**
	\brief set rotation around arbitrary axis
	\param pAngle angle of rotation
	\param pAxis axis of rotation
	*/
	void setRotation( Type pAngle, const Vector3<Type>& pAxis );

	/**
	\brief set non uniform scale
	\param pScale non uniform scale
	*/
    void setScale( const Type pScale[3] );

	/**
	\brief set non uniform scale
	\param pScaleX scale along x axis
	\param pScaleY scale along y axis
	\param pScaleZ scale along z axis
	*/
    void setScale( const Type pScaleX, const Type pScaleY, const Type pScaleZ );

	/**
	\brief set non uniform scale
	\param pScale non uniform scale
	*/
    void setScale( const Vector3< Type >& pScale );

	/**
	\brief set translation
	\param pTransX translation along x axis
	\param pTransY translation along y axis
	\param pTransZ translation along z axis
	*/	
    void setTranslation( const Type pTransX, const Type pTransY, const Type pTransZ );

	/**
	\brief set translation
	\param pTrans translation
	*/	
    void setTranslation( const Type pTrans[3] );

	/**
	\brief set translation
	\param pTrans translation
	*/	
    void setTranslation( const Vector3<Type>& pTrans );
 }; 
 
 typedef Matrix4<int>    Matrix4I;
 typedef Matrix4<float>  Matrix4F;
 typedef Matrix4<double> Matrix4D;
 
 template<class Type>
 const Matrix4<Type> 
 Matrix4<Type>::ZERO(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

 template<class Type>
 const Matrix4<Type> 
 Matrix4<Type>::IDENTITY(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0);
 
 template<class Type>
 Matrix4<Type>::Matrix4()
	: Matrix<Type>(4, 4)
 {}
 
 template<class Type>
 Matrix4<Type>::Matrix4(Type pValue)
	: Matrix<Type>(4, 4, pValue)
 {}
 
 template<class Type>
 Matrix4<Type>::Matrix4(Type* pValues)
	: Matrix<Type>(4, 4, pValues)
 {}
 
 template<class Type>
 Matrix4<Type>::Matrix4( Type p00, Type p01, Type p02, Type p03, Type p10, Type p11, Type p12, Type p13, Type p20, Type p21, Type p22, Type p23, Type p30, Type p31, Type p32, Type p33 )
	: Matrix<Type>(4, 4)
 {
	Matrix<Type>::m[0] = p00;
	Matrix<Type>::m[1] = p01;
	Matrix<Type>::m[2] = p02;
	Matrix<Type>::m[3] = p03;
	Matrix<Type>::m[4] = p10;
	Matrix<Type>::m[5] = p11;
	Matrix<Type>::m[6] = p12;
	Matrix<Type>::m[7] = p13;
	Matrix<Type>::m[8] = p20;
	Matrix<Type>::m[9] = p21;
	Matrix<Type>::m[10] = p22;
	Matrix<Type>::m[11] = p23;
	Matrix<Type>::m[12] = p30;
	Matrix<Type>::m[13] = p31;
	Matrix<Type>::m[14] = p32;
	Matrix<Type>::m[15] = p33;
 }

 template<class Type>
 Matrix4<Type>::Matrix4( const Vector4<Type>& pV0, const Vector4<Type>& pV1, const Vector4<Type>& pV2, const Vector4<Type>& pV3, bool pColumnMode )
	: Matrix<Type>(4, 4)
 {
	if( pColumnMode == false )
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV0[1];
		Matrix<Type>::m[2] = pV0[2];
		Matrix<Type>::m[3] = pV0[3];
		Matrix<Type>::m[4] = pV1[0];
		Matrix<Type>::m[5] = pV1[1];
		Matrix<Type>::m[6] = pV1[2];
		Matrix<Type>::m[7] = pV1[3];
		Matrix<Type>::m[8] = pV2[0];
		Matrix<Type>::m[9] = pV2[1];
		Matrix<Type>::m[10] = pV2[2];
		Matrix<Type>::m[11] = pV2[3];
		Matrix<Type>::m[12] = pV3[0];
		Matrix<Type>::m[13] = pV3[1];
		Matrix<Type>::m[14] = pV3[2];
		Matrix<Type>::m[15] = pV3[3];
	}
	else
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV1[0];
		Matrix<Type>::m[2] = pV2[0];
		Matrix<Type>::m[3] = pV3[0];
		Matrix<Type>::m[4] = pV0[1];
		Matrix<Type>::m[5] = pV1[1];
		Matrix<Type>::m[6] = pV2[1];
		Matrix<Type>::m[7] = pV3[1];
		Matrix<Type>::m[8] = pV0[2];
		Matrix<Type>::m[9] = pV1[2];
		Matrix<Type>::m[10] = pV2[2];
		Matrix<Type>::m[11] = pV3[2];
		Matrix<Type>::m[12] = pV0[3];
		Matrix<Type>::m[13] = pV1[3];
		Matrix<Type>::m[14] = pV2[3];
		Matrix<Type>::m[15] = pV3[3];	
	}
 }

 template<class Type>	
 Matrix4<Type>::Matrix4(const Matrix4<Type>& pMatrix)
	: Matrix<Type>(pMatrix)
 {}
 
 template<class Type>
 template< class Type2 >
 Matrix4<Type>::Matrix4(const Matrix4<Type2>& pMatrix)
	: Matrix<Type>(pMatrix)
 {}

 template<class Type>	
 Matrix4<Type>::Matrix4(const Matrix<Type>& pMatrix) throw (MathException)
	: Matrix<Type>(pMatrix)
 {
 	#ifdef _math_debug_
	if( Matrix<Type>::mDim.x != Matrix<Type>::mDim.y || Matrix<Type>::mDim.x != 4 ) throw MathException( "ERR_MATRIX_MATRIX_MISMATCH", base::String("matrix not 4x4. rows: %1 columns: %2 ").arg( Matrix<Type>::mDim.x ).arg( Matrix<Type>::mDim.y ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
 }
 
 template<class Type>
 Matrix4<Type>::~Matrix4()
 {}
 
 template<class Type>
 void
 Matrix4<Type>::set( Type p00, Type p01, Type p02, Type p03, Type p10, Type p11, Type p12, Type p13, Type p20, Type p21, Type p22, Type p23, Type p30, Type p31, Type p32, Type p33 )
 {
	Matrix<Type>::m[0] = p00;
	Matrix<Type>::m[1] = p01;
	Matrix<Type>::m[2] = p02;
	Matrix<Type>::m[3] = p03;
	Matrix<Type>::m[4] = p10;
	Matrix<Type>::m[5] = p11;
	Matrix<Type>::m[6] = p12;
	Matrix<Type>::m[7] = p13;
	Matrix<Type>::m[8] = p20;
	Matrix<Type>::m[9] = p21;
	Matrix<Type>::m[10] = p22;
	Matrix<Type>::m[11] = p23;
	Matrix<Type>::m[12] = p30;
	Matrix<Type>::m[13] = p31;
	Matrix<Type>::m[14] = p32;
	Matrix<Type>::m[14] = p33;
 }

 template<class Type>
 void
 Matrix4<Type>::set( const Vector4<Type>& pV0, const Vector4<Type>& pV1, const Vector4<Type>& pV2, const Vector4<Type>& pV3, bool pColumnMode )
 {
	if( pColumnMode == false )
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV0[1];
		Matrix<Type>::m[2] = pV0[2];
		Matrix<Type>::m[3] = pV0[3];
		Matrix<Type>::m[4] = pV1[0];
		Matrix<Type>::m[5] = pV1[1];
		Matrix<Type>::m[6] = pV1[2];
		Matrix<Type>::m[7] = pV1[3];
		Matrix<Type>::m[8] = pV2[0];
		Matrix<Type>::m[9] = pV2[1];
		Matrix<Type>::m[10] = pV2[2];
		Matrix<Type>::m[11] = pV2[3];
		Matrix<Type>::m[12] = pV3[0];
		Matrix<Type>::m[13] = pV3[1];
		Matrix<Type>::m[14] = pV3[2];
		Matrix<Type>::m[15] = pV3[3];
	}
	else
	{
		Matrix<Type>::m[0] = pV0[0];
		Matrix<Type>::m[1] = pV1[0];
		Matrix<Type>::m[2] = pV2[0];
		Matrix<Type>::m[3] = pV3[0];
		Matrix<Type>::m[4] = pV0[1];
		Matrix<Type>::m[5] = pV1[1];
		Matrix<Type>::m[6] = pV2[1];
		Matrix<Type>::m[7] = pV3[1];
		Matrix<Type>::m[8] = pV0[2];
		Matrix<Type>::m[9] = pV1[2];
		Matrix<Type>::m[10] = pV2[2];
		Matrix<Type>::m[11] = pV3[2];
		Matrix<Type>::m[12] = pV0[3];
		Matrix<Type>::m[13] = pV1[3];
		Matrix<Type>::m[14] = pV2[3];
		Matrix<Type>::m[15] = pV3[3];	
	}
 }

 template<class Type>
 void 
 Matrix4<Type>::rotateX( const Type pAngle )
 {
   	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;

	Type c = cos(pAngle);
	Type s = sin(pAngle);
	
	Matrix<Type>::m[5] = c;
	Matrix<Type>::m[6] = s;
	Matrix<Type>::m[9] = -s;
	Matrix<Type>::m[10] = c;

	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::rotateY( const Type pAngle )
 {
	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;

	Type c = cos(pAngle);
	Type s = sin(pAngle);
	
    Matrix<Type>::m[0] = c;
    Matrix<Type>::m[2] = -s;
    Matrix<Type>::m[8] = s;
    Matrix<Type>::m[10] = c;

	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::rotateZ( const Type pAngle )
 {
 	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;

	Type c = cos(pAngle);
	Type s = sin(pAngle);
	
    Matrix<Type>::m[0] = c;
    Matrix<Type>::m[1] = s;
    Matrix<Type>::m[4] = -s;
    Matrix<Type>::m[5] = c;

	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::rotate( Type pAngle, const Vector3<Type>& pAxis )
 {
  	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;

	Type* a = pAxis.c;
	Type c = cos(pAngle);
	Type s = sin(pAngle);
	
	Matrix<Type>::m[0] = 1.0 + ( 1.0 - c ) * ( a[0] * a[0] - 1.0 );
	Matrix<Type>::m[1] = -a[2] * s + ( 1.0 - c ) * a[0] * a[1];
	Matrix<Type>::m[2] = a[1] * s + ( 1.0 - c ) * a[0] * a[2];
	Matrix<Type>::m[4] = a[2] * s + ( 1.0 - c ) * a[0] *a[1];
	Matrix<Type>::m[5] = 1.0 + ( 1.0 - c ) * ( a[1] * a[1] - 1.0 );
	Matrix<Type>::m[6] = -a[0] * s + ( 1.0 - c ) * a[1] * a[2];
	Matrix<Type>::m[8] = -a[1] * s + ( 1.0 - c ) * a[0] * a[2];
	Matrix<Type>::m[9] = a[0] * s + ( 1.0 - c ) * a[1] * a[2];
	Matrix<Type>::m[10] =	1.0 + ( 1.0 - c ) * ( a[2] * a[2] - 1.0 );

	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::scale( const Type pScale[3] )
 {
  	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;
	
	Matrix<Type>::m[0]  = scale[0];
    Matrix<Type>::m[5]  = scale[1];
    Matrix<Type>::m[10] = scale[2];

	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::scale( const Type pScaleX, const Type pScaleY, const Type pScaleZ )
 {
   	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;
	
	Matrix<Type>::m[0]  = pScaleX;
    Matrix<Type>::m[5]  = pScaleY;
    Matrix<Type>::m[10] = pScaleZ;

	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::scale( const Vector3< Type >& pScale )
 {
   	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;
	
	Matrix<Type>::m[0]  = pScale.x;
    Matrix<Type>::m[5]  = pScale.y;
    Matrix<Type>::m[10] = pScale.z;

	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::translate( const Type pTransX, const Type pTransY, const Type pTransZ )
 {
	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;
	
	Matrix<Type>::m[3] = pTransX;
	Matrix<Type>::m[7] = pTransY;
	Matrix<Type>::m[11] = pTransZ;
	
	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::translate( const Type pTrans[3] )
 {
 	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;
	
	Matrix<Type>::m[3] = pTrans[0];
	Matrix<Type>::m[7] = pTrans[1];
	Matrix<Type>::m[11] = pTrans[2];
	
	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::translate( const Vector3<Type>& pTrans )
 {
  	Matrix4<Type> tmpMatrix( *this );
	*this = Matrix4<Type>::IDENTITY;
	
	Matrix<Type>::m[3] = pTrans.x;
	Matrix<Type>::m[7] = pTrans.y;
	Matrix<Type>::m[11] = pTrans.z;
	
	(*this) *= tmpMatrix;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::setRotationX( const Type pAngle )
 {
	Type c = cos(pAngle);
	Type s = sin(pAngle);
	
	Matrix<Type>::m[5] = c;
	Matrix<Type>::m[6] = s;
	Matrix<Type>::m[9] = -s;
	Matrix<Type>::m[10] = c;
 }

 template<class Type>
 void 
 Matrix4<Type>::setRotationY( const Type pAngle )
 {
 	Type c = cos(pAngle);
	Type s = sin(pAngle);

    Matrix<Type>::m[0] = c;
    Matrix<Type>::m[2] = -s;
    Matrix<Type>::m[8] = s;
    Matrix<Type>::m[10] = c;
 }

 template<class Type>
 void 
 Matrix4<Type>::setRotationZ( const Type pAngle )
 {
 	Type c = cos(pAngle);
	Type s = sin(pAngle);

    Matrix<Type>::m[0] = c;
    Matrix<Type>::m[1] = s;
    Matrix<Type>::m[4] = -s;
    Matrix<Type>::m[5] = c;
 }

 template<class Type>
 void 
 Matrix4<Type>::setRotation( Type pAngle, const Vector3<Type>& pAxis )
 {
 	Type* a = pAxis.c;
	Type c = cos(pAngle);
	Type s = sin(pAngle);
	
	Matrix<Type>::m[0] = 1.0 + ( 1.0 - c ) * ( a[0] * a[0] - 1.0 );
	Matrix<Type>::m[1] = -a[2] * s + ( 1.0 - c ) * a[0] * a[1];
	Matrix<Type>::m[2] = a[1] * s + ( 1.0 - c ) * a[0] * a[2];
	Matrix<Type>::m[4] = a[2] * s + ( 1.0 - c ) * a[0] *a[1];
	Matrix<Type>::m[5] = 1.0 + ( 1.0 - c ) * ( a[1] * a[1] - 1.0 );
	Matrix<Type>::m[6] = -a[0] * s + ( 1.0 - c ) * a[1] * a[2];
	Matrix<Type>::m[8] = -a[1] * s + ( 1.0 - c ) * a[0] * a[2];
	Matrix<Type>::m[9] = a[0] * s + ( 1.0 - c ) * a[1] * a[2];
	Matrix<Type>::m[10] =	1.0 + ( 1.0 - c ) * ( a[2] * a[2] - 1.0 );
 }

 template<class Type>
 void 
 Matrix4<Type>::setScale( const Type pScale[3] )
 {
 	Matrix<Type>::m[0]  = scale[0];
    Matrix<Type>::m[5]  = scale[1];
    Matrix<Type>::m[10] = scale[2];
 }

 template<class Type>
 void 
 Matrix4<Type>::setScale( const Type pScaleX, const Type pScaleY, const Type pScaleZ )
 {
 	Matrix<Type>::m[0]  = pScaleX;
    Matrix<Type>::m[5]  = pScaleY;
    Matrix<Type>::m[10] = pScaleZ;
 }

 template<class Type>
 void 
 Matrix4<Type>::setScale( const Vector3< Type >& pScale )
 {
 	Matrix<Type>::m[0]  = pScale.x;
    Matrix<Type>::m[5]  = pScale.y;
    Matrix<Type>::m[10] = pScale.z;
 }
 
  template<class Type>
 void 
 Matrix4<Type>::setTranslation( const Type pTransX, const Type pTransY, const Type pTransZ )
 {
	 Matrix<Type>::m[3] = pTransX;
	 Matrix<Type>::m[7] = pTransY;
	 Matrix<Type>::m[11] = pTransZ;
 }
 
 template<class Type>
 void 
 Matrix4<Type>::setTranslation( const Type pTrans[3] )
 {
 	 Matrix<Type>::m[3] = pTrans[0];
	 Matrix<Type>::m[7] = pTrans[1];
	 Matrix<Type>::m[11] = pTrans[2];
 }
 
 template<class Type>
 void 
 Matrix4<Type>::setTranslation( const Vector3<Type>& pTrans )
 {
 	 Matrix<Type>::m[3] = pTrans.x;
	 Matrix<Type>::m[7] = pTrans.y;
	 Matrix<Type>::m[11] = pTrans.z;
 }
 
 }
 
 }
 
 #endif