/** \file iso_math_rectangle.h
 * \class iso::math::Rectangle template rectangle class
 * \brief template rectangle class
 *
 *  Created by Daniel Bisig on 5/10/07.
 */

#ifndef _iso_math_rectangle_h_
#define _iso_math_rectangle_h_

#include "iso_math_vector.h"

namespace iso
{

namespace math
{

template< class Type > 
class Rectangle
{
public:
	/**
	\brief create rectangle
	\param pDim dimension
	*/
	Rectangle(unsigned int pDim);
	
	/**
	\brief create rectangle
	\param pMinPos minimum position
	\param pMaxPos maximum position
	\exception MathException if dimension of minimum and maximum position differs or if maximum position is smaller than minimum position
	*/
	Rectangle(const Vector<Type>& pMinPos, const Vector<Type>& pMaxPos) throw (MathException);

	/**
	\brief create rectangle
	\param pRect rectangle
	*/
	Rectangle(const Rectangle<Type>& pRect);
	
	/**
	\brief destructor
	*/
	~Rectangle();

	/**
	\brief assignment operator
	\param pRect rectangle
	\return rectangle
	*/
	const Rectangle<Type>& operator=( const Rectangle<Type>& pRect ); 

	/**
	\brief return dimension
	\return dimension
	*/
	unsigned int dim() const;
	
	/**
	\brief return minimum position
	*/
	const Vector<Type>& minPos() const;

	/**
	\brief return minimum position
	*/
	Vector<Type>& minPos();
	
	/**
	\brief return maximum position
	*/
	const Vector<Type>& maxPos() const;

	/**
	\brief return maximum position
	*/
	Vector<Type>& maxPos();
	
	/**
	\brief return size
	\return size
	*/
	Vector<Type> size() const;

	/**
	\brief set minimum position
	\param pValue minimum position value
	\exception MathException if position is bigger than maximum position
	*/
	void setMinPos(Type pValue) throw (MathException);
	
	/**
	\brief set minimum position
	\param pMinPos minimum position
	\exception MathException if dimension of position is wrong or position is bigger than maximum position
	*/
	void setMinPos(const Vector<Type>& pMinPos) throw (MathException);

	/**
	\brief set minimum position
	\param pValues array of minimum position values
	\exception MathException if position is bigger than maximum position
	*/
	void setMinPos(const Type* pValues) throw (MathException);
 
	/**
	\brief set maximum position
	\param pValue maximum position value
	\exception MathException if position is smaller than minimum position
	*/
	void setMaxPos(Type pValue) throw (MathException);
	
	/**
	\brief set maximum position
	\param pMaxPos maximum position
	\exception MathException if dimension of position is wrong or position is smaller than minimum position
	*/
	void setMaxPos(const Vector<Type>& pMaxPos) throw (MathException);

	/**
	\brief set maximum position
	\param pValues array of maximum position values
	\exception MathException if position is smaller than minimum position
	*/
	void setMaxPos(const Type* pValues) throw (MathException);

	/**
	\brief set minimum and maximum positions
	\param pMinPos minimum position
	\param pMaxPos maximum position
	\exception MathException if dimension of one position is wrong or minimum position is bigger than maximum position
	*/
	void set(const Vector<Type>& pMinPos, const Vector<Type>& pMaxPos) throw (MathException);

	/**
	\brief set minimum and maximum positions
	\param pMinValues array of minimum position values
	\param pMaxValues arary of maximum position values
	\exception MathException if minimum position is bigger than maximum position
	*/
	void set(const Type* pMinValues, const Type* pMaxValues) throw (MathException);

	/**
	\brief rect == rect
	\param pRect rect to compare
	\return true if min and max positions of both rect's are equal
	*/
    bool operator==( const Rectangle<Type> &pRect ) const;
 
	/**
	\brief vector != vector
	\param pRect rect to compare
	\return true if min or max positions of both rect's are not equal
	*/	
	bool operator!=( const Rectangle<Type> &pRect ) const;
	
	/**
	\brief print rectangle information
	\param pOstream output stream
	\param pRect rect
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const Rectangle<Type>& pRect)
    {
		pOstream << pRect.info().toStdString();
		return pOstream;
    }; 
	
	/**
	\brief print vector information
	*/
	base::String info() const;	
	
protected:
	Rectangle();

	unsigned int mDim;
	Vector<Type> mMinPos;
	Vector<Type> mMaxPos;
};

template< class Type > 
Rectangle< Type >::Rectangle()
{}

template< class Type > 
Rectangle< Type >::Rectangle(unsigned int pDim)
	: mDim(pDim)
	, mMinPos(mDim)
	, mMaxPos(mDim)
{}

template< class Type > 
Rectangle< Type >::Rectangle(const Vector<Type>& pMinPos, const Vector<Type>& pMaxPos) throw (MathException)
	: mDim(pMinPos.dim())
	, mMinPos(pMinPos)
	, mMaxPos(pMaxPos)
{
	#ifdef _math_debug_
	if( mMinPos.dim() != mMaxPos.dim() ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("minPos dimension %1 must match maxPos dimension %2").arg( mMinPos.dim() ).arg( mMaxPos.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	if( mMinPos > mMaxPos ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("minPos %1 must be smaller than maxPos %2").arg( mMinPos.info() ).arg( mMaxPos.info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
}

template< class Type > 
Rectangle< Type >::Rectangle(const Rectangle<Type>& pRect)
	: mDim(pRect.mDim)
	, mMinPos(pRect.mMinPos)
	, mMaxPos(pRect.mMaxPos)
{}

template< class Type > 
Rectangle< Type >::~Rectangle()
{}

template< class Type >
const Rectangle<Type>& 
Rectangle< Type >::operator=( const Rectangle<Type>& pRect )
{
	mMinPos = pRect.mMinPos;
	mMaxPos = pRect.mMaxPos;
	
	return pRect;
}

template< class Type >
unsigned int
Rectangle< Type >::dim() const
{
	return mDim;
}

template< class Type >
const Vector<Type>& 
Rectangle< Type >::minPos() const
{
	return mMinPos;
}

template< class Type >
Vector<Type>& 
Rectangle< Type >::minPos()
{
	return mMinPos;
}

template< class Type >
const Vector<Type>& 
Rectangle< Type >::maxPos() const
{
	return mMaxPos;
}

template< class Type >
Vector<Type>& 
Rectangle< Type >::maxPos()
{
	return mMaxPos;
}

template< class Type >
Vector<Type> 
Rectangle< Type >::size() const
{
	Vector<Type> sizeVec(mDim);
	
	for(unsigned int i=0; i<mDim; ++i) sizeVec[i] = mMaxPos[i] - mMinPos[i];
	
	return sizeVec;
}

template< class Type >
void 
Rectangle< Type >::setMinPos(Type pValue) throw (MathException)
{
	#ifdef _math_debug_
	if( mMaxPos < pValue ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("minPos %1 must be smaller than maxPos %2").arg( pValue ).arg( mMaxPos.info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	mMinPos = pValue;
}

template< class Type >
void 
Rectangle< Type >::setMinPos(const Vector<Type>& pMinPos) throw (MathException)
{
	#ifdef _math_debug_
	if( pMinPos.dim() != mDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("minPos dimension %1 must match rectangle dimension %2").arg( pMinPos.dim() ).arg( mDim ), __FILE__, __FUNCTION__, __LINE__ );
	if( pMinPos > mMaxPos ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("minPos %1 must be smaller than maxPos %2").arg( pMinPos.info() ).arg( mMaxPos.info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	mMinPos = pMinPos;
}

template< class Type >
void 
Rectangle< Type >::setMinPos(const Type* pValues) throw (MathException)
{
	for(unsigned int i=0; i<mDim; ++i) 
	{
		#ifdef _math_debug_
		if( pValues[i] > mMaxPos.c[i] ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("minPos %1 must be smaller than maxPos %2").arg( pValues[i] ).arg( mMaxPos.c[i] ), __FILE__, __FUNCTION__, __LINE__ );
		#endif

		mMinPos[i] = pValues[i];
	}
}

template< class Type >
void 
Rectangle< Type >::setMaxPos(Type pValue) throw (MathException)
{
	#ifdef _math_debug_
	if( mMinPos > pValue ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("maxPos %1 must be larger than minPos %2").arg( pValue ).arg( mMinPos.info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	mMaxPos = pValue;
}

template< class Type >
void 
Rectangle< Type >::setMaxPos(const Vector<Type>& pMaxPos) throw (MathException)
{
	#ifdef _math_debug_
	if( pMaxPos.dim() != mDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("maxPos dimension %1 must match rectangle dimension %2").arg( pMaxPos.dim() ).arg( mDim ), __FILE__, __FUNCTION__, __LINE__ );
	if( pMaxPos < mMinPos ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("maxPos %1 must be larger than minPos %2").arg( pMaxPos.info() ).arg( mMinPos.info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	mMaxPos = pMaxPos;
}

template< class Type >
void 
Rectangle< Type >::setMaxPos(const Type* pValues) throw (MathException)
{
	for(unsigned int i=0; i<mDim; ++i) 
	{
		#ifdef _math_debug_
		if( pValues[i] < mMinPos.c[i] ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("maxPos %1 must be larger than minPos %2").arg( pValues[i] ).arg( mMinPos.c[i] ), __FILE__, __FUNCTION__, __LINE__ );
		#endif
	
		mMaxPos[i] = pValues[i];
	}
}

template< class Type >
void 
Rectangle< Type >::set(const Vector<Type>& pMinPos, const Vector<Type>& pMaxPos) throw (MathException)
{
	#ifdef _math_debug_
	if( pMinPos.dim() != mDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("minPos dimension %1 must match rectangle dimension %2").arg( pMinPos.dim() ).arg( mDim ), __FILE__, __FUNCTION__, __LINE__ );
	if( pMaxPos.dim() != mDim ) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("maxPos dimension %1 must match rectangle dimension %2").arg( pMaxPos.dim() ).arg( mDim ), __FILE__, __FUNCTION__, __LINE__ );
	if( pMaxPos < pMinPos ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("maxPos %1 must be larger than minPos %2").arg( pMaxPos.info() ).arg( pMinPos.info() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	mMinPos = pMinPos;
	mMaxPos = pMaxPos;
}

template< class Type >
void 
Rectangle< Type >::set(const Type* pMinValues, const Type* pMaxValues) throw (MathException)
{
	for(unsigned int i=0; i<mDim; ++i)
	{ 
		#ifdef _math_debug_
		if( pMaxValues[i] < pMinValues[i] ) throw MathException( "ERR_VECTOR_VECTOR_MISMATCH", base::String("maxPos %1 must be larger than minPos %2").arg( pMaxValues[i] ).arg( pMinValues[i] ), __FILE__, __FUNCTION__, __LINE__ );
		#endif
		
		mMinPos[i] = pMinValues[i];
		mMaxPos[i] = pMaxValues[i];
	}
}

template < class Type >  
bool 
Rectangle< Type >::operator==( const Rectangle<Type> &pRect ) const
{
	if( mDim != pRect.mDim ) return false;
	if( mMinPos != pRect.mMinPos ) return false;
	if( mMaxPos != pRect.mMaxPos ) return false;
	
	return true;
}

template < class Type >  
bool 
Rectangle< Type >::operator!=( const Rectangle<Type> &pRect ) const
{
	if( mDim != pRect.mDim ) return true;
	if( mMinPos != pRect.mMinPos ) return true;
	if( mMaxPos != pRect.mMaxPos ) return true;
	
	return false;
}

template< class Type >
base::String
Rectangle<Type>::info() const
{
	base::String infoString = base::String("%1 %2").arg( mMinPos.info() ).arg( mMaxPos.info() );
	
	return infoString;
}

};

};

#endif