/**	\file iso_math_vector.h
 *	\class iso::math::Vector vector class
 *	\brief vector class
 *
 *  Vectors are always treated as column vectors!
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_math_vector_h_
#define _iso_math_vector_h_

#include "iso_math_exception.h"
#include <cmath>
#include <iostream>
#include <iomanip>
#include <cassert>
#include <sstream>

namespace iso
{

namespace math
{

template< class Type > 
class Vector
{
public:
	/**
	\brief vector elements
	*/
	Type* c;

	/**
	\brief create default zero vector
	*/
    Vector();
	
	/**
	\brief create zero vector
	\param pDim size of vector
	*/
    Vector( unsigned int pDim ); 
	
	/**
	\brief create vector containing identical values
	\param pDim size of vector
	\param pValue value
	*/
    Vector( unsigned int pDim, Type pValue );
	
	/**
	\brief create vector from array of values
	\param pDim size of vector
	\param pValues array of values (must be of correct size)
	*/
	Vector( unsigned int pDim, const Type* pValues );

	/**
	\brief create vector from array of values
	\param pDim size of vector
	\param pValues array of values (must be of correct size)
	*/
	template<typename Type2>Vector( unsigned int pDim, const Type2* pValues );
	
	/**
	\brief create vector from parameter list of values
	\param pDim size of vector
	\param pValue1 value 1
	\param pValue2 value 2
	\exception MathException pDim smaller than 2
	*/
	Vector( unsigned int pDim, double pValue1, double pValue2, ... ) throw (MathException);
	
	/**
	\brief create vector from vector
	\param pVector vector to copy
	*/
	Vector( const Vector<Type>& pVector );

	/**
	\brief create vector from vector
	\param pVector vector to copy
	*/
	template< class Type2 > Vector( const Vector<Type2>& pVector );
	
	/**
	\brief destructor
	*/
    virtual ~Vector();

	/**
	\brief set all vector elements to 0
	*/
	void reset();
	
	/**
	\brief set dimension of vector
	\param pDim new vector dimension
	
	original values are preserved as far as possible.
	when vector dimension increases, new elements contain zero
	when vector dimension decreases, previous values at positions that are beyond new dimension
	*/
	virtual void setDim(unsigned int pDim);

	/**
	\brief set vector to identical values
	\param pValue value
	*/
    inline void set( Type pValue );

	/**
	\brief set vector to new dimension and set all it's components to pValue
	\param pValue value
	\param pDim new vector dimension
	\exception MathException always since this is an illegal function
	*/
	inline void set( Type pValue, unsigned int pDim ) throw (MathException);
	
	/**
	\brief set vector to values from array
	\param pValues array of values
	\remarks array of values must be of correct size
	*/
	inline void set( const Type* pValues );
	
	/**
	\brief set vector to parameter list of values
	\param pDim size of vector
	\param pValue1 value 1
	\param pValue2 value 2
	\exception MathException pDim smaller than 2 or dimension mismatch
	*/
	inline void set( unsigned int pDim, double pValue1, double pValue2, ... ) throw (MathException);
	
	/**
	\brief set vector to values from vector
	\param pVector vector to copy values from
	\exception MathException dimension mismatch
	*/
	inline void set( const Vector<Type>& pVector ) throw (MathException);

	/**
	\brief set vector to values from vector
	\param pVector vector to copy values from
	\param pResize change vector dim if necessary
	\exception MathException always since this is an illegal vector function
	*/
	inline void set( const Vector<Type>& pVector, bool pResize ) throw (MathException);
	
	/**
	\brief scale vector to new length
	\param pLength new length
	*/
	void setLength( Type pLength );

	/**
	\brief vector = value
	\param pValue value
	\return vector
	*/
	inline const Vector<Type>& operator=( Type pValue );

	/**
	\brief vector = values[]
	\param pValues array of values (array must be of correct length)
	\return vector
	*/
	inline const Vector<Type>& operator=( const Type* pValues ); 
	
	/**
	\brief vector = vector
	\param pVector vector to copy values from
	\return vector
	\exception MathException dimension mismatch
	*/
    inline const Vector<Type>& operator=( const Vector<Type>& pVector ) throw (MathException); 
	
	/**
	\brief vector[]
	\param pIndex index of vector component
	\return value
	\exception MathException index out of bounds
	*/
	inline Type& operator[]( unsigned int pIndex ) throw (MathException);
	
	/**
	\brief operator[]
	\param pIndex index of vector component
	\return constant value
	\exception MathException index out of bounds
	*/
    inline const Type& operator[]( unsigned int pIndex ) const throw (MathException);
	
	/**
	\brief return size of vector
	\return size of vector
	*/
	inline const unsigned int& dim() const;
	
	/**
	\brief return vector values
	\return vector values (as array)
	*/
	Type* values();
	
	/**
	\brief return vector values
	\return vector values (as const array)
	*/
	const Type* values() const;
	
	/**
	\brief return length of vector
	\return length of vector
	*/
	inline Type length() const;
	
	/**
	\brief return length^2 of vector
	\return length^2 of vector
	*/
	inline Type lengthSquared() const;

	/**
	\brief vector += value
	\param pValue value
	\return vector
	*/
    inline const Vector<Type>& operator+=( const Type  pValue );
 
	/**
	\brief vector -= value
	\param pValue value
	\return vector
	*/	
	inline const Vector<Type>& operator-=( const Type  pValue ); 

	/**
	\brief vector *= value
	\param pValue value
	\return vector
	*/	
    inline const Vector<Type>& operator*=( const Type  pValue );

	/**
	\brief vector /= value
	\param pValue value
	\return vector
	*/	
    inline const Vector<Type>& operator/=( Type  pValue );
	
	/**
	\brief vector + value
	\param pValue value
	\return vector
	*/	 
	inline Vector<Type> operator+( const Type  pValue ) const;
	
	/**
	\brief vector - value
	\param pValue value
	\return vector
	*/
    inline Vector<Type> operator-( const Type  pValue ) const;
	
	/**
	\brief vector * value
	\param pValue value
	\return vector
	*/
    inline Vector<Type> operator*( const Type  pValue ) const;
	
	/**
	\brief vector / value
	\param pValue value
	\return vector
	*/
    inline Vector<Type> operator/( Type  pValue ) const;

	/**
	\brief vector += vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline const Vector<Type>& operator+=( const Vector<Type>& pVector ) throw (MathException); 
	
	/**
	\brief vector -= vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline const Vector<Type>& operator-=( const Vector<Type>& pVector ) throw (MathException); 
	
	/**
	\brief vector *= vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    virtual const Vector<Type>& operator*=( const Vector<Type>& pVector ) throw (MathException); 
	
	/**
	\brief vector /= vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline const Vector<Type>& operator/=( const Vector<Type>& pVector ) throw (MathException); 

	/**
	\brief vector + vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
	inline Vector<Type> operator+( const Vector<Type>& pVector ) const throw (MathException);
	
	/**
	\brief vector - vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/ 
    inline Vector<Type> operator-( const Vector<Type>& pVector ) const throw (MathException);
	
	/**
	\brief vector * vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    virtual Vector<Type> operator*( const Vector<Type>& pVector ) const throw (MathException); 
	
	/**
	\brief vector / vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline Vector<Type> operator/( const Vector<Type>& pVector ) const throw (MathException); 
	
	/**
	\brief vector dot product
	\param pVector vector
	\return dot product
	\exception MathException dimension mismatch
	*/
	inline Type dot( const Vector<Type>& pVector) const throw (MathException);

	/**
	\brief vector dot product
	\param pVector1 vector 1
	\param pVector2 vector 2
	\return dot product
	\exception MathException dimension mismatch
	*/
    static Type dot( const Vector<Type>& pVector1, const Vector<Type>& pVector2) throw (MathException);

	/**
	\brief return sum of all vector components
	\return return sum of all vector components
	*/
	inline Type sum() const;

	/**
	\brief return product of all vector components
	\return return product of all vector components
	*/
	inline Type product() const;
	
	/**
	\brief projects vector 1 on vector 2
	\param pVector1 vector 1
	\param pVector2 vector 2
	\return returns projected vector
	\exception MathException dimension mismatch
	\
	*/
	static Vector<Type> project( const Vector<Type>& pVector1, const Vector<Type>& pVector2 ) throw (MathException);

	/**
	\brief vector == pValue
	\param pValue scalar value
	\return true if all elements are equal to scalar value, false otherwise
	*/
    inline bool operator==( const Type& pValue ) const;
 
	/**
	\brief vector != pValue
	\param pValue scalar value
	\return true if one of the elements is not equal to scalar value, false otherwise
	*/	
	inline bool operator!=( const Type& pValue ) const;

	/**
	\brief vector > vector
	\param pValue scalar value
	\return true if all elements are larger than scalar value, false otherwise
	*/
    inline bool operator>( const Type& pValue ) const;

	/**
	\brief vector > vector
	\param pValue scalar value
	\return true if all elements are smaller than scalar value, false otherwise
	*/
    inline bool operator<( const Type& pValue ) const;

	/**
	\brief vector > vector
	\param pValue scalar value
	\return true if all elements are larger or equal than scalar value, false otherwise
	*/
    inline bool operator>=( const Type& pValue ) const;

	/**
	\brief vector > vector
	\param pValue scalar value
	\return true if all elements are smaller or equal than scalar value, false otherwise
	*/
    inline bool operator<=( const Type& pValue ) const;

	/**
	\brief vector == vector
	\return true if both vector length and all elements are equal, false otherwise
	*/
    inline bool operator==( const Vector<Type> &pVector ) const;
 
	/**
	\brief vector != vector
	\return true if either vector length or one of the elements is not equal, false otherwise
	*/	
	inline bool operator!=( const Vector<Type> &pVector ) const;

	/**
	\brief vector > vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline bool operator>( const math::Vector<Type>& pVector ) const throw (MathException);

	/**
	\brief vector < vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline bool operator<( const math::Vector<Type>& pVector ) const throw (MathException);

	/**
	\brief vector >= vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline bool operator>=( const math::Vector<Type>& pVector ) const throw (MathException);

	/**
	\brief vector <= vector
	\param pVector vector
	\return vector
	\exception MathException dimension mismatch
	*/
    inline bool operator<=( const math::Vector<Type>& pVector ) const throw (MathException);

	/**
	\brief normalise vector length
	*/
    inline void normalise();
	
	/**
	\brief limit vector length to max length
	*/
	inline void limit(Type pMaxLength);

    /**
    \brief truncates a signal, crops everything above the upper threshold and below the lower threshold
	\param pThresh1 lower threshold
	\param pThresh2 upper threshold
    */  
	inline void truncate(Type pThresh1, Type pThresh2);
	    
    /**
    \brief truncates a signal, crops everything above the upper threshold and below the lower threshold
	\param pThresh1 lower threshold
	\param pThresh2 upper threshold
	\exception MathException dimension mismatch
    */  
    inline void truncate(const Vector<Type>& pThresh1, const Vector<Type>& pThresh2) throw (MathException);
    
    /**
	\brief invert vector
	*/
    inline void invert();

	/**
	\brief print vector information
	\param pOstream output stream
	\param pVector vector
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const Vector<Type>& pVector)
    {
		pOstream << pVector.info().toStdString();
		return pOstream;
    }; 
	
	operator base::String () const
	{
		return info();
	};	
	
	/**
	\brief print vector information
	*/
	base::String info() const;
	
protected:
	/**
	\brief dimension
	*/
	unsigned int mDim;
	
	/**
	\brief default dimension
	*/
	static unsigned int sDim;
};

typedef Vector<int>    VectorI;
typedef Vector<float>  VectorF;
typedef Vector<double> VectorD;

template<typename Type> 
unsigned int Vector<Type>::sDim = 3;

template < class Type > 
Vector< Type >::Vector() 
	: mDim( sDim )
{
	c = new Type[ mDim ];
	for(unsigned int d=0; d<mDim; ++d) c[d] = 0;
}

template < class Type > 
Vector< Type >::Vector( unsigned int pDim ) 
	: mDim( pDim )
{
	c = new Type[ mDim ];
	for(unsigned int d=0; d<mDim; ++d) c[d] = 0;
}

template < class Type > 
Vector< Type >::Vector( unsigned int pDim, Type pValue )
	: mDim( pDim )
{
	c = new Type[ mDim ];
	for(unsigned int i=0; i<mDim; ++i) c[i] = pValue;
}

template < class Type > 
Vector< Type >::Vector( unsigned int pDim, const Type* pValues )
	: mDim( pDim )
{
	c = new Type[ mDim ];
	for(unsigned int d=0; d<mDim; ++d) c[d] = pValues[d];
}

template < class Type > 
template< typename Type2 >
Vector< Type >::Vector( unsigned int pDim, const Type2* pValues )
	: mDim( pDim )
{
	c = new Type[ mDim ];
	for(int d=0; d<mDim; ++d) c[d] = pValues[d];	
}

template < class Type > 
Vector< Type >::Vector( unsigned int pDim, double pValue1, double pValue2, ...) throw (MathException)
	: mDim( pDim )
{
	#ifdef _math_debug_
	if(pDim < 2) throw MathException( "ERR_VECTOR_ERROR", base::String("dimension %1 to small, must be at least 2").arg( pDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	c = new Type[ mDim ];
	
	c[0] = static_cast<Type>( pValue1 );
	c[1] = static_cast<Type>( pValue2 );

	va_list ap;
	va_start( ap, pValue2 );
    for( unsigned int i=2; i < mDim;i++) c[i] = static_cast<Type>( va_arg( ap,double ) );
	va_end(ap);	
}

template < class Type > 
Vector< Type >::Vector( const Vector<Type>& pVector )
	: mDim( pVector.mDim )
{
	c = new Type[ mDim ];
	for(unsigned int d=0; d<mDim; ++d) c[d] = pVector.c[d];
}

template < class Type > 
template< class Type2 >
Vector< Type >::Vector( const Vector<Type2>& pVector )
	: mDim( pVector.dim() )
{
	c = new Type[ mDim ];
	for(unsigned int i=0; i<mDim; ++i) c[i] = static_cast<Type>( pVector.c[i] );
} 
 
template < class Type > 
Vector< Type >::~Vector()
{
	delete [] c;
}

template < class Type > 
void 
Vector< Type >::reset()
{
	for(unsigned int i=0; i<mDim; ++i) c[i] = static_cast<Type>(0.0);	
}

template < class Type > 	
void 
Vector< Type >::setDim(unsigned int pDim)
{
	if(mDim == pDim) return;
	
	Type* newC = new Type[pDim];
	unsigned minDim = mDim > pDim ? pDim : mDim;
	unsigned int i=0;
	
	for(; i<minDim; ++i)
	{
		newC[i] = c[i];
	}
	for(; i<pDim; ++i)
	{
		newC[i] = 0.0;
	}
	
	delete [] c;
	c = newC;
	mDim = pDim;
}

template < class Type > 
void 
Vector< Type >::set( Type pValue )
{
	for(unsigned int i=0; i<mDim; ++i) c[i] = pValue;
}

template <typename Type>
void 
Vector<Type>::set( Type pValue, unsigned int pDim ) throw (MathException)
{ 
	#ifdef _math_debug_
	throw MathException( "ERR_VECTOR_ERROR", base::String("illegal vector function"), __FILE__, __FUNCTION__, __LINE__ );
	#endif
}

template < class Type > 
void 
Vector< Type >::set( const Type* pValues )
{
	for(unsigned int d=0; d<mDim; ++d) c[d] = pValues[d];
}

template < class Type > 
void 
Vector< Type >::set( unsigned int pDim, double pValue1, double pValue2, ... ) throw (MathException)
{
	#ifdef _math_debug_
	if(pDim < 2) throw MathException( "ERR_VECTOR_ERROR", base::String("dimension %1 to small, must be at least 2").arg( pDim ), __FILE__, __FUNCTION__, __LINE__ );
	if(mDim != pDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	c[0] = static_cast<Type>( pValue1 );
	c[1] = static_cast<Type>( pValue2 );

	va_list ap;
	va_start( ap, pValue2 );
    for( unsigned int i=2; i < mDim;i++) c[i] = static_cast<Type>( va_arg( ap,double ) );
	va_end(ap);	
}

template < class Type > 
void 
Vector< Type >::set( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	for(unsigned int d=0; d<mDim; ++d) c[d] = pVector.c[d];
}

template <typename Type>
void 
Vector<Type>::set( const Vector<Type>& pVector, bool pResize ) throw (MathException)
{ 
	#ifdef _math_debug_
	throw MathException( "ERR_VECTOR_ERROR", base::String("illegal vector function"), __FILE__, __FUNCTION__, __LINE__ );
	#endif
}

template < class Type > 
void 
Vector< Type >::setLength( Type pLength )
{
	this->operator*=(pLength / length());
}

template < class Type > 
const Vector<Type>& 
Vector< Type >::operator=( Type pValue )
{
	for(unsigned int i=0; i<mDim; ++i) c[i] = pValue;
	
	return (*this);
}

template < class Type > 
const Vector<Type>& 
Vector< Type >::operator=( const Type* pValues )
{
	for(unsigned int d=0; d<mDim; ++d) c[d] = pValues[d];
	
	return (*this);
}

template < class Type > 
const Vector<Type>& 
Vector< Type >::operator=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int d=0; d<mDim; ++d) c[d] = pVector.c[d];
	
	return (*this);
}

template < class Type > 
Type& 
Vector< Type >::operator[]( unsigned int pIndex ) throw (MathException)
{
	#ifdef _math_debug_
	if( mDim <= pIndex ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index %1 out of bounds. max: %2").arg( pIndex ).arg( mDim  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	return c[pIndex];
}

template < class Type > 
const Type& 
Vector< Type >::operator[]( unsigned int pIndex ) const throw (MathException)
{
	#ifdef _math_debug_
	if( mDim <= pIndex ) throw MathException( "ERR_OUT_OF_BOUNDS", base::String("index %1 out of bounds. max: %2").arg( pIndex ).arg( mDim  - 1 ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	return c[pIndex];
}

template < class Type >
const unsigned int& 
Vector< Type >::dim() const
{
	return mDim;
}

template < class Type >
Type* 
Vector< Type >::values()
{
	return c;
}

template < class Type >
const Type* 
Vector< Type >::values() const
{
	return c;
}

template < class Type >
Type 
Vector< Type >::length() const
{
	return sqrt(lengthSquared());
}

template < class Type >
Type 
Vector< Type >::lengthSquared() const
{
	Type squared = 0;
	
	for(unsigned int i=0; i<mDim; ++i) squared += c[i] * c[i];
	
	return squared;
}

template < class Type >
const Vector<Type>& 
Vector< Type >::operator+=( const Type  pValue )
{
	for(unsigned int i=0; i<mDim; ++i) c[i] += pValue;
	return (*this);
}

template < class Type >
const Vector<Type>& 
Vector< Type >::operator-=( const Type  pValue )
{
	for(unsigned int i=0; i<mDim; ++i) c[i] -= pValue;
	return (*this);
}

template < class Type >
const Vector<Type>& 
Vector< Type >::operator*=( const Type  pValue )
{
	for(unsigned int i=0; i<mDim; ++i) c[i] *= pValue;
	return (*this);
}

template < class Type >
const Vector<Type>& 
Vector< Type >::operator/=( Type  pValue )
{
	for(unsigned int i=0; i<mDim; ++i) c[i] /= pValue;
	return (*this);
}

template < class Type >
Vector< Type > 
Vector< Type >::operator+( const Type  pValue ) const
{
	Vector<Type> vec(*this);
	vec += pValue;
	return vec;
}

template < class Type >
Vector< Type >
Vector< Type >::operator-( const Type  pValue ) const
{
	Vector<Type> vec(*this);
	vec -= pValue;
	return vec;
}

template < class Type >
Vector< Type > 
Vector< Type >::operator*( const Type  pValue ) const
{
	//std::cout << "Vector< Type >::operator*( const Type  pValue ) const\n";

	Vector<Type> vec(*this);
	
	vec *= pValue;
	
	return vec;
}

template < class Type >
Vector< Type >
Vector< Type >::operator/( Type  pValue ) const
{
	Vector<Type> vec(*this);
	vec /= pValue;
	return vec;
}

template < class Type >
const Vector<Type>& 
Vector< Type >::operator+=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for( unsigned int i=0; i<mDim; ++i ) c[i] += pVector.c[i];
	return(*this);
}

template < class Type >
const Vector<Type>& 
Vector< Type >::operator-=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for( unsigned int i=0; i<mDim; ++i ) c[i] -= pVector.c[i];
	return(*this);
}

template < class Type >
const Vector<Type>& 
Vector< Type >::operator*=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for( unsigned int i=0; i<mDim; ++i ) c[i] *= pVector.c[i];
	return(*this);
}

template < class Type >  
const Vector<Type>& 
Vector< Type >::operator/=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for( unsigned int i=0; i<mDim; ++i ) c[i] /= pVector.c[i];
	return(*this);
}

template < class Type >  
Vector<Type> 
Vector< Type >::operator+( const Vector<Type>& pVector ) const throw (MathException)
{
	try
	{
		Vector<Type> vec(*this);
		vec += pVector;
		return vec;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
Vector<Type> 
Vector< Type >::operator-( const Vector<Type>& pVector ) const throw (MathException)
{
	try
	{
		Vector<Type> vec(*this);
		vec -= pVector;
		return vec;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
Vector<Type> 
Vector< Type >::operator*( const Vector<Type>& pVector ) const throw (MathException)
{
	try
	{
		Vector<Type> vec(*this);
		vec *= pVector;
		return vec;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
Vector<Type> 
Vector< Type >::operator/( const Vector<Type>& pVector ) const throw (MathException)
{
	try
	{
		Vector<Type> vec(*this);
		vec /= pVector;
		return vec;
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
Type 
Vector< Type >::dot( const Vector<Type>& pVector) const throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Type product = 0.0;
	
	for( unsigned int i=0; i<mDim; ++i ) product += c[i] * pVector.c[i]; 
	
	return product; 
}

template < class Type >  
Type 
Vector< Type >::dot( const Vector<Type>& pVector1, const Vector<Type>& pVector2) throw (MathException)
{
	#ifdef _math_debug_
	if(pVector1.mDim != pVector2.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, vector 1 dim %1 does not match vector 2 dim %2").arg( pVector1.mDim ).arg( pVector2.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	Type product = 0.0;
	
	for( unsigned int i=0; i<pVector1.mDim; ++i ) product += pVector1.c[i] * pVector2.c[i]; 
	
	return product; 
}

template < class Type >  
Type 
Vector< Type >::sum() const
{
	Type vecsum = 0;
	
	for(unsigned int i=0; i<mDim; ++i) vecsum += c[i];
	
	return vecsum;
}

template < class Type >  
Type 
Vector< Type >::product() const
{
	Type vecprod = 1;
	
	for(unsigned int i=0; i<mDim; ++i) vecprod *= c[i];
	
	return vecprod;
}

template < class Type >  
Vector<Type> 
Vector< Type >::project( const Vector<Type>& pVector1, const Vector<Type>& pVector2 ) throw (MathException)
{
	try
	{
		Type lengthSquared = pVector2.lengthSquared();
		if(lengthSquared == 0.0) return pVector1;
		else return pVector2 * ( dot(pVector1, pVector2) / lengthSquared );
	}
	catch(MathException& e)
	{
		throw;
	}
}

template < class Type >  
bool 
Vector< Type >::operator==( const Type& pValue ) const
{
	for(unsigned int i=0; i<mDim; ++i) if(c[i] != pValue) return false;
	return true;
}

template < class Type >  
bool 
Vector< Type >::operator!=( const Type& pValue ) const
{
	for(unsigned int i=0; i<mDim; ++i) if(c[i] == pValue) return false;
	return true;
}

template < class Type >
bool 
Vector< Type >::operator>( const Type& pValue ) const
{
	for(unsigned int d=0; d<mDim; ++d) if( c[d] <= pValue ) return false;
	return true;
}

template < class Type >
bool 
Vector< Type >::operator<( const Type& pValue ) const
{
	for(unsigned int d=0; d<mDim; ++d) if( c[d] >= pValue ) return false;
	return true;
}

template < class Type >
bool 
Vector< Type >::operator>=( const Type& pValue ) const
{
	for(unsigned int d=0; d<mDim; ++d) if( c[d] < pValue ) return false;
	return true;
}

template < class Type >
bool 
Vector< Type >::operator<=( const Type& pValue ) const
{
	for(unsigned int d=0; d<mDim; ++d) if( c[d] > pValue ) return false;
	return true;
}

template < class Type >  
bool 
Vector< Type >::operator==( const Vector<Type> &pVector ) const
{
	if( mDim != pVector.mDim ) return false;
	
	for(unsigned int i=0; i<mDim; ++i) if(c[i] != pVector.c[i]) return false;
	
	return true;
}

template < class Type >  
bool 
Vector< Type >::operator!=( const Vector<Type> &pVector ) const
{
	if( mDim != pVector.mDim ) return true;
	
	for(unsigned int i=0; i<mDim; ++i) if(c[i] != pVector.c[i]) return true;
	
	return false;
}

template < class Type >
bool 
Vector< Type >::operator>( const math::Vector<Type>& pVector ) const throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int d=0; d<mDim; ++d) if( c[d] <= pVector.c[d] ) return false;
	return true;
}

template < class Type >
bool 
Vector< Type >::operator<( const math::Vector<Type>& pVector ) const throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int d=0; d<mDim; ++d) if( c[d] >= pVector.c[d] ) return false;
	return true;
}

template < class Type >
bool 
Vector< Type >::operator>=( const math::Vector<Type>& pVector ) const throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int d=0; d<mDim; ++d) if( c[d] < pVector.c[d] ) return false;
	return true;
}

template < class Type >
bool 
Vector< Type >::operator<=( const math::Vector<Type>& pVector ) const throw (MathException)
{
	#ifdef _math_debug_
	if(mDim != pVector.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( mDim ).arg( pVector.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	for(unsigned int d=0; d<mDim; ++d) if( c[d] > pVector.c[d] ) return false;
	return true;
}

template < class Type >  
void 
Vector< Type >::normalise()
{
	Type l = length();
	if( l == 0 ) return;
	
	l = 1.0 / l;	
	for( unsigned int i=0; i<mDim; ++i ) c[i] *= l;
}

template < class Type >
void
Vector< Type >:: limit(Type pMaxLength)
{ 
	Type l = length();
	
	if(l > pMaxLength && l != 0)
	{
		l = 1.0 / l;
		
		for(int i=0; i<mDim; ++i) c[i] = c[i] * pMaxLength * l;
	}
}

template < class Type >
void
Vector< Type >:: truncate(Type pThresh1, Type pThresh2)
{ 
	if (pThresh1 > pThresh2)
	{
		Type tmp = pThresh1;
		pThresh1 = pThresh2;
		pThresh2 = tmp;
	} 
	
	for( unsigned int i=0; i<mDim; ++i ) 
	{
        if (c[i] > pThresh2) c[i] = pThresh2;
		else if (c[i] < pThresh1) c[i] = pThresh1;
    }
}


template < class Type >
void
Vector< Type >::truncate(const Vector<Type>& pThresh1, const Vector<Type>& pThresh2) throw (MathException)
{ 
	#ifdef _math_debug_
	if(mDim != pThresh1.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 threshold 1 dimension supplied %2").arg( mDim ).arg( pThresh1.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	if(mDim != pThresh2.mDim) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 threshold 2 dimension supplied %2").arg( mDim ).arg( pThresh2.mDim ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
    
    const Vector<Type>* top = NULL;
    const Vector<Type>* bottom = NULL;
    
    for( unsigned int i=0; i<mDim; ++i ) {
        if (pThresh1[i] > pThresh2[i])
        {
            top = &pThresh1;
            bottom = &pThresh2;
        } else {
            top = &pThresh2;
            bottom = &pThresh1;    
        }
        if (c[i] > top->c[i]) 
            c[i] = top->c[i];

        if (c[i] < bottom->c[i])
            c[i] = bottom->c[i];
    }

}

template < class Type >  
void 
Vector< Type >::invert()
{
	for( unsigned int i=0; i<mDim; ++i ) c[i] = -c[i];
}

template< class Type >
base::String
Vector<Type>::info() const
{
	std::ostringstream os;

	const std::ios::fmtflags flags = os.flags();
	const int prec  = os.precision();

	os.setf( std::ios::right, std::ios::adjustfield );
	os.precision( 5 );
	os << "[ ";
	for( unsigned int i=0; i<mDim; ++i) os << std::setw(10) << c[i] << " ";
	os << "]";
	os.precision( prec );
	os.setf( flags );
	
	std::string stdString = os.str();
	
	base::String infoString(stdString.c_str());
	
	return infoString;
}

}

}

#endif
