/**	\file iso_math_vector3.h
 *	\class iso::math::Vector3 3D vector class
 *	\brief 3D vector class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_math_vector3_h_
#define _iso_math_vector3_h_

#include "iso_math_vector.h"

namespace iso
{

namespace math
{

template< class Type >
class Vector3 : public Vector<Type>
{
public:
 	/**
	\brief zero vector
	*/
	static const Vector3<Type> ZERO;

	/**
	\brief one vector
	*/
	static const Vector3<Type> ONE;

	/**
	\brief references to vector components (euclidian coordinates)
	*/
    Type &x, &y, &z;
	
	/**
	\brief references to vector components (rgb color components)
	*/
	Type &r, &g, &b;

	/**
	\brief create zero vector
	*/
    Vector3();
	
	/**
	\brief create vector with identical vector components
	\param pValue value
	*/	
    Vector3( Type pValue );

	/**
	\brief create vector from parameters
	\param p0 0. value
	\param p1 1. value
	\param p2 2. value
	*/
	Vector3( Type p0, Type p1, Type p2 );
	
	/**
	\brief create vector from array of values
	\param pValues array of values (must be of correct size)
	*/
	Vector3( Type* pValues );

	/**
	\brief create vector from vector
	\param pVector vector of size 3
	*/
	Vector3( const Vector3<Type>& pVector );

	/**
	\brief create vector from vector
	\param pVector vector of size 3
	*/
	template< class Type2> Vector3( const Vector3<Type2>& pVector );

	/**
	\brief create vector from vector
	\param pVector vector
	\exception MathException pVector dimension not 3
	*/
	Vector3( const Vector<Type>& pVector ) throw (MathException);

	/**
	\brief create vector from vector
	\param pVector vector
	\exception MathException pVector dimension not 3
	*/
	template< class Type2 > Vector3( const Vector<Type2>& pVector ) throw (MathException);

	/**
	\brief destructor
	*/
    virtual ~Vector3();

	/**
	\brief set dimension of vector
	\param pDim new vector dimension
	\exception MathException illegal function
	*/
	virtual void setDim(unsigned int pDim) throw (MathException);

	/**
	\brief set vector components from parameters
	\param p0 0. value
	\param p1 1. value
	\param p2 2. value
	*/	
	void set( Type p0, Type p1, Type p2 );

	/**
	\brief set vector components from parameters
	\param pValues values
	\remarks make sure the value array is of the correct size
	*/	
	void set( Type *pValues );

	/**
	\brief set vector components from vector
	\param pVector of size 3
	*/	
	const Vector3<Type>& operator=( const Vector3<Type>& pVector ); 

	/**
	\brief set vector components from vector
	\param pVector (must be of correct size)
	\exception MathException pVector dimension not 2
	*/
	const Vector3<Type>& operator=( const Vector<Type>& pVector ) throw (MathException); 

	/**
	\brief vector cross product
	\param pVector second vector
	\return cross product
	*/	
    Vector3<Type> cross( const Vector3& pVector ) const;
	
	/**
	\brief vector cross product
	\param pVector1 first vector
	\param pVector2 second vector
	\return cross product
	*/		
    static Vector3<Type> cross( const Vector3<Type> &pVector1, const Vector3<Type> &pVector2);
	
	/**
	\brief rotate vector around axis
	\param pTheta angle of rotation (in radians)
	\param pRx x component of rotation axis
	\param pRy y component of rotation axis
	\param pRz z component of rotation axis	
	*/
	void rotate( Type pTheta, Type pRx, Type pRy, Type pRz ); 

	/**
	\brief rotate vector around axis
	\param pTheta angle of rotation (in radians)
	\param pVector rotation axis
	*/
	void rotate( Type pTheta, const Vector3<Type>& pVector ); 
	
	/**
	\brief print vector information
	\param pOstream output stream
	\param pVector vector
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const Vector3<Type>& pVector)
    {
		pOstream << pVector.info().toStdString();
		return pOstream;
    }; 
};

typedef Vector3<int>    Vector3I;
typedef Vector3<float>  Vector3F;
typedef Vector3<double> Vector3D;

 template<class Type>
 const Vector3<Type> 
 Vector3<Type>::ZERO(0.0, 0.0, 0.0);

 template<class Type>
 const Vector3<Type> 
 Vector3<Type>::ONE(1.0, 1.0, 1.0);

template< class Type > 
Vector3<Type>::Vector3()
	: Vector<Type>( 3 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{}

template< class Type > 
Vector3<Type>::Vector3( Type pValue )
	: Vector<Type>( 3, pValue )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{}

template< class Type > 
Vector3<Type>::Vector3( Type p0, Type p1, Type p2 )
	: Vector<Type>( 3 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{
	x = p0;
	y = p1;
	z = p2;
}

template< class Type > 
Vector3<Type>::Vector3( Type* pValues )
	: Vector<Type>( 3, pValues )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{}

template< class Type > 
Vector3<Type>::Vector3( const Vector3<Type>& pVector )
	: Vector<Type>( pVector )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{}

template< class Type > 
template< class Type2 > 
Vector3<Type>::Vector3( const Vector3<Type2>& pVector )
	: Vector<Type>( pVector )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{}

template< class Type > 
Vector3<Type>::Vector3( const Vector<Type>& pVector ) throw (MathException)
	: Vector<Type>( 3 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	x = pVector.c[0];
	y = pVector.c[1];
	z = pVector.c[2];
}

template< class Type > 
template< class Type2 > 
Vector3<Type>::Vector3( const Vector<Type2>& pVector ) throw (MathException)
	: Vector<Type>( 3 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	x = static_cast<Type>( pVector.c[0] );
	y = static_cast<Type>( pVector.c[1] );
	z = static_cast<Type>( pVector.c[2] );
}

template< class Type > 
Vector3<Type>::~Vector3()
{}

template < class Type > 	
void 
Vector3< Type >::setDim(unsigned int pDim) throw (MathException)
{
	#ifdef _math_debug_
	throw MathException( "ERR_VECTOR_ERROR", base::String("illegal vector function"), __FILE__, __FUNCTION__, __LINE__ );
	#endif
}

template< class Type > 
void 
Vector3<Type>::set( Type pX, Type pY, Type pZ )
{
	x = pX;
	y = pY;
	z = pZ;
}

template< class Type >
void 
Vector3<Type>::set( Type *pValues )
{
	x = pValues[0];
	y = pValues[1];
	z = pValues[2];
}

template< class Type > 
const Vector3<Type>& 
Vector3<Type>::operator=( const Vector3<Type>& pVector )
{
	x = pVector.x;
	y = pVector.y;
	z = pVector.z;
	
	return (*this);
}

template< class Type > 
const Vector3<Type>& 
Vector3<Type>::operator=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	x = pVector.c[0];
	y = pVector.c[1];
	z = pVector.c[2];
	
	return (*this);
}

template< class Type > 
Vector3<Type> 
Vector3<Type>::cross( const Vector3& pVector ) const
{
	return Vector3<Type>( y * pVector.z - z * pVector.y, z * pVector.x - x * pVector.z, x * pVector.y - y * pVector.x ); 
}

template< class Type > 
Vector3<Type> 
Vector3<Type>::cross( const Vector3<Type> &pVector1, const Vector3<Type> &pVector2)
{
	return Vector3<Type>( pVector1.y * pVector2.z - pVector1.z * pVector2.y, pVector1.z * pVector2.x - pVector1.x * pVector2.z, pVector1.x * pVector2.y - pVector1.y * pVector2.x ); 
}

template< class Type > 
void
Vector3<Type>::rotate( Type pTheta, Type pRx, Type pRy, Type pRz )
{
    Vector3<Type> qvec( 0, 0, 0 ), rvec( pRx, pRy, pRz );
    r.normalise();
    const Type costheta = static_cast<Type>( cos( pTheta ) );
    const Type sintheta = static_cast<Type>( sin( pTheta ) );
    qvec.x += ( costheta + ( 1.0f - costheta ) * r.x * r.x ) * x;
    qvec.x += ( ( 1 - costheta ) * rvec.x * rvec.y - rvec.z * sintheta ) * y;
    qvec.x += ( ( 1 - costheta ) * rvec.x * rvec.z + rvec.y * sintheta ) * z;
    qvec.y += ( ( 1 - costheta ) * rvec.x * rvec.y + rvec.z * sintheta ) * x;
    qvec.y += ( costheta + ( 1 - costheta ) * rvec.y * rvec.y ) * y;
    qvec.y += ( ( 1 - costheta ) * rvec.y * rvec.z - rvec.x * sintheta ) * z;
    qvec.z += ( ( 1 - costheta ) * rvec.x * rvec.z - rvec.y * sintheta ) * x;
    qvec.z += ( ( 1 - costheta ) * rvec.y * rvec.z + rvec.x * sintheta ) * y;
    qvec.z += ( costheta + ( 1 - costheta ) * rvec.z * rvec.z ) * z;
    
	x = qvec.x;
	y = qvec.y;
	z = qvec.z;
}

template< class Type > 
void
Vector3<Type>::rotate( Type pTheta, const Vector3<Type>& pVector )
{
    Vector3<Type> qvec( 0, 0, 0 ), rvec( pVector );
    r.normalise();
    const Type costheta = static_cast<Type>( cos( pTheta ) );
    const Type sintheta = static_cast<Type>( sin( pTheta ) );
    qvec.x += ( costheta + ( 1.0f - costheta ) * r.x * r.x ) * x;
    qvec.x += ( ( 1 - costheta ) * rvec.x * rvec.y - rvec.z * sintheta ) * y;
    qvec.x += ( ( 1 - costheta ) * rvec.x * rvec.z + rvec.y * sintheta ) * z;
    qvec.y += ( ( 1 - costheta ) * rvec.x * rvec.y + rvec.z * sintheta ) * x;
    qvec.y += ( costheta + ( 1 - costheta ) * rvec.y * rvec.y ) * y;
    qvec.y += ( ( 1 - costheta ) * rvec.y * rvec.z - rvec.x * sintheta ) * z;
    qvec.z += ( ( 1 - costheta ) * rvec.x * rvec.z - rvec.y * sintheta ) * x;
    qvec.z += ( ( 1 - costheta ) * rvec.y * rvec.z + rvec.x * sintheta ) * y;
    qvec.z += ( costheta + ( 1 - costheta ) * rvec.z * rvec.z ) * z;
    
	x = qvec.x;
	y = qvec.y;
	z = qvec.z;
}

}

}

#endif