/**	\file iso_math_vector4.h
 *	\class iso::math::Vector4 4D vector class
 *	\brief 4D vector class
 *
 *
 *  Created by Daniel Bisig on 7/16/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
#ifndef _iso_math_vector4_h_
#define _iso_math_vector4_h_

#include "iso_math_vector.h"
#include "iso_math_vector3.h"

namespace iso
{

namespace math
{

template< class Type >
class Vector4 : public Vector<Type>
{
public:
 	/**
	\brief zero vector
	*/
	static const Vector4<Type> ZERO;

	/**
	\brief one vector
	*/
	static const Vector4<Type> ONE;

	/**
	\brief references to vector components (homogenous coordinates)
	*/
    Type &x, &y, &z, &w;
	
	/**
	\brief references to vector components (rgba color components)
	*/
	Type &r, &g, &b, &a;

	/**
	\brief create zero vector
	*/
    Vector4();

	/**
	\brief create vector with identical vector components
	\param pValue value
	*/
    Vector4( Type pValue );
	
	/**
	\brief create vector from parameters
	\param p0 0. value
	\param p1 1. value
	\param p2 2. value
	\param p3 3. value
	*/
	Vector4( Type p0, Type p1, Type p2, Type p3 );

	/**
	\brief create vector from array of values
	\param pValues array of values
	\remarks pValues array must be of correct size
	*/
	Vector4( Type* pValues );

	/**
	\brief create vector from vector
	\param pVector vector of size 4
	*/
	Vector4( const Vector4<Type>& pVector );

	/**
	\brief create vector from vector
	\param pVector vector of size 4
	*/
	template< class Type2> Vector4( const Vector4<Type2>& pVector );

	/**
	\brief create vector from vector and value
	\param pVector vector of size 3
	\param p3 last vector component
	*/
	Vector4( const Vector3<Type>& pVector, Type p3 );

	/**
	\brief create vector from vector and value
	\param pVector vector of size 3
	\param p3 last vector component
	*/
	template< class Type2> Vector4( const Vector3<Type2>& pVector, Type2 p3 );

	/**
	\brief create vector from vector
	\param pVector vector
	\exception MathException pVector dimension not 4
	*/
	Vector4( const Vector<Type>& pVector ) throw (MathException);

	/**
	\brief create vector from vector
	\param pVector vector
	\exception MathException pVector dimension not 4
	*/
	template< class Type2> Vector4( const Vector<Type2>& pVector ) throw (MathException);

	/**
	\brief destructor
	*/
    virtual ~Vector4();

	/**
	\brief set dimension of vector
	\param pDim new vector dimension
	\exception MathException illegal function
	*/
	virtual void setDim(unsigned int pDim) throw (MathException);

	/**
	\brief set vector components from parameters
	\param p0 0. value
	\param p1 1. value
	\param p2 2. value
	\param p3 3. value
	*/		
	void set( Type p0, Type p1, Type p2, Type p3 );

	/**
	\brief set vector components from vector and value
	\param pVector vector of size 3
	\param p3 last vector component
	*/		
	void set( const Vector3<Type>& pVector, Type p3 );

	/**
	\brief set vector components from vector
	\param pVector of size 4
	*/		
	const Vector4<Type>& operator=( const Vector4<Type>& pVector );

	/**
	\brief set vector components from vector
	\param pVector
	\exception MathException pVector dimension not 4
	*/
	const Vector4<Type>& operator=( const Vector<Type>& pVector ) throw (MathException); 

	/**
	\brief print vector information
	\param pOstream output stream
	\param pVector vector
	\return output stream	
	*/	
	friend std::ostream& operator << ( std::ostream& pOstream, const Vector4<Type>& pVector)
    {
		pOstream << pVector.info().toStdString();
		return pOstream;
    }; 
};

typedef Vector4<int>    Vector4I;
typedef Vector4<float>  Vector4F;
typedef Vector4<double> Vector4D;

 template<class Type>
 const Vector4<Type> 
 Vector4<Type>::ZERO(0.0, 0.0, 0.0, 0.0);

 template<class Type>
 const Vector4<Type> 
 Vector4<Type>::ONE(1.0, 1.0, 1.0, 1.0);

template< class Type > 
Vector4<Type>::Vector4()
	: Vector<Type>( 4 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{}

template< class Type > 
Vector4<Type>::Vector4( Type pValue )
	: Vector<Type>( 4, pValue )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{}

template< class Type > 
Vector4<Type>::Vector4( Type p0, Type p1, Type p2, Type p3 )
	: Vector<Type>( 4 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{
	x = p0;
	y = p1;
	z = p2;
	w = p3;
}

template< class Type > 
Vector4<Type>::Vector4( Type* pValues )
	: Vector<Type>( 4, pValues )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{}

template< class Type > 
Vector4<Type>::Vector4( const Vector4<Type>& pVector )
	: Vector<Type>( pVector )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{}

template< class Type >
template< class Type2 > 
Vector4<Type>::Vector4( const Vector4<Type2>& pVector )
	: Vector<Type>( pVector )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{}

template< class Type > 
Vector4<Type>::Vector4( const Vector3<Type>& pVector, Type p3 )
	: Vector<Type>( 4 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{
	x = pVector.x;
	y = pVector.y;
	z = pVector.z;
	w = p3;	
}

template< class Type > 
template< class Type2 > 
Vector4<Type>::Vector4( const Vector3<Type2>& pVector, Type2 p3 )
	: Vector<Type>( 4 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{
	x = static_cast<Type>( pVector.x );
	y = static_cast<Type>( pVector.y );
	z = static_cast<Type>( pVector.z );
	w = static_cast<Type>( p3 );	
}

template< class Type > 
Vector4<Type>::Vector4( const Vector<Type>& pVector ) throw (MathException)
	: Vector<Type>( 4 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
	
	x = pVector.c[0];
	y = pVector.c[1];
	z = pVector.c[2];
	w = pVector.c[3];
}

template< class Type > 
template< class Type2 > 
Vector4<Type>::Vector4( const Vector<Type2>& pVector ) throw (MathException)
	: Vector<Type>( 4 )
	, x( Vector<Type>::c[0] )
	, y( Vector<Type>::c[1] )
	, z( Vector<Type>::c[2] )
	, w( Vector<Type>::c[3] )
	, r( Vector<Type>::c[0] )
	, g( Vector<Type>::c[1] )
	, b( Vector<Type>::c[2] )
	, a( Vector<Type>::c[3] )
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif
		
	x = static_cast<Type>( pVector.c[0] );
	y = static_cast<Type>( pVector.c[1] );
	z = static_cast<Type>( pVector.c[2] );
	w = static_cast<Type>( pVector.c[3] );
}

template< class Type > 
Vector4<Type>::~Vector4()
{}

template < class Type > 	
void 
Vector4< Type >::setDim(unsigned int pDim) throw (MathException)
{
	#ifdef _math_debug_
	throw MathException( "ERR_VECTOR_ERROR", base::String("illegal vector function"), __FILE__, __FUNCTION__, __LINE__ );
	#endif
}

template< class Type > 
void 
Vector4<Type>::set( Type p0, Type p1, Type p2, Type p3 )
{
	x = p0;
	y = p1;
	z = p2;
	w = p3;
}

template< class Type > 
void 
Vector4<Type>::set( const Vector3<Type>& pVector, Type p3 )
{
	x = pVector.x;
	y = pVector.y;
	z = pVector.z;
	w = p3;	
}

template< class Type > 
const Vector4<Type>& 
Vector4<Type>::operator=( const Vector4<Type>& pVector )
{
	x = pVector.x;
	y = pVector.y;
	z = pVector.z;
	w = pVector.w;
	
	return (*this);
}

template< class Type > 
const Vector4<Type>& 
Vector4<Type>::operator=( const Vector<Type>& pVector ) throw (MathException)
{
	#ifdef _math_debug_
	if(Vector<Type>::mDim != pVector.dim()) throw MathException( "ERR_DIMENSION_MISMATCH", base::String("dimension mismatch, dimension expected %1 dimension supplied %2").arg( Vector<Type>::mDim ).arg( pVector.dim() ), __FILE__, __FUNCTION__, __LINE__ );
	#endif

	x = pVector.c[0];
	y = pVector.c[1];
	z = pVector.c[2];
	w = pVector.c[3];
	
	return (*this);
}

}

}

#endif