/**	\file iso_serialize_exception.h
 *  \class iso::serialize::SerializeException class for handling all serialization related run time errors
 *	\brief class for handling all serialization related run time errors
 *
 *	Defines a variety of run time errors, which are specific to the config classes:
 *  ERR_NOT_SERIALIZABLE : Object Cannot Be Serialized
 *  ERR_SERIALIZE_UNKNOWN_PARAMETER : Failed to Retreive Parameter, Unknown Serialization Parameter
 *  ERR_SERIALIZE_DUPLICATE_PARAMETER : Failed to Store Parameter, Duplicate Serialization Parameter
 *  ERR_SERIALIZE_UNKNOWN_CLASSNAME : Unknown ClassName for Serializable Object
 *  ERR_SERIALIZE_UNKNOWN_NAME : Unknown Name for Serializable Object
 *  ERR_SERIALIZE_DUPLICATE_CLASSNAME : Duplicate ClassName for Serializable Object
 *  ERR_SERIALIZE_FILESUFFIX : Failed to Store File Path, Duplicate File Suffix
 *  ERR_SERIALIZE_PARSE_FILE : Error Parsing Serialization File
 *  ERR_SERIALIZE : Serialization Error
 *	ERR_DESERIALIZE : Deserialization Error
 *
 *  Created by Daniel Bisig on 5/17/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_serialize_exception_h_
#define _iso_serialize_exception_h_

#include "iso_serialize_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace serialize
{
	
class SerializeException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	SerializeException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	SerializeException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief destructor
	*/
	~SerializeException();
		
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const SerializeException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
			
protected:
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();
	
	/**
	\brief default constructur
	*/
	SerializeException();
};

};

};


#endif


