/**	\file iso_serialize_manager.h
 *  \class iso::serialize::SerializeManager handles serialization
 *	\brief handles serialization
 *
 *  read and writes serialization files
 *  it parses the file suffixes and then delegates everything it does to an appropriate specialized config manager 
 *  config files are distinguished purely based on their suffix
 *  in addition, is stores all kinds of intermediate values that help in object serialization:
 *  - unit name / pointers : required for config based port creation
 *  - unit class-name / pointer : required for unit class-name based unit creation
 *  - filepath: path for serialization files
 *
 *  Created by Daniel Bisig on 8/19/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */
 
 #ifndef _iso_serialize_manager_h_ 
 #define _iso_serialize_manager_h_
 
 #include "iso_serialize_types.h"
 #include "iso_serialize_exception.h"
 #include "iso_serialize_serializable.h"
 #include <iso_xml/iso_xml_tools.h>
 #include <QHash>
 
 namespace iso
 {
 
 namespace serialize
 {
 
 typedef Serializable* (*CreateSerializablePtr)(const SerializeData&);
 
 class SerializeParser;
 
 class SerializeManager : public Serializable
 {
 public:
	/**
	\brief get singleton instance
	\return singleton instance
	*/
	static SerializeManager& get();

	/**
	\brief get singleton instance
	\param pSerializeData serialization data
	\return singleton instance
	\exception SerializeException unable to create serialization manager
	*/
	static SerializeManager& get(const SerializeData& pSerializeData) throw (SerializeException);
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy();

	/**
	\brief default name
	*/
	static const base::String sClassName;

	/**
	\brief create serializeData for serialize manager
	\return serializeData
	*/
	virtual SerializeData serialize(const base::String& pSerialName = sClassName) const;

	/**
	\brief create serialization manager from SerializeData
	\param pSerializeData SerializeData
	\return Serializable
	\exception SerializeException incompatible SerializeData
	*/
	static Serializable* deSerialize(const SerializeData& pSerializeData) throw (SerializeException);
	
	/**
	\brief retrieve pointer to serializable object via name
	\param pName name of serializable object
	\return serializable
	\exception SerializeException unknown name
	*/
	Serializable* getSerializable(const base::String& pName) throw (SerializeException);
	
	/**
	\brief create new serializable from serialize data
	\param pName hash key for serializable (typically the class name)
	\param pSerializeData serializeData
	\return serializable
	\exception SerializeException unknown object class name
	*/
	Serializable* createSerializable(const base::String& pName, const SerializeData& pSerializeData) throw (SerializeException);

	/**
	\brief store pointer to serializable object in hash table
	\param pName serializable object name
	\param pSerializable serializable pointer
	\exception SerializeException serializable cannot be stored (duplicate class name)
	
	stores pointer to a serializable object which might need to be retrieved by other serializable objects during the deSerialization process
	*/
	void storeSerializable(const base::String& pName, Serializable* pSerializable) throw (SerializeException);

	/**
	\brief store deSerialization function pointer in hash table
	\param pName hash key for serializable (typically the class name)
	\exception SerializeException serializable cannot be stored (duplicate class name)
	
	this function stores a pointer to the deSerialize function of a serializable class. 
	this function pointer will be dereferenced when a serializable object needs to be created via getSerializable function
	*/
	template <class Type> base::String registerSerializable(const base::String& pName) throw (SerializeException);		

	/**
	\brief set file path for serialization files
	\param pFileSuffix file suffix
	\param pFilePath file path
	\exception SerializeException duplicate file suffix
	*/
	void setFilePath(const base::String& pFileSuffix, const base::String& pFilePath)  throw (SerializeException);
	
	/**
	\brief read serialization file
	\param pFile serialization file name
	\return SerializeData
	\exception SerializeException read error
	*/
	SerializeData read(const base::String& pFile) throw (SerializeException);
	
	/**
	\brief write serialization file
	\param pSerializeData serialization data
	\param pFileName serialization file name
	\param pFileType serialization file type
	\exception SerializeException write error
	*/
	void write(const SerializeData& pSerializeData, const base::String& pFileName, const base::String& pFileType) throw (SerializeException);

	/**
	\brief obtain textual serialize manager information
	\return String containing serialize manager information
	*/
	base::String info() const;

	/**
	\brief print serialize manager information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const SerializeManager& pSerializeManager)
	{
		pOstream << pSerializeManager.info().toStdString(); 
		
		return pOstream;
	};
 
 protected:
	/**
	\brief default constructor
	*/
	SerializeManager();

	/**
	\brief serialization based constructor
	\param pSerializeData serialization data
	\exception SerializeException unable to create serialization manager
	*/
	SerializeManager(const SerializeData& pSerializeData) throw (SerializeException);
	
	/**
	\brief destructor
	*/
	~SerializeManager();

	/**
	\brief construct file name
	\param pFileName original file name
	\return new file name
	
	if the filename does not contain path information, it tries to construct path information based on the file name's suffix.
	*/
	base::String createFileName(const base::String& pFileName) const;

	/**
	\brief singleton class instance
	*/
	static SerializeManager* sSerializeManager;
	
	/**
	\brief association : serializable class name / deSerialize function pointer
	*/
	QHash< base::String, CreateSerializablePtr > mSerializableFactory;
		
	/**
	\brief asssociation : serializable object name / serializable object pointer
	
	the serializables stored in this hashtable are objects, whose memory position needs to be know by other objects. 
	These other objects can retreive these pointers by providing the names of the other objects. 
	*/
	QHash<base::String, Serializable*> mSerializableStorage;
	
	/**
	\brief XML <-> SerializeData converter
	*/
	SerializeParser& mSerializeParser;

 
	/**
	\brief file suffix / file path association
	*/
	QHash<base::String, base::String> mFilePaths;
 };

template <class Type> 
base::String 
SerializeManager::registerSerializable(const base::String& pName) throw (SerializeException)
{
	Type* serializable;

	CreateSerializablePtr funPtr = &Type::deSerialize;	
	mSerializableFactory[pName] = funPtr;

	return pName;
}

};
 
};
 
 #endif