/** \file iso_space_neighbor_relation.h
 *
 *  Created by Daniel Bisig on 4/6/07.
 */
 
#ifndef _iso_space_neighbor_relation_h_
#define _iso_space_neighbor_relation_h_

#include "iso_space_types.h"
#include <iso_math/iso_math_vector.h>

namespace iso
{

namespace space
{

class SpaceObject;

class SpaceNeighborRelation
{
public:
	/**
	\brief class name
	*/
	static const base::String sClassName;

	/**
	\brief create space neighbor relation
	\param pDim dimension of space neighbor (> 0)
	*/
	SpaceNeighborRelation(unsigned int pDim);

	/**
	\brief create space neighbor relation (automated distance and direction calculation)
	\param pObject space object
	\param pNeighborObject neighboring space object
	\remarks fails if either the two objects are identical are if they differ in their respective dimensions
	*/
	SpaceNeighborRelation(SpaceObject* pObject, SpaceObject* pNeighborObject);

	/**
	\brief create space neighbor relation (manually set distance and direction)
	\param pObject space object
	\param pNeighborObject neighboring space object
	\param pDistance distance
	\param pDirection direction
	\remarks fails if either the two objects are identical are if they differ in their respective dimensions
	*/
	SpaceNeighborRelation(SpaceObject* pObject, SpaceObject* pNeighborObject, real pDistance, const math::Vector<real>& pDirection);

	/**
	\brief create space neighbor relation (manually set distance and direction)
	\param pObject space object
	\param pNeighborObject neighboring space object
	\param pValue value
	\param pDirection direction
	\param pDistance distance
	\remarks fails if either the two objects are identical are if they differ in their respective dimensions
	*/
	SpaceNeighborRelation(SpaceObject* pObject, SpaceObject* pNeighborObject, const math::Vector<real>& pValue, const math::Vector<real>& pDirection, real pDistance);
	
	/**
	\brief destructor
	*/
	~SpaceNeighborRelation();
	
	/**
	\brief return neighbor
	*/
	SpaceObject* neighbor();

	/**
	\brief return value of neighbor relation
	\returns value of neighbor relation
	*/
	const math::Vector<real>& value() const;
	
	/**
	\brief return direction vector pointing to neighbor
	\returns direction vector pointing to neighbor
	*/
	const math::Vector<real>& direction() const;
	
	/**
	\brief return distance to neighbor
	\returns distance to neighbor
	*/
	real distance() const;

	/**
	\brief manually set neighbor space object
	\param pNeighborObject neighbor space object
	\remarks fails if space object dimension is wrong
	*/
	void setNeighbor(SpaceObject* pNeighborObject);

	/**
	\brief manually set value
	\param pValue value
	\remarks fails if value dimension is wrong
	*/
	void setValue(const math::Vector<real> pValue);

	/**
	\brief manually set direction
	\param pDirection direction
	\remarks fails if direction dimension is wrong
	*/
	void setDirection(const math::Vector<real> pDirection);

	/**
	\brief manually set distance
	\param pDistance distance
	*/
	void setDistance(real pDistance);

	/**
	\brief set new space object and neighbor object
	\param pObject space object
	\param pNeighborObject neighboring space object
	\remarks fails if either the two objects are identical are if they differ in their respective dimensions
	*/
	void set(SpaceObject* pObject, SpaceObject* pNeighborObject);

 	/**
	\brief retrieve textual neighbor info
	\param pOstream output text stream
	\param pNeighbor neighbor
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceNeighborRelation& pNeighbor )
    {
		pOstream << pNeighbor.info().toStdString(); 
					
        return pOstream;
    }; 
 
	/**
	\brief print neighbor information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;

protected:
	/**
	\brief default constructor
	*/
	SpaceNeighborRelation();

	/**
	\brief space object
	*/
	SpaceObject* mObject;
	
	/**
	\brief neighboring space object
	*/
	SpaceObject* mNeighborObject;

	/**
	\brief value of neighbor relation
	
	usually the same value as mDirection
	*/
	math::Vector<real> mValue;

	/**
	\brief direction of neighbor relation
	
	usually vector pointing from position of mObject to position of mNeighborObject
	*/
	math::Vector<real> mDirection;
	
	/**
	\brief distance of neighbor relation
	
	usually length of mDirection
	*/
	real mDistance;
};

};

};

#endif