/**	\file iso_space_ntree.h
 *	\class iso::space::NTree n-dimensional space segmentation tree for space objects
 *	\brief n-dimensional space segmentation tree for space objects
 *
 *
 *  Created by Daniel Bisig on 3/29/07.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

 #ifndef _iso_space_ntree_h_
 #define _iso_space_ntree_h_

 #include "iso_space_types.h"
 #include "iso_space_ntree_node_pool.h"
 #include <iso_math/iso_math_vector.h>

 namespace iso
 {
 
 namespace space
 {
 
 class NTreeNode;
 
 class NTree
 {
 
 friend class NTreeVisitor;
 
 public:
	/**
	\brief create ntree
	\param pMinPos minimum position in space
	\param pMaxPos maximum position in space
	*/
	NTree(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);
	
	/**
	\brief destructor
	*/
	~NTree();
	
	/**
	\brief change space size
	\param pMinPos minimum position in space
	\param pMaxPos maximum position in space
	\remarks fails if dimension of new minimum and maximum position don't match tree dimension
	
	doing this requires that the tree must be cleared and rebuilt from scratch.\n
	this is fairly dangerous because there is no mechanism that ensures that this happens, the user must take care if it\n
	*/
	void resize(const math::Vector<real>& pMinPos, const math::Vector<real>& pMaxPos);

	/**
	\brief return root node
	\return root node
	*/
	NTreeNode* rootNode();
	
	/**
	\brief obtain textual ntree information
	\return String containing textual ntree information
	*/
	base::String info() const;

	/**
	\brief retrieve textual ntree information
	\param pOstream output stream
	\param pTree ntree
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const NTree& pTree)
	{
		pOstream << pTree.info().toStdString(); 
		
		return pOstream;
	}
 
 protected:
	/**
	/brief default constructor
	*/
	NTree();
	
	/**
	\brief maximum ntree depth
	
	> 0 : enforce depth limit
	< 0 : no depth limit 
	*/
	int mMaxDepth;
	
	/**
	\brief minimum space object count
	
	minimum number of space objects in a node for recursion to continue creating node children
	
	> 0 : enforce space object count limit
	< 0 : no space object count limit
	*/
	int mMinObjectCount;

	/**
	\brief ntree minimum space position
	*/
	math::Vector<real> mMinPos;

	/**
	\brief ntree maximum space position
	*/
	math::Vector<real> mMaxPos;
	
	/**
	\brief root node
	*/
	NTreeNode* mRootNode;
 };
 
 };
 
 };
 
 #endif