/**	\file iso_space_ntree_node.h
 *	\class iso::space::NTreeNode parameter space tree node
 *	\brief parameter space tree node
 *
 *
 *  Created by Daniel Bisig on 3/29/07.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

 #ifndef _iso_space_ntree_node_h_
 #define _iso_space_ntree_node_h_
 
 #include "iso_space_types.h"
 #include "iso_space_proxy_object.h"
 #include <iso_math/iso_math_vector.h>
 #include <QVector>
 
 namespace iso
 {
 
 namespace space
 {
 
 class NTreeNode
 {
 friend class NTreeVisitor;
 
 public:
	/**
	\brief create ntree node
	\param pDimension dimension of node
	*/
	NTreeNode(unsigned int pDimension);
	
	/**
	\brief destructor
	*/
	~NTreeNode();
	
	/**
	\brief return dimension
	*/
	unsigned int dim() const;
	
	/**
	\brief return number of children nodes
	*/
	unsigned int childrenCount() const;
	
	/**
	\brief return node level within ntree
	*/
	unsigned int level() const;
	
	/**
	\brief return number of parameters stored in node
	*/
	unsigned int objectCount() const;
	
	/**
	\brief clear node
	
	removes all node children and parameters
	*/
	void clear();
	
	/**
	\brief obtain textual node information
	\return String containing node information
	*/
	base::String info() const;

	/**
	\brief retrieve textual node pool information
	\param pOstream output stream
	\param pNode ntree node
	*/
	friend std::ostream& operator<< (std::ostream & pOstream, const NTreeNode& pNode)
	{
		pOstream << pNode.info().toStdString(); 
		
		return pOstream;
	}

 protected:
	/**
	\brief default constructor
	*/
	NTreeNode();
 
	/**
	\brief parent node
	*/
 	NTreeNode* mParent;
	
	/**
	\brief array of children nodes
	*/
	NTreeNode** mChildren;
	
	/**
	\brief number of children nodes
	*/
	unsigned int mChildrenCount;
	
	/**
	\brief stored parameters
	*/
	QVector<SpaceProxyObject*> mObjects;
	
	/**
	\brief last checked parameter
	
	for internal use only\n
	required when building space opject neighbor lists
	*/
	SpaceProxyObject* mLastCheckedObject;
	
	/**
	\brief node level within ntree
	*/
	unsigned int mLevel;
	
	/**
	\brief minimum corner of node hypercube
	*/
	math::Vector<real> mMinPos;
	
	/**
	\brief maximum corner of node hypercube
	*/
	math::Vector<real> mMaxPos;
 };
 
 };
 
 };
 
 #endif