/** \file iso_space_objects_analyze_manager.h
 *  \class iso::space::SpaceObjectsAnalyzeManager space objects analyze manager
 *  \brief space objects analyze manager
 *
 *  Created by Daniel Bisig on 6/4/08.
 */

#ifndef _iso_space_objects_analyze_manager_h_
#define _iso_space_objects_analyze_manager_h_

#include "iso_space_objects_analyzer.h"
#include <iso_data/iso_data_indexhash.h>

namespace iso
{

namespace space
{

class SpaceObjectsAnalyzeManager
{
public:
	static const base::String sClassName;
 
	/**
	\brief get singleton instance
	\return space object analyze manager
	*/
	static SpaceObjectsAnalyzeManager& get();
	
	/**
	\brief destroy singleton instance
	*/
	static void destroy(); 
 
	/**
	\brief return space objects analyzer
	\param pAnalyzerName space objects analyzer name
	\remarks fails if space object analyzer doesn't exist
	*/
	SpaceObjectsAnalyzer& analyzer(const base::String& pAnalyzerName);
 
 	/**
	\brief check if space objects analyzer exists
	\param pAnalyzerName space objects analyzer name
	\return true if space objects analyzer exists, false otherwise
	*/
	bool checkAnalyzer(const base::String& pAnalyzerName);
	
	/**
	\brief add space objects analyzer to space objects analyzer manager
	\param pAnalyzer space objects analyzer
	\remarks fails if space objects analyzer already exists	
	*/
	void addAnalyzer(SpaceObjectsAnalyzer* pAnalyzer);
 	
	/**
	\brief remove space objects analyzer
	\param pAnalyzerName name of space objects analyzer
	\remarks fails if space objects analyzer does not exist
	*/
	void removeAnalyzer(const base::String& pAnalyzerName);
 
	/**
	\brief add space object group
	\param pAnalyzerName name of space objects analyzer
	\param pGroupName space object group name
	\remarks fails if space objects analyzer does not exist
	*/
	void addObjectGroup( const base::String& pAnalyzerName, const base::String& pGroupName );
	
	/**
	\brief remove space object group
	\param pAnalyzerName name of space objects analyzer
	\param pGroupIndex space object group index
	\remarks fails if space objects analyzer does not exist
	*/
	void removeObjectGroup( const base::String& pAnalyzerName, unsigned int pGroupIndex );

	/**
	\brief remove space object group
	\param pAnalyzerName name of space objects analyzer
	\param pGroupName space object group name
	\remarks fails if space object analyzer does not exist
	*/
	void removeObjectGroup( const base::String& pAnalyzerName, const base::String& pGroupName );
	
	/**
	\brief add space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupIndex space object group index
	\param pSpaceObject space object
	\remarks fails if space objects analyzer does not exist
	*/
	void addObject( const base::String& pAnalyzerName, unsigned int pGroupIndex, const SpaceObject* pSpaceObject );

	/**
	\brief add space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupName space object group name
	\param pSpaceObject space object
	\remarks fails if space objects analyzer does not exist
	*/
	void addObject( const base::String& pAnalyzerName, const base::String& pGroupName, const SpaceObject* pSpaceObject );
	
	/**
	\brief add space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupIndex space object group index
	\param pNeighborGroup neighbor group
	\remarks fails if space objects analyzer does not exist
	*/
	void addObject( const base::String& pAnalyzerName, unsigned int pGroupIndex, const NeighborGroup* pNeighborGroup );

	/**
	\brief add space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupName space object group name
	\param pNeighborGroup neighbor group
	\remarks fails if space objects analyzer does not exist
	*/
	void addObject( const base::String& pAnalyzerName, const base::String& pGroupName, const NeighborGroup* pNeighborGroup );

	/**
	\brief remove space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupIndex space object group index
	\param pSpaceObject space object
	\remarks fails if space objects analyzer does not exist
	*/
	void removeObject( const base::String& pAnalyzerName, unsigned int pGroupIndex, const SpaceObject* pSpaceObject );

	/**
	\brief remove space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupName space object group name
	\param pSpaceObject space object
	\remarks fails if space objects analyzer does not exist
	*/
	void removeObject( const base::String& pAnalyzerName, const base::String& pGroupName, const SpaceObject* pSpaceObject );

	/**
	\brief remove space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupIndex space object group index
	\param pNeighborGroup neighbor group
	\remarks fails if space objects analyzer does not exist
	*/
	void removeObject( const base::String& pAnalyzerName, unsigned int pGroupIndex, const NeighborGroup* pNeighborGroup );

	/**
	\brief remove space object
	\param pAnalyzerName name of space objects analyzer
	\param pGroupName space object group name
	\param pNeighborGroup neighbor group
	\remarks fails if space objects analyzer does not exist
	*/
	void removeObject( const base::String& pAnalyzerName, const base::String& pGroupName, const NeighborGroup* pNeighborGroup );
	
	/**
	\brief remove space objects
	\param pAnalyzerName name of space objects analyzer
	\param pGroupIndex space object group index
	\remarks fails if space objects analyzer does not exist
	*/
	void removeObjects( const base::String& pAnalyzerName, unsigned int pGroupIndex );

	/**
	\brief remove space objects
	\param pAnalyzerName name of space objects analyzer
	\param pGroupName space data group name
	\remarks fails if space objects analyzer does not exist
	*/
	void removeObjects( const base::String& pAnalyzerName, const base::String& pGroupName );

	/**
	\brief return analysis result
	\param pAnalyzerName name of space objects analyzer
	\param pResultIndex result index
	\return result as vector
	\remarks fails if space objects analyzer does not exist
	*/
	const math::Vector<real>& result( const base::String& pAnalyzerName, unsigned int pResultIndex ) const;

	/**
	\brief return analysis result
	\param pAnalyzerName name of space objects analyzer
	\param pResultName result name
	\return result as vector
	\remarks fails if space objects analyzer does not exist
	*/
	const math::Vector<real>& result( const base::String& pAnalyzerName, const base::String& pResultName ) const;
	
	/**
	\brief return analysis results
	\param pAnalyzerName name of space objects analyzer
	\return IndexHash of all vector results
	\remarks fails if space objects analyzer does not exist
	*/
	const data::IndexHash< base::String, math::Vector<real> >& results(  const base::String& pAnalyzerName ) const;

	/**
	\brief update all space objects analyzers
	*/	
	void update();
	
	/**
	\brief update all space objects analyzers
	*/
	void analyze();
 
  	/**
	\brief retrieve textual space objects analyzer manager info
	\param pOstream output text stream
	\param pAnalyzeManager space objects analyze manager
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceObjectsAnalyzeManager& pAnalyzeManager )
    {
		pOstream << pAnalyzeManager.info().toStdString(); 
					
        return pOstream;
    }; 
 
	/**
	\brief print data analyzer manager information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;
 
 protected:
	/**
	\brief default constructor
	*/
	SpaceObjectsAnalyzeManager();
	
	/**
	\brief destructor
	*/
	~SpaceObjectsAnalyzeManager();
 
	/**
	\brief singleton instance
	*/
	static SpaceObjectsAnalyzeManager* sSpaceObjectsAnalyzeManager;
	
	/**
	\brief spaces
	*/
	data::IndexHash<base::String, SpaceObjectsAnalyzer> mAnalyzers;
};

};

};

#endif