/** \file iso_space_objects_analyzer.h
 *  \class iso::space::SpaceObjectsAnalyzer abstract base class for analyzing space object distributions
 *  \brief abstract base class for analyzing space object distributions
 *
 *  Created by Daniel Bisig on 5/30/08.
 */
 
#ifndef _iso_space_objects_analyzer_h_
#define _iso_space_objects_analyzer_h_

#include "iso_space_proxy_object.h"
#include <iso_data/iso_data_indexhash.h>
#include <QVector>
#include <QMutex>

namespace iso
{

namespace space
{

class SpaceProxyObjectGroup
{

public:
	SpaceProxyObjectGroup();
	~SpaceProxyObjectGroup();
	
	unsigned int objectCount() const;
	const SpaceProxyObject* proxyObject( unsigned int pIndex) const;
	const SpaceObject* spaceObject( unsigned int pIndex) const;
	const NeighborGroup* neighborGroup( unsigned int pIndex) const;
	void addObject(SpaceObject* pSpaceObject);
	void addObject(NeighborGroup* pNeighborGroup);
	void removeObject(SpaceObject* pSpaceObject);
	void removeObject(NeighborGroup* pNeighborGroup);
	void removeObjects();
	
protected:
	QVector< SpaceProxyObject* > mSpaceObjects;
	QMutex mLock;
};

class SpaceObjectsAnalyzer
{
public:
	/**
	\brief default constructor
	*/
	SpaceObjectsAnalyzer();

	/**
	\brief parameter based constructor
	\param pName of space data analyzer
	*/
	SpaceObjectsAnalyzer(base::String pName);
	
	/**
	\brief destructor
	*/
	virtual ~SpaceObjectsAnalyzer();

	/**
	\brief return name
	\return name
	*/
	const base::String& name() const;
	
	/**
	\brief check whether the analyzer is modifiable
	\return true if it is, false otherwise
	*/
	bool modifiable() const;

	/**
	\brief check whether object group exists
	\param pGroupName name of object group
	\return true if it does, false otherwise
	*/
	bool checkObjectGroup( const base::String& pGroupName ) const;
	
	/**
	\brief add space object group
	\param pGroupName name of object group
	*/
	virtual void addObjectGroup( const base::String& pGroupName );
	
	/**
	\brief remove space object group
	\param pGroupIndex index of object group
	*/
	virtual void removeObjectGroup( unsigned int pGroupIndex );

	/**
	\brief remove space object group
	\param pGroupName name of object group
	*/
	virtual void removeObjectGroup( const base::String& pGroupName );
	
	/**
	\brief add space object
	\param pGroupIndex space object group index
	\param pSpaceObject space object
	*/
	virtual void addObject( unsigned int pGroupIndex, const SpaceObject* pSpaceObject );
	
	/**
	\brief add space objects
	\param pGroupIndex space object group index
	\param pSpaceObjects vector of space objects
	*/
	virtual void addObjects( unsigned int pGroupIndex, QVector< const SpaceObject* >& pSpaceObjects );	

	/**
	\brief add space object
	\param pGroupName space object group name
	\param pSpaceObject space objects
	*/
	virtual void addObject( const base::String& pGroupName, const SpaceObject* pSpaceObject );

	/**
	\brief add space objects
	\param pGroupName space object group name
	\param pSpaceObjects vector of space objects
	*/
	virtual void addObjects( const base::String& pGroupName, QVector< const SpaceObject* >& pSpaceObjects );
	
	/**
	\brief add space object
	\param pDataGroupIndex space object group index
	\param pNeighborGroup neighbor group
	*/
	virtual void addObject( unsigned int pDataGroupIndex, const NeighborGroup* pNeighborGroup );

	/**
	\brief add space objects
	\param pDataGroupIndex space object group index
	\param pNeighborGroups vector of neighbor groups
	*/
	virtual void addObjects( unsigned int pDataGroupIndex, QVector< const NeighborGroup* >& pNeighborGroups );

	/**
	\brief add space object
	\param pGroupName space object group name
	\param pNeighborGroup neighbor group
	*/
	virtual void addObject( const base::String& pGroupName, const NeighborGroup* pNeighborGroup );

	/**
	\brief add space objects
	\param pGroupName space object group name
	\param pNeighborGroups vector of neighbor groups
	*/
	virtual void addObjects( const base::String& pGroupName, QVector< const NeighborGroup* >& pNeighborGroups );

	/**
	\brief remove space object
	\param pGroupIndex space object group index
	\param pSpaceObject space object
	*/
	void removeObject( unsigned int pGroupIndex, const SpaceObject* pSpaceObject );

	/**
	\brief remove space object
	\param pGroupName space object group name
	\param pSpaceObject space object
	*/
	void removeObject( const base::String& pGroupName, const SpaceObject* pSpaceObject );

	/**
	\brief remove space object
	\param pGroupIndex space object group index
	\param pNeighborGroup neighbor group
	*/
	void removeObject( unsigned int pGroupIndex, const NeighborGroup* pNeighborGroup );

	/**
	\brief remove space object
	\param pGroupName space object group name
	\param pNeighborGroup neighbor group
	*/
	void removeObject( const base::String& pGroupName, const NeighborGroup* pNeighborGroup );

	/**
	\brief remove space objects
	\param pDataGroupIndex space object group index
	*/
	void removeObjects( unsigned int pDataGroupIndex );

	/**
	\brief remove space objects
	\param pGroupName space object group name
	*/
	void removeObjects( const base::String& pGroupName );

	/**
	\brief return analysis result
	\param pResultIndex result index
	\return result as vector
	*/
	const math::Vector<real>& result( unsigned int pResultIndex ) const;

	/**
	\brief return analysis result
	\param pResultName result name
	\return result as vector
	*/
	const math::Vector<real>& result( const base::String& pResultName ) const;
	
	/**
	\brief return analysis results
	\return QVector of all vector results
	*/
	const data::IndexHash< base::String, math::Vector<real> >& results() const;
	
	/**
	\brief perform analysis
	*/
	virtual void analyze();

protected:
	data::IndexHash< base::String, SpaceProxyObjectGroup > mSpaceObjectGroups; ///\brief space data groups
	data::IndexHash< base::String, math::Vector<real> > mResults; ///\brief analysis results
	base::String mName; ///\brief name of analyzer
	
	bool mModifiable; ///\brief flag whether number of data groups can be modified or not
	QMutex mLock;
};

};

};

#endif