/** \file iso_space_shape.h
 *
 *  Created by Daniel Bisig on 5/10/07.
 */

#ifndef _iso_space_shape_h_
#define _iso_space_shape_h_

#include "iso_space_object.h"
#include <iso_math/iso_math_matrixh.h>
#include <iso_math/iso_math_rectangle.h>
#include <iso_geom/iso_geom_geometry.h>
#include <iostream>

namespace iso
{

namespace space
{

class SpaceShape : public SpaceObject
{
public:
	/**
	\brief create shape
	\param pGeometry shape geometry
	*/
	SpaceShape(geom::Geometry* pGeometry);

	/**
	\brief create shape
	\param pGeometry shape geometry
	\param pDim shape dimension
	*/
	SpaceShape(geom::Geometry* pGeometry, unsigned int pDim);
	
	/**
	\brief destructor
	*/
	virtual ~SpaceShape();
	
	/**
	\brief return geometry
	\return geometry
	*/
	geom::Geometry& geometry();
	
	/**
	\brief set geometry
	\param pGeometry geometry
	\remark fails if new geometry has different dimension than previous geometry
	
	replaces current geometry with new geometry
	*/
	void setGeometry( geom::Geometry* pGeometry );
	
	/**
	\brief return translation matrix
	\return translation matrix
	*/
	const math::MatrixH<real>& translation() const;
	
	/**
	\brief return rotation matrix
	\return return rotation matrix
	*/
	const math::MatrixH<real>& orientation() const;

	/**
	\brief return scale matrix
	\return return scale matrix
	*/
	const math::MatrixH<real>& scale() const;

	/**
	\brief return transformation matrix
	\return transformation matrix
	*/
	const math::MatrixH<real>& matrix() const;

	/**
	\brief return object to world transformation matrix
	\return object to world transformation matrix
	*/
	const math::MatrixH<real>& o2wMatrix() const;

	/**
	\brief return world to object transformation matrix
	\return world to object transformation matrix
	*/
	const math::MatrixH<real>& w2oMatrix() const;
	
	/**
	\brief return axis aligned bounding box (in world coordinates)
	*/
	math::Rectangle<real> AABB() const;

	/**
	\brief return axis aligned bounding box (in object coordinates)
	*/
	math::Rectangle<real> ocAABB() const;

	/**
	\brief set shape object position
	\param pPosition shape object position
	\remarks fails if new position has wrong dimension
	*/
	virtual void setPosition(const math::Vector<real>& pPosition);

	/**
	\brief set orientation
	\param pAngle angle of rotation (radians)
	\param pAxis axis of rotation
	\remarks fails if orientation vector has wrong dimension
	*/
	void setOrientation(real pAngle, const math::Vector<real>& pAxis);

	/**
	\brief set orientation
	\param pOrientation orientation as quaternion
	*/
	void setOrientation(const math::Quaternion<real>& pOrientation);

	/**
	\brief set scale
	\param pScale uniform scale value
	*/
	void setScale(real pScale);

	/**
	\brief set scale
	\param pScale scale vector
	\remarks fails if scale vector has wrong dimension
	*/
	void setScale(const math::Vector<real>& pScale);

	/**
	\brief change position
	\param pPosition shape object position
	\remarks fails if new position has wrong dimension
	*/
	virtual void changePosition(const math::Vector<real>& pPosition);

	/**
	\brief change orientation
	\param pAngle angle of rotation (radians)
	\param pAxis axis of rotation
	\remarks fails if orientation vector has wrong dimension
	*/
	void changeOrientation(real pAngle, const math::Vector<real>& pAxis);

	/**
	\brief change scale
	\param pScale uniform scale value
	*/
	void changeScale(real pScale);

	/**
	\brief change scale
	\param pScale scale vector
	\remarks fails if scale vector has wrong dimension
	*/
	void changeScale(const math::Vector<real>& pScale);
	
	/**
	\brief update axis aligned bounding box
	*/
	virtual void updateAABB();

	/**
	\brief return closest point
	\param pRefPoint reference point (in world coordinates)
	\param pResPoint result point (in world coordinates)
	\remarks fails if dimension of reference and result points don't match dimension of shape
	*/
	void closestPoint(const math::Vector<real>& pRefPoint, math::Vector<real>& pResPoint);
	
 	/**
	\brief obtain textual space shape information
	\param pPropagationLevel how far the propagation method proceeds through composite classes (-1: unlimited, 0: no proceeding, >0: limited proceeding)
	*/
	virtual base::String info(int pPropagationLevel = 0) const;
 
	/**
	\brief retrieve textual space shape info
	\param pOstream output text stream
	\param pShape space shape
	*/
 	friend std::ostream& operator << ( std::ostream& pOstream, const SpaceShape& pShape )
    {
		pOstream << pShape.info().toStdString(); 
					
        return pOstream;
    }; 

protected:
	/**
	\brief default constructor
	*/
	SpaceShape();
	
	/**
	\brief update homogenous matrix
	*/
	void updateMatrix();

	geom::Geometry* mGeometry; /// \brief shape geometry
	math::Rectangle<real> mAABB; /// \brief axis aligned bounding box
	
	math::MatrixH<real> mRotation; /// \brief matrix representing only rotation
	math::MatrixH<real> mScale; /// \brief matrix representing only scale
	math::MatrixH<real> mTranslation; /// \brief matrix representing only translation
	math::MatrixH<real> mInvTranslation; /// \brief inverse of the previous translation matrix
	
	math::MatrixH<real> mMatrix; /// \brief homogenous transformation matrix: object coordinates to world coordinates
	math::MatrixH<real> mInvMatrix; /// \brief homogenous transformation matrix: world coordinates to object coordinates
	math::MatrixH<real> mInvScaleMatrix; /// \brief temporary matrix for calculating the inverse matrix
};

};

};

#endif
