/**	\file iso_xml_exception.h
 *  \class iso::xml::XMLException class for handling all xml related run time errors
 *	\brief class for handling all xml related run time errors
 *
 *	Defines a variety of run time errors, which are specific to xml:
 *  ERR_XML_INITIALIZATION : XML Initialization Error
 *  ERR_XML_FINALIZATION : XML Finalization Error
 *  ERR_XML_PARSE = XML Parse Error;
 *  ERR_XML_DOM = XML DOM Error;
 *  ERR_XML = XML Error;
 *
 *  Created by Daniel Bisig on 10/24/06.
 *  Copyright 2006 Daniel Bisig All rights reserved.
 */

#ifndef _iso_xml_exception_h_
#define _iso_xml_exception_h_

#include "iso_xml_types.h"
#include <iso_base/iso_base_exception.h>

namespace iso
{

namespace xml
{
	
class XMLException : public base::Exception
{
public:
	/**
	\brief parameter based contstructor
	\param pType type of exception
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for specific exception types
	*/
	XMLException(base::String pType, base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief parameter based contstructor
	\param pMessage addition text describing exception
	\param pSourceFile source code file name in which exception occurred
	\param pFunction function name in which exception occurred
	\param pLineNumber line number within source file at which exception occurred
	
	Constructor for generic exception types
	*/
	XMLException(base::String pMessage, base::String pSourceFile, base::String pFunction, int pLineNumber);
	
	/**
	\brief destructor
	*/
	~XMLException();
		
	/**
	\brief print exception information
	*/
	friend std::ostream& operator<< (std::ostream& pOstream, const XMLException& pException)
	{		
		return pOstream << "type: " << pException.mType.toStdString() << "\nmessage: " << pException.mMessage.toStdString() << "\nsourcefile: " << pException.mSourceFile.toStdString() << "\nfunction: " << pException.mFunction.toStdString() << "\nlinenumber: " << pException.mLineNumber;
	};
			
protected:
	/**
	\brief reflects whether exception types have been initialized
	*/
	static bool sExceptionTypesInitialized;
	
	/**
	\brief initialize exception types
	*/
	static void initExceptionTypes();
	
	/**
	\brief default constructur
	*/
	XMLException();
};

};

};


#endif


